@@ -38,23 +38,20 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#if defined(__alpha__) && defined(__GNUC__) && ((__GNUC__ == 2) && (__GNUC_MINOR__ < 96))	/* gets rid of some misalignment in GCC */
-#pragma 2
-#endif
-
 #define EDOX_DEFAULT_W  512
 #define EDOX_DEFAULT_H  400
 
 Display            *disp;
 Root                VRoot;
 
-Window              win_main, win_title, win_exit, win_next, win_prev, win_text,
-   win_cover;
-Imlib_Image         im_text;
-Imlib_Image         im_title;
-Imlib_Image         im_prev1, im_prev2;
-Imlib_Image         im_next1, im_next2;
-Imlib_Image         im_exit1, im_exit2;
+static Window       win_main = None, win_text = None;
+static Window       win_title = None, win_exit = None;
+static Window       win_next = None, win_prev = None;
+
+static Imlib_Image  im_title = NULL;
+static Imlib_Image  im_prev1 = NULL, im_prev2 = NULL;
+static Imlib_Image  im_next1 = NULL, im_next2 = NULL;
+static Imlib_Image  im_exit1 = NULL, im_exit2 = NULL;
 
 static const char   doxdir[] = ENLIGHTENMENT_ROOT "/E-docs";
 char               *docdir = NULL;
@@ -62,6 +59,8 @@ char               *docdir = NULL;
 static Atom         ATOM_WM_DELETE_WINDOW = None;
 static Atom         ATOM_WM_PROTOCOLS = None;
 
+static char         show_top_bar = 1;
+
 static              Window
 FindRootWindow(Display * dpy)
 {
@@ -242,6 +241,15 @@ LoadFile(const char *file, const char *docfile)
    fclose(f);
 }
 
+static void
+usage(const char *prog)
+{
+   printf("usage:\n"
+	  "%s [-page page_number] [-file Edoc_fname] [-size width height] [Edoc_dir]\n",
+	  prog);
+   exit(1);
+}
+
 #define ApplyImage3(win, im) \
 	XClearWindow(disp, win)
 
@@ -294,17 +302,36 @@ main(int argc, char **argv)
 
    w = EDOX_DEFAULT_W;
    h = EDOX_DEFAULT_H;
-   x = 0;
-   y = 0;
    pagenum = 0;
 
-   if (argc < 1)
+   docfile = "MAIN";
+   for (i = 1; i < argc; i++)
      {
-	printf("usage:\n"
-	       "%s [-page page_number] [-file Edoc_fname] [-size width height] [Edoc_dir]\n",
-	       argv[0]);
-	exit(1);
+	if (!strcmp(argv[i], "-help"))
+	   usage(argv[0]);
+	else if ((!strcmp(argv[i], "-page")) && (i < (argc - 1)))
+	   pagenum = atoi(argv[++i]);
+	else if ((!strcmp(argv[i], "-file")) && (i < (argc - 1)))
+	   docfile = argv[++i];
+	else if ((!strcmp(argv[i], "-size")) && (i < (argc - 2)))
+	  {
+	     w = atoi(argv[++i]);
+	     h = atoi(argv[++i]);
+	  }
+	else
+	   docdir = strdup(argv[i]);
      }
+   if (docdir == NULL)
+      docdir = strdup(doxdir);
+   s = EMALLOC(char, strlen(docdir) + strlen(docfile) + 2 + 20);
+
+   if (strstr(docdir, "/ABOUT"))
+      show_top_bar = 0;
+
+   sprintf(s, "%s/%s", docdir, docfile);
+   findLocalizedFile(s);
+
+   LoadFile(s, docfile);
 
    if ((disp = XOpenDisplay(NULL)) == NULL)
      {
@@ -324,44 +351,8 @@ main(int argc, char **argv)
 
    VRootInit();
 
-   docfile = "MAIN";
-   for (i = 1; i < argc; i++)
-     {
-	if ((!strcmp(argv[i], "-page")) && (i < (argc - 1)))
-	   pagenum = atoi(argv[++i]);
-	else if ((!strcmp(argv[i], "-file")) && (i < (argc - 1)))
-	   docfile = argv[++i];
-	else if ((!strcmp(argv[i], "-size")) && (i < (argc - 2)))
-	  {
-	     w = atoi(argv[++i]);
-	     h = atoi(argv[++i]);
-	  }
-	else
-	   docdir = strdup(argv[i]);
-     }
-   if (docdir == NULL)
-      docdir = strdup(doxdir);
-   s = EMALLOC(char, strlen(docdir) + strlen(docfile) + 2 + 20);
+   t = (show_top_bar) ? 16 : 0;
 
-   sprintf(s, "%s/%s", docdir, docfile);
-   findLocalizedFile(s);
-
-   im_title = ImageLoadDox("title.png");
-   imlib_context_set_image(im_title);
-   ibd.left = 50;
-   ibd.right = 2;
-   ibd.top = 2;
-   ibd.bottom = 2;
-   imlib_image_set_border(&ibd);
-
-   im_prev1 = ImageLoadDox("prev1.png");
-   im_prev2 = ImageLoadDox("prev2.png");
-   im_next1 = ImageLoadDox("next1.png");
-   im_next2 = ImageLoadDox("next2.png");
-   im_exit1 = ImageLoadDox("exit1.png");
-   im_exit2 = ImageLoadDox("exit2.png");
-
-   t = 16;
    wx = (VRoot.w - w) / 2;
    wy = (VRoot.h - (h + t)) / 2;
 #ifdef USE_XINERAMA
@@ -394,41 +385,57 @@ main(int argc, char **argv)
      }
 #endif
    win_main = CreateWindow(VRoot.win, wx, wy, w, h + t);
-   win_title =
-      XCreateSimpleWindow(disp, win_main, 0, 0, (w - 64 - 64 - t), t, 0, 0, 0);
-   win_prev =
-      XCreateSimpleWindow(disp, win_main, (w - 64 - 64 - t), 0, 64, t, 0, 0, 0);
-   win_next =
-      XCreateSimpleWindow(disp, win_main, (w - 64 - 64 - t) + 64, 0, 64, t, 0,
-			  0, 0);
-   win_exit =
-      XCreateSimpleWindow(disp, win_main, (w - 64 - 64 - t) + 64 + 64, 0, t, t,
-			  0, 0, 0);
-   win_text = XCreateSimpleWindow(disp, win_main, 0, t, w, h, 0, 0, 0);
-
    XSelectInput(disp, win_main, KeyPressMask | KeyReleaseMask);
-   XSelectInput(disp, win_prev, ButtonPressMask | ButtonReleaseMask);
-   XSelectInput(disp, win_next, ButtonPressMask | ButtonReleaseMask);
-   XSelectInput(disp, win_exit, ButtonPressMask | ButtonReleaseMask);
+   win_text = XCreateSimpleWindow(disp, win_main, 0, t, w, h, 0, 0, 0);
    XSelectInput(disp, win_text, ButtonPressMask | ButtonReleaseMask |
 		PointerMotionMask);
 
    draw = XCreatePixmap(disp, win_text, w, h, VRoot.depth);
 
-   ApplyImage1(win_title, im_title);
-   ApplyImage1(win_prev, im_prev1);
-   ApplyImage1(win_next, im_next1);
-   ApplyImage1(win_exit, im_exit1);
-
-   LoadFile(s, docfile);
-
    l = RenderPage(draw, pagenum, w, h);
    UPDATE_NOW;
 
-   XMapWindow(disp, win_title);
-   XMapWindow(disp, win_prev);
-   XMapWindow(disp, win_next);
-   XMapWindow(disp, win_exit);
+   if (show_top_bar)
+     {
+	win_title = XCreateSimpleWindow(disp, win_main,
+					0, 0, (w - 64 - 64 - t), t, 0, 0, 0);
+	win_prev = XCreateSimpleWindow(disp, win_main,
+				       (w - 64 - 64 - t), 0, 64, t, 0, 0, 0);
+	win_next = XCreateSimpleWindow(disp, win_main,
+				       (w - 64 - 64 - t) + 64, 0, 64, t,
+				       0, 0, 0);
+	win_exit = XCreateSimpleWindow(disp, win_main,
+				       (w - 64 - 64 - t) + 64 + 64, 0, t, t,
+				       0, 0, 0);
+	XSelectInput(disp, win_prev, ButtonPressMask | ButtonReleaseMask);
+	XSelectInput(disp, win_next, ButtonPressMask | ButtonReleaseMask);
+	XSelectInput(disp, win_exit, ButtonPressMask | ButtonReleaseMask);
+
+	im_title = ImageLoadDox("title.png");
+	imlib_context_set_image(im_title);
+	ibd.left = 50;
+	ibd.right = 2;
+	ibd.top = 2;
+	ibd.bottom = 2;
+	imlib_image_set_border(&ibd);
+
+	im_prev1 = ImageLoadDox("prev1.png");
+	im_prev2 = ImageLoadDox("prev2.png");
+	im_next1 = ImageLoadDox("next1.png");
+	im_next2 = ImageLoadDox("next2.png");
+	im_exit1 = ImageLoadDox("exit1.png");
+	im_exit2 = ImageLoadDox("exit2.png");
+
+	ApplyImage1(win_title, im_title);
+	ApplyImage1(win_prev, im_prev1);
+	ApplyImage1(win_next, im_next1);
+	ApplyImage1(win_exit, im_exit1);
+
+	XMapWindow(disp, win_title);
+	XMapWindow(disp, win_prev);
+	XMapWindow(disp, win_next);
+	XMapWindow(disp, win_exit);
+     }
    XMapWindow(disp, win_text);
    XMapWindow(disp, win_main);
 
