@@ -3,7 +3,7 @@
 /*****************************************************************************/
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -127,6 +127,7 @@
 #define XEXT_FIXES       6
 #define XEXT_RENDER      7
 #define XEXT_GLX         8
+#define XEXT_XI          9
 
 #define XEXT_CM_ALL     16
 
@@ -388,6 +389,7 @@ typedef struct {
    struct {
       char               *paths;
       char               *path;
+      char               *variant;
    } theme;
    struct {
       char               *exec_name;	/* argv[0] */
@@ -413,7 +415,6 @@ typedef struct {
    } wm;
    int                 mode;
    int                 op_source;
-   char                action_inhibit;
    EWin               *focuswin;
    EWin               *mouse_over_ewin;
    EWin               *context_ewin;
@@ -441,6 +442,7 @@ void                About(void);
 int                 execApplication(const char *params, int flags);
 void                Espawn(int argc, char **argv);
 void                EspawnCmd(const char *cmd);
+int                 Esystem(const char *cmd);
 
 /* config.c */
 void                SkipTillEnd(FILE * ConfigFile);
