@@ -49,7 +49,7 @@ struct _background {
    char               *name;
    Pixmap              pmap;
    time_t              last_viewed;
-   EColor              bg_solid;
+   unsigned int        bg_solid;
    char                bg_tile;
    BgPart              bg;
    BgPart              top;
@@ -76,7 +76,7 @@ BackgroundGetUniqueString(const Background * bg)
    int                 r, g, b;
    int                 n1, n2, n3, n4, n5, f1, f2, f3, f4, f5, f6;
 
-   GET_COLOR(&(bg->bg_solid), r, g, b);
+   COLOR32_TO_RGB(bg->bg_solid, r, g, b);
    n1 = (r << 24) | (g << 16) | (b << 8) | (bg->bg_tile << 7)
       | (bg->bg.keep_aspect << 6) | (bg->top.keep_aspect << 5);
    n2 = (bg->bg.xjust << 16) | (bg->bg.yjust);
@@ -282,8 +282,8 @@ BackgroundDelete(Background * bg)
 #endif /* ENABLE_DIALOGS */
 
 static Background  *
-BackgroundCreate(const char *name, EColor * solid, const char *bgn, char tile,
-		 char keep_aspect, int xjust, int yjust, int xperc,
+BackgroundCreate(const char *name, unsigned int solid, const char *bgn,
+		 char tile, char keep_aspect, int xjust, int yjust, int xperc,
 		 int yperc, const char *top, char tkeep_aspect, int txjust,
 		 int tyjust, int txperc, int typerc)
 {
@@ -293,13 +293,15 @@ BackgroundCreate(const char *name, EColor * solid, const char *bgn, char tile,
    if (!bg)
       return NULL;
 
+   if (!name)
+     {
+	name = "NONE";
+	bg->external = 1;
+     }
    bg->name = Estrdup(name);
 
-   SET_COLOR(&(bg->bg_solid), 160, 160, 160);
-   if (solid)
-      bg->bg_solid = *solid;
-   else
-      bg->external = 1;
+   COLOR32_FROM_RGB(bg->bg_solid, 160, 160, 160);
+   bg->bg_solid = solid;
    if (bgn)
       bg->bg.file = Estrdup(bgn);
    bg->bg_tile = tile;
@@ -356,9 +358,7 @@ BackgroundCmp(Background * bg, Background * bgx)
    else if (bg->top.file || bgx->top.file)
       return 1;
 
-   if ((bg->bg_solid.red != bgx->bg_solid.red) ||
-       (bg->bg_solid.green != bgx->bg_solid.green) ||
-       (bg->bg_solid.blue != bgx->bg_solid.blue))
+   if (bg->bg_solid != bgx->bg_solid)
       return 1;
    if (bg->bg_tile != bgx->bg_tile)
       return 1;
@@ -400,20 +400,16 @@ BackgroundInvalidate(Background * bg, int refresh)
 }
 
 static int
-BackgroundModify(Background * bg, EColor * solid, const char *bgn, char tile,
-		 char keep_aspect, int xjust, int yjust, int xperc,
+BackgroundModify(Background * bg, unsigned int solid, const char *bgn,
+		 char tile, char keep_aspect, int xjust, int yjust, int xperc,
 		 int yperc, const char *top, char tkeep_aspect, int txjust,
 		 int tyjust, int txperc, int typerc)
 {
    int                 updated = 0;
 
-   if (solid->red != bg->bg_solid.red)
+   if (solid != bg->bg_solid)
       updated = 1;
-   if (solid->green != bg->bg_solid.green)
-      updated = 1;
-   if (solid->blue != bg->bg_solid.blue)
-      updated = 1;
-   bg->bg_solid = *solid;
+   bg->bg_solid = solid;
 
    if ((bg->bg.file) && (bgn))
      {
@@ -576,22 +572,24 @@ BackgroundRealize(Background * bg, Win win, Drawable draw, unsigned int rw,
 
    if (!hasbg && !hasfg)
      {
+	unsigned int        pixel;
+
 	/* Solid color only */
-	EAllocColor(WinGetCmap(VROOT), &bg->bg_solid);
+	pixel = EAllocColor(WinGetCmap(VROOT), bg->bg_solid);
 
 	if (!is_win)
 	  {
 	     gc = EXCreateGC(draw, 0, NULL);
 	     XSetClipMask(disp, gc, 0);
 	     XSetFillStyle(disp, gc, FillSolid);
-	     XSetForeground(disp, gc, bg->bg_solid.pixel);
+	     XSetForeground(disp, gc, pixel);
 	     XFillRectangle(disp, draw, gc, 0, 0, rw, rh);
 	     EXFreeGC(gc);
 	  }
 	if (ppmap)
 	   *ppmap = None;
 	if (ppixel)
-	   *ppixel = bg->bg_solid.pixel;
+	   *ppixel = pixel;
 	return;
      }
 
@@ -650,8 +648,7 @@ BackgroundRealize(Background * bg, Win win, Drawable draw, unsigned int rw,
 	if (!hasbg || !bg->bg_tile)
 	  {
 	     /* Fill solid */
-	     EImageFill(im, 0, 0, rw, rh, bg->bg_solid.red, bg->bg_solid.green,
-			bg->bg_solid.blue, 255);
+	     EImageFill(im, 0, 0, rw, rh, bg->bg_solid);
 	  }
 	if (hasbg)
 	  {
@@ -753,7 +750,7 @@ BrackgroundCreateFromImage(const char *bgid, const char *file,
 {
    Background         *bg;
    EImage             *im, *im2;
-   EColor              color;
+   unsigned int        color;
    char                tile = 1, keep_asp = 0;
    int                 width, height;
    int                 scalex = 0, scaley = 0;
@@ -852,9 +849,9 @@ BrackgroundCreateFromImage(const char *bgid, const char *file,
 	keep_asp = 1;
      }
 
-   SET_COLOR(&color, 0, 0, 0);
+   COLOR32_FROM_RGB(color, 0, 0, 0);
 
-   bg = BackgroundCreate(bgid, &color, file, tile,
+   bg = BackgroundCreate(bgid, color, file, tile,
 			 keep_asp, justx, justy,
 			 scalex, scaley, NULL, 0, 0, 0, 0, 0);
 
@@ -966,7 +963,7 @@ BackgroundGetInfoString1(const Background * bg, char *buf, int len)
 {
    int                 r, g, b;
 
-   GET_COLOR(&(bg->bg_solid), r, g, b);
+   COLOR32_TO_RGB(bg->bg_solid, r, g, b);
    Esnprintf(buf, len,
 	     "%s ref_count %u keepim %u\n"
 	     " bg.solid\t %i %i %i \n"
@@ -991,7 +988,7 @@ BackgroundGetInfoString2(const Background * bg, char *buf, int len)
 {
    int                 r, g, b;
 
-   GET_COLOR(&(bg->bg_solid), r, g, b);
+   COLOR32_TO_RGB(bg->bg_solid, r, g, b);
    Esnprintf(buf, len,
 	     "%s %i %i %i %s %i %i %i %i %i %i %s %i %i %i %i %i",
 	     bg->name, r, g, b, S(bg->bg.file), bg->bg_tile,
@@ -1064,7 +1061,7 @@ BackgroundsConfigLoad(FILE * fs)
 {
    int                 err = 0;
    Background         *bg = 0;
-   EColor              color;
+   unsigned int        color;
    char                s[FILEPATH_LEN_MAX];
    char                s2[FILEPATH_LEN_MAX];
    int                 ii1;
@@ -1077,7 +1074,7 @@ BackgroundsConfigLoad(FILE * fs)
    char                ignore = 0;
    int                 desk;
 
-   SET_COLOR(&color, 0, 0, 0);
+   COLOR32_FROM_RGB(color, 0, 0, 0);
 
    while (GetLine(s, sizeof(s), fs))
      {
@@ -1086,7 +1083,7 @@ BackgroundsConfigLoad(FILE * fs)
 	  {
 	  case CONFIG_CLOSE:
 	     if (!ignore && !bg && name)
-		bg = BackgroundCreate(name, &color, bg1, i1, i2, i3, i4, i5,
+		bg = BackgroundCreate(name, color, bg1, i1, i2, i3, i4, i5,
 				      i6, bg2, j1, j2, j3, j4, j5);
 	     goto done;
 
@@ -1110,7 +1107,7 @@ BackgroundsConfigLoad(FILE * fs)
 
 	  case BG_DESKNUM:
 	     if (!ignore && !bg && name)
-		bg = BackgroundCreate(name, &color, bg1, i1, i2, i3, i4, i5,
+		bg = BackgroundCreate(name, color, bg1, i1, i2, i3, i4, i5,
 				      i6, bg2, j1, j2, j3, j4, j5);
 	     if (!bg)
 		break;
@@ -1139,7 +1136,7 @@ BackgroundsConfigLoad(FILE * fs)
 	  case BG_RGB:
 	     r = g = b = 0;
 	     sscanf(s, "%*s %d %d %d", &r, &g, &b);
-	     SET_COLOR(&color, r, g, b);
+	     COLOR32_FROM_RGB(color, r, g, b);
 	     if (ignore)
 		bg->bg_solid = color;
 	     break;
@@ -1238,7 +1235,7 @@ BackgroundsConfigSave(void)
 	fprintf(fs, "5 999\n");
 
 	fprintf(fs, "100 %s\n", bg->name);
-	GET_COLOR(&(bg->bg_solid), r, g, b);
+	COLOR32_TO_RGB(bg->bg_solid, r, g, b);
 	fprintf(fs, "560 %d %d %d\n", r, g, b);
 
 	if ((bg->bg.file) && (!bg->bg.real_file))
@@ -1367,8 +1364,7 @@ BackgroundsSighan(int sig, void *prm __UNUSED__)
 	EDirMake(EDirUserCache(), "cached/bgsel");
 	EDirMake(EDirUserCache(), "cached/img");
 	/* create a fallback background in case no background is found */
-	BackgroundCreate("NONE", NULL, NULL, 0, 0, 0, 0, 0, 0, NULL, 0, 0, 0, 0,
-			 0);
+	BackgroundCreate(NULL, 0, NULL, 0, 0, 0, 0, 0, 0, NULL, 0, 0, 0, 0, 0);
 	break;
 
      case ESIGNAL_CONFIGURE:
@@ -1392,7 +1388,6 @@ BackgroundsSighan(int sig, void *prm __UNUSED__)
 /*
  * Configuration dialog
  */
-static Dialog      *bg_sel_dialog;
 static DItem       *bg_sel;
 static DItem       *bg_sel_slider;
 static DItem       *bg_mini_disp;
@@ -1439,7 +1434,7 @@ CB_ConfigureBG(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
 	Conf.backgrounds.user = tmp_userbg;
 	Conf.hints.set_xroot_info_on_root_window = tmp_root_hint;
 
-	SET_COLOR(&(tmp_bg->bg_solid), tmp_bg_r, tmp_bg_g, tmp_bg_b);
+	COLOR32_FROM_RGB(tmp_bg->bg_solid, tmp_bg_r, tmp_bg_g, tmp_bg_b);
 	tmp_bg->bg_tile = tmp_bg_tile;
 	tmp_bg->bg.keep_aspect = tmp_bg_keep_aspect;
 	tmp_bg->bg.xjust = tmp_bg_xjust;
@@ -1468,7 +1463,7 @@ CB_DesktopMiniDisplayRedraw(Dialog * d __UNUSED__, int val __UNUSED__,
    int                 w, h;
    DItem              *di;
    Win                 win;
-   EColor              color;
+   unsigned int        color;
    const char         *fbg, *ffg;
 
    if (!tmp_bg)
@@ -1481,8 +1476,8 @@ CB_DesktopMiniDisplayRedraw(Dialog * d __UNUSED__, int val __UNUSED__,
    pmap = EGetWindowBackgroundPixmap(win);
    fbg = (tmp_bg_image) ? BackgroundGetBgFile(tmp_bg) : NULL;
    ffg = (tmp_bg_image) ? BackgroundGetFgFile(tmp_bg) : NULL;
-   SET_COLOR(&color, tmp_bg_r, tmp_bg_g, tmp_bg_b);
-   bg = BackgroundCreate("TEMP", &color,
+   COLOR32_FROM_RGB(color, tmp_bg_r, tmp_bg_g, tmp_bg_b);
+   bg = BackgroundCreate("TEMP", color,
 			 fbg, tmp_bg_tile, tmp_bg_keep_aspect,
 			 tmp_bg_xjust, tmp_bg_yjust,
 			 tmp_bg_xperc, tmp_bg_yperc,
@@ -1501,7 +1496,7 @@ BG_GetValues(void)
 {
    tmp_bg_image = (tmp_bg->bg.file) ? 1 : 0;
 
-   GET_COLOR(&(tmp_bg->bg_solid), tmp_bg_r, tmp_bg_g, tmp_bg_b);
+   COLOR32_TO_RGB(tmp_bg->bg_solid, tmp_bg_r, tmp_bg_g, tmp_bg_b);
    tmp_bg_tile = tmp_bg->bg_tile;
    tmp_bg_keep_aspect = tmp_bg->bg.keep_aspect;
    tmp_bg_xjust = tmp_bg->bg.xjust;
@@ -1526,8 +1521,6 @@ BG_DialogSetFileName(DItem * di)
 static void
 BgDialogSetNewCurrent(Background * bg)
 {
-   int                 i;
-
    if (tmp_bg && tmp_bg != bg)
       BackgroundImagesKeep(tmp_bg, 0);
    tmp_bg = bg;
@@ -1538,7 +1531,6 @@ BgDialogSetNewCurrent(Background * bg)
 
    /* Update dialog items */
    BG_DialogSetFileName(bg_filename);
-   DialogDrawItems(bg_sel_dialog, bg_filename, 0, 0, 99999, 99999);
 
    DialogItemCheckButtonSetState(tmp_w[0], tmp_bg_image);
    DialogItemCheckButtonSetState(tmp_w[1], tmp_bg_keep_aspect);
@@ -1556,9 +1548,6 @@ BgDialogSetNewCurrent(Background * bg)
 
    /* Redraw scrolling BG list */
    BG_RedrawView();
-
-   for (i = 0; i < 10; i++)
-      DialogDrawItems(bg_sel_dialog, tmp_w[i], 0, 0, 99999, 99999);
 }
 
 /* Duplicate current (tmp_bg) to new */
@@ -1567,14 +1556,14 @@ CB_ConfigureNewBG(Dialog * d __UNUSED__, int val __UNUSED__,
 		  void *data __UNUSED__)
 {
    char                s[1024];
-   EColor              color;
+   unsigned int        color;
    int                 lower, upper;
 
    Esnprintf(s, sizeof(s), "__NEWBG_%i", (unsigned)time(NULL));
 
-   SET_COLOR(&color, tmp_bg_r, tmp_bg_g, tmp_bg_b);
+   COLOR32_FROM_RGB(color, tmp_bg_r, tmp_bg_g, tmp_bg_b);
 
-   tmp_bg = BackgroundCreate(s, &color,
+   tmp_bg = BackgroundCreate(s, color,
 			     tmp_bg->bg.file, tmp_bg_tile, tmp_bg_keep_aspect,
 			     tmp_bg_xjust, tmp_bg_yjust,
 			     tmp_bg_xperc, tmp_bg_yperc,
@@ -1587,7 +1576,6 @@ CB_ConfigureNewBG(Dialog * d __UNUSED__, int val __UNUSED__,
    DialogItemSliderSetBounds(bg_sel_slider, lower, upper);
 
    DialogItemSliderSetVal(bg_sel_slider, 0);
-   DialogDrawItems(bg_sel_dialog, bg_sel_slider, 0, 0, 99999, 99999);
 
    DeskBackgroundSet(DesksGetCurrent(), tmp_bg);
 
@@ -1601,7 +1589,7 @@ CB_ConfigureDelBG(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
 {
    Background         *bg;
    int                 i, num;
-   int                 slider, lower, upper;
+   int                 lower, upper;
 
    num = ecore_list_count(bg_list);
    if (num <= 1)
@@ -1625,10 +1613,9 @@ CB_ConfigureDelBG(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
    tmp_bg = NULL;
 
    DialogItemSliderGetBounds(bg_sel_slider, &lower, &upper);
-   slider = DialogItemSliderGetVal(bg_sel_slider);
    upper -= 4;
    DialogItemSliderSetBounds(bg_sel_slider, lower, upper);
-   if (slider > upper)
+   if (tmp_bg_sel_sliderval > upper)
       DialogItemSliderSetVal(bg_sel_slider, upper);
 
    BgDialogSetNewCurrent(bg);
@@ -1655,7 +1642,7 @@ BG_RedrawView(void)
    int                 x, w, h, num;
    Win                 win;
    Pixmap              pmap;
-   ImageClass         *ic_button;
+   ImageClass         *ic;
 
    num = ecore_list_count(bg_list);
    if (num <= 0)
@@ -1666,9 +1653,11 @@ BG_RedrawView(void)
 
    pmap = EGetWindowBackgroundPixmap(win);
 
-   ic_button = ImageclassFind("DIALOG_BUTTON", 1);
+   ic = ImageclassFind("DIALOG_BUTTON", 0);
+   if (!ic)
+      ic = ImageclassFind("DIALOG_WIDGET_BUTTON", 1);
 
-   ImageclassApplySimple(ic_button, win, pmap, STATE_NORMAL, 0, 0, w, h);
+   ImageclassApplySimple(ic, win, pmap, STATE_NORMAL, 0, 0, w, h);
 
    x = -(num * (64 + 8) - w) * tmp_bg_sel_sliderval / (4 * num);
 
@@ -1678,7 +1667,7 @@ BG_RedrawView(void)
 	{
 	   EImage             *im;
 
-	   ImageclassApplySimple(ic_button, win, pmap,
+	   ImageclassApplySimple(ic, win, pmap,
 				 (bg == tmp_bg) ? STATE_CLICKED : STATE_NORMAL,
 				 x, 0, 64 + 8, 48 + 8);
 
@@ -1737,18 +1726,17 @@ CB_BGScan(Dialog * d, int val __UNUSED__, void *data __UNUSED__)
 
    num = ecore_list_count(bg_list);
    DialogItemSliderSetBounds(bg_sel_slider, 0, num * 4);
-   DialogDrawItems(d, bg_sel_slider, 0, 0, 99999, 99999);
    DialogItemCallCallback(d, bg_sel_slider);
 }
 
 static void
-CB_BGAreaEvent(DItem * di __UNUSED__, int val __UNUSED__, void *data)
+CB_BGAreaEvent(DItem * di, int val __UNUSED__, void *data)
 {
    int                 x, num, w, h;
    Background         *bg;
    XEvent             *ev = (XEvent *) data;
 
-   DialogItemAreaGetSize(bg_sel, &w, &h);
+   DialogItemAreaGetSize(di, &w, &h);
 
    switch (ev->type)
      {
@@ -1775,7 +1763,6 @@ CB_BGAreaEvent(DItem * di __UNUSED__, int val __UNUSED__, void *data)
 	   do_slide:
 	     DialogItemSliderSetVal(bg_sel_slider, tmp_bg_sel_sliderval);
 	     CB_BGAreaSlide(NULL, 0, NULL);
-	     DialogDrawItems(bg_sel_dialog, bg_sel_slider, 0, 0, 99999, 99999);
 	     break;
 	  }
      }
@@ -1793,7 +1780,6 @@ CB_DesktopTimeout(Dialog * d __UNUSED__, int val __UNUSED__, void *data)
 	     (tmp_bg_timeout / 60) - (60 * (tmp_bg_timeout / 3600)),
 	     (tmp_bg_timeout) - (60 * (tmp_bg_timeout / 60)));
    DialogItemSetText(di, s);
-   DialogDrawItems(bg_sel_dialog, di, 0, 0, 99999, 99999);
 }
 
 static void
@@ -1816,7 +1802,6 @@ BGSettingsGoTo(Background * bg)
    else if (i > 4 * num)
       i = 4 * num;
    DialogItemSliderSetVal(bg_sel_slider, i);
-   DialogDrawItems(bg_sel_dialog, bg_sel_slider, 0, 0, 99999, 99999);
    BgDialogSetNewCurrent(bg);
 }
 
@@ -1990,8 +1975,6 @@ _DlgFillBackground(Dialog * d, DItem * table, void *data)
    tmp_root_hint = Conf.hints.set_xroot_info_on_root_window;
    tmp_bg_timeout = Conf.backgrounds.timeout;
 
-   bg_sel_dialog = d;
-
    DialogItemTableSetOptions(table, 1, 0, 0, 0);
 
    table2 = DialogAddItem(table, DITEM_TABLE);
@@ -2257,7 +2240,7 @@ BackgroundSet1(const char *name, const char *params)
    char                type[FILEPATH_LEN_MAX];
    int                 len, value;
    Background         *bg;
-   EColor              color;
+   unsigned int        color;
 
    if (!p || !p[0])
       return;
@@ -2265,12 +2248,12 @@ BackgroundSet1(const char *name, const char *params)
    bg = BackgroundFind(name);
    if (!bg)
      {
-	SET_COLOR(&color, 0, 0, 0);
-	bg = BackgroundCreate(name, &color, NULL, 0, 0, 0,
+	COLOR32_FROM_RGB(color, 0, 0, 0);
+	bg = BackgroundCreate(name, color, NULL, 0, 0, 0,
 			      0, 0, 0, NULL, 0, 0, 0, 0, 0);
 	if (!bg)
 	  {
-	     IpcPrintf("Error: could not create background '%s'.", name);
+	     IpcPrintf("Error: could not create background '%s'\n", name);
 	     return;
 	  }
      }
@@ -2287,7 +2270,7 @@ BackgroundSet1(const char *name, const char *params)
 
 	r = g = b = 0;
 	sscanf(p, "%i %i %i", &r, &g, &b);
-	SET_COLOR(&(bg->bg_solid), r, g, b);
+	COLOR32_FROM_RGB(bg->bg_solid, r, g, b);
      }
    else if (!strcmp(type, "bg.file"))
      {
@@ -2345,7 +2328,7 @@ BackgroundSet1(const char *name, const char *params)
      }
    else
      {
-	IpcPrintf("Error: unknown background value type '%s'.", type);
+	IpcPrintf("Error: unknown background value type '%s'\n", type);
      }
    autosave();
 }
@@ -2354,7 +2337,7 @@ static void
 BackgroundSet2(const char *name, const char *params)
 {
    Background         *bg;
-   EColor              color;
+   unsigned int        color;
    unsigned int        i;
    int                 r, g, b;
    char                bgf[FILEPATH_LEN_MAX], topf[FILEPATH_LEN_MAX];
@@ -2372,18 +2355,18 @@ BackgroundSet2(const char *name, const char *params)
 	      &r, &g, &b,
 	      bgf, &tile, &keep_aspect, &xjust, &yjust, &xperc, &yperc,
 	      topf, &tkeep_aspect, &txjust, &tyjust, &txperc, &typerc);
-   SET_COLOR(&color, r, g, b);
+   COLOR32_FROM_RGB(color, r, g, b);
 
    bg = BackgroundFind(name);
    if (bg)
      {
-	BackgroundModify(bg, &color, bgf, tile, keep_aspect, xjust,
+	BackgroundModify(bg, color, bgf, tile, keep_aspect, xjust,
 			 yjust, xperc, yperc, topf, tkeep_aspect,
 			 txjust, tyjust, txperc, typerc);
      }
    else
      {
-	bg = BackgroundCreate(name, &color, bgf, tile, keep_aspect, xjust,
+	bg = BackgroundCreate(name, color, bgf, tile, keep_aspect, xjust,
 			      yjust, xperc, yperc, topf, tkeep_aspect,
 			      txjust, tyjust, txperc, typerc);
      }
@@ -2469,7 +2452,7 @@ BackgroundsIpc(const char *params)
 	     IpcPrintf("%s\n", buf);
 	  }
 	else
-	   IpcPrintf("Error: background '%s' does not exist.", prm);
+	   IpcPrintf("Error: background '%s' does not exist\n", prm);
      }
    else if (!strcmp(cmd, "use"))
      {
@@ -2493,7 +2476,7 @@ BackgroundsIpc(const char *params)
 	     IpcPrintf("%s\n", buf);
 	  }
 	else
-	   IpcPrintf("Error: background '%s' does not exist.", prm);
+	   IpcPrintf("Error: background '%s' does not exist\n", prm);
      }
    else if (!strncmp(cmd, "xset", 2))
      {
