@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -31,6 +31,7 @@
 #include "grabs.h"
 #include "hints.h"
 #include "iclass.h"
+#include "icons.h"
 #include "snaps.h"
 #include "tclass.h"
 #include "timers.h"
@@ -75,9 +76,11 @@ static void
 BorderWinpartITclassApply(EWin * ewin, int i, int force)
 {
    EWinBit            *ewb = &ewin->bits[i];
+   WinPart            *ebp = &ewin->border->part[i];
    ImageState         *is;
    TextState          *ts;
    const char         *txt;
+   int                 flags;
 
    if (ewb->win == None)
       return;
@@ -88,23 +91,25 @@ BorderWinpartITclassApply(EWin * ewin, int i, int force)
 	   EwinGetTitle(ewin));
 #endif
 
-   is = ImageclassGetImageState(ewin->border->part[i].iclass, ewb->state,
+   is = ImageclassGetImageState(ebp->iclass, ewb->state,
 				ewin->state.active, EoIsSticky(ewin));
 
    ts = NULL;
    txt = NULL;
-   switch (ewin->border->part[i].flags)
+   flags = 0;
+   if (ebp->flags & FLAG_TITLE)
      {
-     case FLAG_TITLE:
 	txt = EwinGetTitle(ewin);
-	if (txt && ewin->border->part[i].tclass)
-	   ts = TextclassGetTextState(ewin->border->part[i].tclass, ewb->state,
-				      ewin->state.active, EoIsSticky(ewin));
-	break;
-     case FLAG_MINIICON:
-	break;
-     default:
-	break;
+	if (txt && ebp->tclass)
+	  {
+	     ts = TextclassGetTextState(ebp->tclass, ewb->state,
+					ewin->state.active, EoIsSticky(ewin));
+	     flags = ITA_BGPMAP | ITA_JUSTV;
+	  }
+     }
+   if (ebp->flags & FLAG_MINIICON)
+     {
+	flags |= ITA_BGPMAP;
      }
 
    if (!force && ewb->is == is && ewb->ts == ts)
@@ -112,9 +117,61 @@ BorderWinpartITclassApply(EWin * ewin, int i, int force)
    ewb->is = is;
    ewb->ts = ts;
 
-   ITApply(ewb->win, ewin->border->part[i].iclass, is,
-	   ewb->state, ewin->state.active, EoIsSticky(ewin),
-	   ST_BORDER, ewin->border->part[i].tclass, ts, txt, 1);
+   ITApply(ewb->win, ebp->iclass, is, ewb->state,
+	   ewin->state.active, EoIsSticky(ewin), ST_BORDER,
+	   ebp->tclass, ts, txt, flags);
+
+   if (ebp->flags & FLAG_MINIICON)
+     {
+	EImage             *im;
+
+	im = EwinIconImageGet(ewin, 16, Conf.warplist.icon_mode);
+	if (im)
+	  {
+	     Pixmap              pmap;
+	     EImageBorder       *pad;
+	     int                 x, y, w, h;
+
+	     x = y = 0;
+	     w = WinGetW(ewb->win);
+	     h = WinGetH(ewb->win);
+	     pad = ImageclassGetPadding(ebp->iclass);
+	     if (ts)
+	       {
+		  /* Occupy square on one side */
+		  switch (ts->style.orientation)
+		    {
+		    default:
+		    case FONT_TO_RIGHT:
+		    case FONT_TO_LEFT:
+		       w = h;
+		       break;
+		    case FONT_TO_UP:
+		       y = h - w;
+		    case FONT_TO_DOWN:
+		       h = w;
+		       break;
+		    }
+	       }
+	     if (pad)
+	       {
+		  /* Probably not quite right for vertical borders */
+		  x += pad->left;
+		  y += pad->top;
+		  w -= pad->left + pad->right;
+		  h -= pad->top + pad->bottom;
+	       }
+	     if (w < 8 || h < 8)
+		goto skip_icon;
+	     pmap = EGetWindowBackgroundPixmap(ewb->win);
+	     EImageRenderOnDrawable(im, ewb->win, pmap,
+				    EIMAGE_BLEND | EIMAGE_ANTI_ALIAS,
+				    x, y, w, h);
+	     EClearWindow(ewb->win);
+	   skip_icon:
+	     EImageFree(im);
+	  }
+     }
 }
 
 static int
@@ -162,9 +219,6 @@ EwinBorderDraw(EWin * ewin, int do_shape, int do_paint)
 {
    int                 i;
 
-   if (!ewin)
-      return;
-
 #if 0				/* Debug */
    Eprintf("EwinBorderDraw %#lx %s d=%d s=%d p=%d\n",
 	   EwinGetClientXwin(ewin), EoGetName(ewin), EoGetDeskNum(ewin),
@@ -185,13 +239,13 @@ EwinBorderUpdateInfo(EWin * ewin)
 
    for (i = 0; i < ewin->border->num_winparts; i++)
      {
-	if (ewin->border->part[i].flags == FLAG_TITLE)
+	if (ewin->border->part[i].flags & FLAG_TITLE)
 	   BorderWinpartITclassApply(ewin, i, 1);
      }
 }
 
 static void
-BorderWinpartCalc(EWin * ewin, int i, int ww, int hh)
+BorderWinpartCalc(const EWin * ewin, int i, int ww, int hh)
 {
    int                 x, y, w, h, ox, oy, max, min;
    int                 topleft, bottomright;
@@ -258,7 +312,7 @@ BorderWinpartCalc(EWin * ewin, int i, int ww, int hh)
     * just a little bit more than the size of the title text.
     */
 
-   if (max == 0 && ewin->border->part[i].flags == FLAG_TITLE)
+   if (max == 0 && ewin->border->part[i].flags & FLAG_TITLE)
      {
 	int                 dummywidth, wmax, wmin;
 	ImageClass         *iclass;
@@ -323,7 +377,7 @@ BorderWinpartCalc(EWin * ewin, int i, int ww, int hh)
 	 * size to just a little bit more than the size of the title text.
 	 */
 
-	if (max == 0 && ewin->border->part[i].flags == FLAG_TITLE)
+	if (max == 0 && ewin->border->part[i].flags & FLAG_TITLE)
 	  {
 	     int                 dummyheight;
 	     ImageClass         *iclass;
@@ -377,8 +431,6 @@ EwinBorderCalcSizes(EWin * ewin, int propagate)
    int                 i, ww, hh;
    char                reshape;
 
-   if (!ewin)
-      return;
    if (!ewin->border)
       return;
 
@@ -471,10 +523,10 @@ EwinBorderDetach(EWin * ewin)
 
    TooltipsSetPending(0, NULL, NULL);
 
-   EventCallbackUnregister(EoGetWin(ewin), 0, BorderFrameHandleEvents, ewin);
+   EventCallbackUnregister(EoGetWin(ewin), BorderFrameHandleEvents, ewin);
    for (i = 0; i < b->num_winparts; i++)
      {
-	EventCallbackUnregister(ewin->bits[i].win, 0,
+	EventCallbackUnregister(ewin->bits[i].win,
 				BorderWinpartHandleEvents, &ewin->bits[i]);
 	if (ewin->bits[i].win)
 	   EDestroyWindow(ewin->bits[i].win);
@@ -512,7 +564,7 @@ EwinBorderSetTo(EWin * ewin, const Border * b)
 
    ewin->state.no_border = b->num_winparts <= 0;
 
-   EventCallbackRegister(EoGetWin(ewin), 0, BorderFrameHandleEvents, ewin);
+   EventCallbackRegister(EoGetWin(ewin), BorderFrameHandleEvents, ewin);
 
    if (b->num_winparts > 0)
       ewin->bits = EMALLOC(EWinBit, b->num_winparts);
@@ -524,7 +576,7 @@ EwinBorderSetTo(EWin * ewin, const Border * b)
 	ewin->bits[i].win = ECreateWindow(EoGetWin(ewin), -10, -10, 1, 1, 0);
 	ECursorApply(b->part[i].ec, ewin->bits[i].win);
 	EMapWindow(ewin->bits[i].win);
-	EventCallbackRegister(ewin->bits[i].win, 0,
+	EventCallbackRegister(ewin->bits[i].win,
 			      BorderWinpartHandleEvents, &ewin->bits[i]);
 	if (b->part[i].flags & FLAG_TITLE)
 	   ESelectInput(ewin->bits[i].win, EWIN_BORDER_TITLE_EVENT_MASK);
@@ -712,20 +764,14 @@ BorderWinpartAdd(Border * b, const char *iclass, const char *aclass,
 }
 
 void
-EwinBorderMinShadeSize(EWin * ewin, int *mw, int *mh)
+EwinBorderMinShadeSize(const EWin * ewin, int *mw, int *mh)
 {
    int                 i, pw, ph, w, h, min_w, min_h;
    int                 leftborderwidth, rightborderwidth;
    int                 topborderwidth, bottomborderwidth;
 
-   min_w = 32;
-   min_h = 32;
-
-   if (!ewin)
-      goto done;
-
-   pw = EoGetW(ewin);
-   ph = EoGetH(ewin);
+   pw = ewin->client.w + ewin->border->border.left + ewin->border->border.right;
+   ph = ewin->client.h + ewin->border->border.top + ewin->border->border.bottom;
 
    for (i = 0; i < ewin->border->num_winparts; i++)
       ewin->bits[i].w = -2;
@@ -750,7 +796,7 @@ EwinBorderMinShadeSize(EWin * ewin, int *mw, int *mh)
 		leftborderwidth = w;
 
 	     w = ewin->bits[i].x + ewin->bits[i].w -
-		(EoGetW(ewin) - ewin->border->border.right);
+		(pw - ewin->border->border.right);
 	     if (rightborderwidth < w)
 		rightborderwidth = w;
 	  }
@@ -769,7 +815,7 @@ EwinBorderMinShadeSize(EWin * ewin, int *mw, int *mh)
 		topborderwidth = h;
 
 	     h = ewin->bits[i].y + ewin->bits[i].h -
-		(EoGetH(ewin) - ewin->border->border.bottom);
+		(ph - ewin->border->border.bottom);
 	     if (bottomborderwidth < h)
 		bottomborderwidth = h;
 	  }
@@ -801,7 +847,6 @@ EwinBorderMinShadeSize(EWin * ewin, int *mw, int *mh)
 	   min_h = h;
      }
 
- done:
    *mw = min_w;
    *mh = min_h;
 }
@@ -856,7 +901,15 @@ BorderWinpartEventMouseUp(EWinBit * wbit, XEvent * ev)
    if (ev)
      {
 	GrabPointerRelease();
-	ActionsEnd(ewin);
+	/*
+	 * During a move/resize operation initiated by a border part click
+	 * the ButtonRelease event may be reported on the border part window
+	 * if the event happens before the pointer grab is moved to the
+	 * move/resize event window.
+	 * This can be tested e.g. by inserting usleep(100000) at the start
+	 * of BorderWinpartEventMouseDown().
+	 */
+	MoveResizeEnd(ewin);
      }
 
    if ((wbit->state == STATE_CLICKED) && (!wbit->left))
