@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2008 Kim Woelders
+ * Copyright (C) 2004-2009 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -133,7 +133,9 @@ ButtonCreate(const char *name, int id, const char *iclass,
    ecore_list_append(button_list, b);
 
    b->id = id;
-   b->label = Estrdup(label);
+
+   if (label && *label)
+      b->label = Estrdup(label);
 
    b->iclass = ImageclassAlloc(iclass, 1);
    b->aclass = ActionclassAlloc(aclass);
@@ -165,7 +167,7 @@ ButtonCreate(const char *name, int id, const char *iclass,
    EoSetFade(b, 1);
 
    ESelectInput(EoGetWin(b), BUTTON_EVENT_MASK);
-   EventCallbackRegister(EoGetWin(b), 0, ButtonHandleEvents, b);
+   EventCallbackRegister(EoGetWin(b), ButtonHandleEvents, b);
 
    return b;
 }
@@ -394,7 +396,7 @@ ButtonEmbedWindow(Button * b, Window WindowToEmbed)
    EGetGeometry(WindowToEmbed, NULL, NULL, NULL, &w, &h, NULL, NULL);
    EMoveWindow(b->inside_win, (EoGetW(b) - w) >> 1, (EoGetH(b) - h) >> 1);
    b->event_win = ECreateEventWindow(EoGetWin(b), 0, 0, w, h);
-   EventCallbackRegister(b->event_win, 0, ButtonHandleEvents, b);
+   EventCallbackRegister(b->event_win, ButtonHandleEvents, b);
 
    ESelectInput(b->event_win,
 		ButtonPressMask | ButtonReleaseMask | EnterWindowMask |
@@ -694,13 +696,17 @@ ButtonsConfigLoad(FILE * fs)
 	       }
 	     else if (pbt)
 	       {
-		  _EFDUP(pbt->label, label);
+		  if (label[0])
+		     _EFDUP(pbt->label, label);
 		  EoSetLayer(pbt, ontop);
 		  EoSetSticky(pbt, sticky);
 		  ButtonMoveToDesktop(pbt, DeskGet(desk));
-		  pbt->iclass = ImageclassFind(iclass, 1);
-		  pbt->aclass = ActionclassFind(aclass);
-		  pbt->tclass = TextclassFind(tclass, 1);
+		  if (iclass[0])
+		     pbt->iclass = ImageclassFind(iclass, 1);
+		  if (aclass[0])
+		     pbt->aclass = ActionclassFind(aclass);
+		  if (tclass[0])
+		     pbt->tclass = TextclassFind(tclass, 1);
 		  pbt->flags = flags;
 		  pbt->internal = internal;
 		  pbt->default_show = show;
@@ -848,12 +854,14 @@ ButtonsConfigSave(void)
 
       fprintf(fs, "4 999\n");
       fprintf(fs, "100 %s\n", EoGetName(b));
+#if 0				/* Remove? */
       if (b->iclass)
 	 fprintf(fs, "12 %s\n", ImageclassGetName(b->iclass));
       if (b->aclass)
 	 fprintf(fs, "11 %s\n", ActionclassGetName(b->aclass));
       if (EoGetLayer(b) >= 0)
 	 fprintf(fs, "453 %i\n", EoGetLayer(b));
+#endif
       fprintf(fs, "456 %i\n", b->geom.width_min);
       fprintf(fs, "457 %i\n", b->geom.width_max);
       fprintf(fs, "468 %i\n", b->geom.height_min);
