@@ -48,6 +48,14 @@ static void         ContainerEventArrow1Win(Win win, XEvent * ev, void *prm);
 static void         ContainerEventArrow2Win(Win win, XEvent * ev, void *prm);
 static void         ContainerEventIconWin(Win win, XEvent * ev, void *prm);
 
+/* Widget state */
+#define WS_HILITE 0x01
+#define WS_CLICK  0x02
+
+static const unsigned char _ic_state[4] =
+   { STATE_NORMAL, STATE_HILITED, STATE_CLICKED, STATE_CLICKED };
+#define ST(b) _ic_state[b]
+
 ContainerCfg        Conf_containers;
 
 static Ecore_List  *container_list = NULL;
@@ -105,29 +113,27 @@ ContainerCreate(const char *name)
    ct->w = 0;
    ct->h = 0;
    ct->pos = 0;
-   ct->max = 1;
-   ct->arrow1_hilited = 0;
-   ct->arrow1_clicked = 0;
-   ct->arrow2_hilited = 0;
-   ct->arrow2_clicked = 0;
-   ct->icon_clicked = 0;
-   ct->scrollbar_hilited = 0;
-   ct->scrollbar_clicked = 0;
+   ct->iwin_maxl = 1;
+
+   ct->scrollbar_state = 0;
+   ct->arrow1_state = 0;
+   ct->arrow2_state = 0;
    ct->scrollbox_clicked = 0;
+   ct->icon_clicked = 0;
 
    ct->win = ECreateClientWindow(VROOT, 0, 0, 1, 1);
    ct->icon_win = ECreateWindow(ct->win, 0, 0, 128, 26, 0);
-   EventCallbackRegister(ct->icon_win, 0, ContainerEventIconWin, ct);
+   EventCallbackRegister(ct->icon_win, ContainerEventIconWin, ct);
    ct->cover_win = ECreateWindow(ct->win, 0, 0, 128, 26, 0);
-   EventCallbackRegister(ct->cover_win, 0, ContainerEventCoverWin, ct);
+   EventCallbackRegister(ct->cover_win, ContainerEventCoverWin, ct);
    ct->scroll_win = ECreateWindow(ct->win, 6, 26, 116, 6, 0);
-   EventCallbackRegister(ct->scroll_win, 0, ContainerEventScrollWin, ct);
+   EventCallbackRegister(ct->scroll_win, ContainerEventScrollWin, ct);
    ct->arrow1_win = ECreateWindow(ct->win, 0, 26, 6, 6, 0);
-   EventCallbackRegister(ct->arrow1_win, 0, ContainerEventArrow1Win, ct);
+   EventCallbackRegister(ct->arrow1_win, ContainerEventArrow1Win, ct);
    ct->arrow2_win = ECreateWindow(ct->win, 122, 26, 6, 6, 0);
-   EventCallbackRegister(ct->arrow2_win, 0, ContainerEventArrow2Win, ct);
+   EventCallbackRegister(ct->arrow2_win, ContainerEventArrow2Win, ct);
    ct->scrollbar_win = ECreateWindow(ct->scroll_win, 122, 26, 6, 6, 0);
-   EventCallbackRegister(ct->scrollbar_win, 0, ContainerEventScrollbarWin, ct);
+   EventCallbackRegister(ct->scrollbar_win, ContainerEventScrollbarWin, ct);
    ct->scrollbarknob_win = ECreateWindow(ct->scrollbar_win, -20, -20, 4, 4, 0);
 
    ESelectInput(ct->icon_win,
@@ -185,52 +191,22 @@ ContainerDestroy(Container * ct, int exiting)
 static void
 ContainerReconfigure(Container * ct)
 {
-   ImageClass         *ic, *ic2;
-   EImageBorder       *pad;
-   EWin               *ewin;
-   int                 extra;
    unsigned int        wmin, hmin, wmax, hmax;
 
-   ewin = ct->ewin;
-
-   wmin = hmin = 8;
    wmax = hmax = 16384;
 
-   extra = 0;
    if (ct->orientation)
      {
-	ic = ImageclassFind("ICONBOX_VERTICAL", 0);
-	pad = ImageclassGetPadding(ic);
-	if (ic)
-	   extra = pad->left + pad->right;
-	if (ct->draw_icon_base)
-	  {
-	     ic2 = ImageclassFind("DEFAULT_ICON_BUTTON", 0);
-	     pad = ImageclassGetPadding(ic2);
-	     if (ic2)
-		extra += pad->left + pad->right;
-	  }
-	wmax = wmin = ct->iconsize + ct->scroll_thickness + extra;
-	ct->max_min = hmin;
+	wmax = wmin = ct->iwin_fixh + ct->scroll_thickness_set;
+	hmin = ct->iwin_maxl_min;
      }
    else
      {
-	ic = ImageclassFind("ICONBOX_HORIZONTAL", 0);
-	pad = ImageclassGetPadding(ic);
-	if (ic)
-	   extra = pad->top + pad->bottom;
-	if (ct->draw_icon_base)
-	  {
-	     ic2 = ImageclassFind("DEFAULT_ICON_BUTTON", 0);
-	     pad = ImageclassGetPadding(ic2);
-	     if (ic2)
-		extra += pad->top + pad->bottom;
-	  }
-	hmax = hmin = ct->iconsize + ct->scroll_thickness + extra;
-	ct->max_min = wmin;
+	hmax = hmin = ct->iwin_fixh + ct->scroll_thickness_set;
+	wmin = ct->iwin_maxl_min;
      }
 
-   ICCCM_SetSizeConstraints(ewin, wmin, hmin, wmax, hmax, 0, 0, 1, 1,
+   ICCCM_SetSizeConstraints(ct->ewin, wmin, hmin, wmax, hmax, 0, 0, 1, 1,
 			    0.0, 65535.0);
 }
 
@@ -257,10 +233,13 @@ static void
 _ContainerEwinLayout(EWin * ewin, int *px, int *py, int *pw, int *ph)
 {
    Container          *ct = (Container *) ewin->data;
+   int                 w, h;
 
+   w = ct->w;
+   h = ct->h;
    ContainerLayout(ct, px, py, pw, ph);
 
-   if (*pw != ct->w || *ph != ct->h)
+   if (*pw != w || *ph != h)
       ct->do_update = 1;
 }
 
@@ -275,6 +254,7 @@ _ContainerEwinMoveResize(EWin * ewin, int resize)
    ct->w = ewin->client.w;
    ct->h = ewin->client.h;
 
+   ContainerReconfigure(ct);
    ContainerDraw(ct);
    ct->do_update = 0;
 }
@@ -307,18 +287,14 @@ ContainerShow(Container * ct)
    if (!ewin)
       return;
 
-   ContainerReconfigure(ct);
-
    if (ewin->state.placed)
      {
-	EwinMoveToDesktop(ewin, EoGetDesk(ewin));
-	EwinMoveResize(ewin, EoGetX(ewin), EoGetY(ewin), ewin->client.w,
-		       ewin->client.h);
+	EwinMoveResize(ewin, EoGetX(ewin), EoGetY(ewin),
+		       ewin->client.w, ewin->client.h);
      }
    else
      {
 	/* The first one */
-	EwinMoveToDesktop(ewin, EoGetDesk(ewin));
 	EwinResize(ewin, 128, 32);
 	EwinMove(ewin, WinGetW(VROOT) - EoGetW(ewin),
 		 WinGetH(VROOT) - EoGetH(ewin));
@@ -516,22 +492,40 @@ ContainerLayoutImageWin(Container * ct)
      }
 
    if (ct->orientation)
-      ct->max = yo - item_pad;
+      ct->iwin_maxl = yo - item_pad;
    else
-      ct->max = xo - item_pad;
+      ct->iwin_maxl = xo - item_pad;
 
-   if (ct->max < ct->max_min)
-      ct->max = ct->max_min;
+   if (ct->orientation)
+     {
+	ct->iwin_fixh = ct->iconsize;
+	if (ct->ic_box)
+	   ct->iwin_fixh += pad->left + pad->right;
+	if (ct->draw_icon_base)
+	   ct->iwin_fixh += padl + padr;
+	ct->iwin_maxl_min = 8;
+     }
+   else
+     {
+	ct->iwin_fixh = ct->iconsize;
+	if (ct->ic_box)
+	   ct->iwin_fixh += pad->top + pad->bottom;
+	if (ct->draw_icon_base)
+	   ct->iwin_fixh += padt + padb;
+	ct->iwin_maxl_min = 8;
+     }
+
+   if (ct->iwin_maxl < ct->iwin_maxl_min)
+      ct->iwin_maxl = ct->iwin_maxl_min;
 }
 
 static void
-ContainerDrawScroll(Container * ct)
+ContainerLayoutScroll(Container * ct)
 {
    ImageClass         *ic, *ic_sbb;
    EImageBorder       *pad;
    int                 arrow_mode = ct->arrow_side;
    int                 bs, bw, bx;
-   int                 state;
 
    switch (ct->orientation)
      {
@@ -547,14 +541,14 @@ ContainerDrawScroll(Container * ct)
 	   bs = ct->h;
 	if (pad)
 	   bs -= pad->top + pad->bottom;
-	bw = (ct->h * bs) / ct->max;
+	bw = (ct->h * bs) / ct->iwin_maxl;
 	if (bs < 1)
 	   bs = 1;
 	if (bw > bs)
 	   bw = bs;
 	if (bw < 1)
 	   bw = 1;
-	bx = ((ct->pos * bs) / ct->max);
+	bx = (ct->pos * bs) / ct->iwin_maxl;
 	if (pad)
 	   bx += pad->top;
 	if ((ct->scrollbar_hide) && (bw == bs))
@@ -682,23 +676,10 @@ ContainerDrawScroll(Container * ct)
 	       }
 	  }
 
-	ImageclassApply(ic_sbb, ct->scroll_win, 0, 0, STATE_NORMAL, ST_ICONBOX);
-
 	EMoveResizeWindow(ct->scrollbar_win,
 			  (ct->scroll_thickness - ct->bar_thickness) / 2, bx,
 			  ct->bar_thickness, bw);
 
-	ic = ImageclassFind("ICONBOX_SCROLLBAR_KNOB_VERTICAL", 1);
-	if (ic)
-	  {
-	     state = STATE_NORMAL;
-	     if (ct->scrollbar_hilited)
-		state = STATE_HILITED;
-	     if (ct->scrollbar_clicked)
-		state = STATE_CLICKED;
-	     ImageclassApply(ic, ct->scrollbar_win, 0, 0, state, ST_ICONBOX);
-	  }
-
 	ic = ImageclassFind("ICONBOX_SCROLLKNOB_VERTICAL", 0);
 	if ((ic) && (bw > ct->knob_length))
 	  {
@@ -706,44 +687,12 @@ ContainerDrawScroll(Container * ct)
 	     EMoveResizeWindow(ct->scrollbarknob_win, 0,
 			       (bw - ct->knob_length) / 2, ct->bar_thickness,
 			       ct->knob_length);
-
-	     state = STATE_NORMAL;
-	     if (ct->scrollbar_hilited)
-		state = STATE_HILITED;
-	     if (ct->scrollbar_clicked)
-		state = STATE_CLICKED;
-	     ImageclassApply(ic, ct->scrollbarknob_win, 0, 0, state,
-			     ST_ICONBOX);
 	  }
 	else
 	  {
 	     EUnmapWindow(ct->scrollbarknob_win);
 	  }
-
-	if (arrow_mode < 3)
-	  {
-	     ic = ImageclassFind("ICONBOX_ARROW_UP", 1);
-	     if (ic)
-	       {
-		  state = STATE_NORMAL;
-		  if (ct->arrow1_hilited)
-		     state = STATE_HILITED;
-		  if (ct->arrow1_clicked)
-		     state = STATE_CLICKED;
-		  ImageclassApply(ic, ct->arrow1_win, 0, 0, state, ST_ICONBOX);
-	       }
-
-	     ic = ImageclassFind("ICONBOX_ARROW_DOWN", 1);
-	     if (ic)
-	       {
-		  state = STATE_NORMAL;
-		  if (ct->arrow2_hilited)
-		     state = STATE_HILITED;
-		  if (ct->arrow2_clicked)
-		     state = STATE_CLICKED;
-		  ImageclassApply(ic, ct->arrow2_win, 0, 0, state, ST_ICONBOX);
-	       }
-	  }
+	ct->scroll_thickness_set = ct->scroll_thickness;
 	break;
 
      case 0:
@@ -758,14 +707,14 @@ ContainerDrawScroll(Container * ct)
 	   bs = ct->w;
 	if (pad)
 	   bs -= pad->left + pad->right;
-	bw = (ct->w * bs) / ct->max;
+	bw = (ct->w * bs) / ct->iwin_maxl;
 	if (bs < 1)
 	   bs = 1;
 	if (bw > bs)
 	   bw = bs;
 	if (bw < 1)
 	   bw = 1;
-	bx = ((ct->pos * bs) / ct->max);
+	bx = (ct->pos * bs) / ct->iwin_maxl;
 	if (pad)
 	   bx += pad->left;
 	if ((ct->scrollbar_hide) && (bw == bs))
@@ -896,19 +845,6 @@ ContainerDrawScroll(Container * ct)
 			  (ct->scroll_thickness - ct->bar_thickness) / 2, bw,
 			  ct->bar_thickness);
 
-	ImageclassApply(ic_sbb, ct->scroll_win, 0, 0, STATE_NORMAL, ST_ICONBOX);
-
-	ic = ImageclassFind("ICONBOX_SCROLLBAR_KNOB_HORIZONTAL", 1);
-	if (ic)
-	  {
-	     state = STATE_NORMAL;
-	     if (ct->scrollbar_hilited)
-		state = STATE_HILITED;
-	     if (ct->scrollbar_clicked)
-		state = STATE_CLICKED;
-	     ImageclassApply(ic, ct->scrollbar_win, 0, 0, state, ST_ICONBOX);
-	  }
-
 	ic = ImageclassFind("ICONBOX_SCROLLKNOB_HORIZONTAL", 0);
 	if ((ic) && (bw > ct->knob_length))
 	  {
@@ -916,63 +852,84 @@ ContainerDrawScroll(Container * ct)
 	     EMoveResizeWindow(ct->scrollbarknob_win,
 			       (bw - ct->knob_length) / 2, 0, ct->knob_length,
 			       ct->bar_thickness);
-
-	     state = STATE_NORMAL;
-	     if (ct->scrollbar_hilited)
-		state = STATE_HILITED;
-	     if (ct->scrollbar_clicked)
-		state = STATE_CLICKED;
-	     ImageclassApply(ic, ct->scrollbarknob_win, 0, 0, state,
-			     ST_ICONBOX);
 	  }
 	else
 	  {
 	     EUnmapWindow(ct->scrollbarknob_win);
 	  }
-
-	if (arrow_mode < 3)
-	  {
-	     ic = ImageclassFind("ICONBOX_ARROW_LEFT", 1);
-	     if (ic)
-	       {
-		  state = STATE_NORMAL;
-		  if (ct->arrow1_hilited)
-		     state = STATE_HILITED;
-		  if (ct->arrow1_clicked)
-		     state = STATE_CLICKED;
-		  ImageclassApply(ic, ct->arrow1_win, 0, 0, state, ST_ICONBOX);
-	       }
-
-	     ic = ImageclassFind("ICONBOX_ARROW_RIGHT", 1);
-	     if (ic)
-	       {
-		  state = STATE_NORMAL;
-		  if (ct->arrow2_hilited)
-		     state = STATE_HILITED;
-		  if (ct->arrow2_clicked)
-		     state = STATE_CLICKED;
-		  ImageclassApply(ic, ct->arrow2_win, 0, 0, state, ST_ICONBOX);
-	       }
-	  }
+	ct->scroll_thickness_set = ct->scroll_thickness;
 	break;
 
       do_hide_sb:
 	EUnmapWindow(ct->scroll_win);
 	EUnmapWindow(ct->arrow1_win);
 	EUnmapWindow(ct->arrow2_win);
+	ct->scroll_thickness_set = 0;
 	break;
      }
 }
 
+static void
+ContainerDrawScroll(Container * ct)
+{
+   ImageClass         *ic_base, *ic_knob, *ic_snob, *ic_arr1, *ic_arr2;
+
+   ContainerLayoutScroll(ct);
+
+   if (!WinIsMapped(ct->scroll_win))
+      return;
+
+   ic_snob = ic_arr1 = ic_arr2 = NULL;
+   if (ct->orientation)
+     {
+	ic_base = ImageclassFind("ICONBOX_SCROLLBAR_BASE_VERTICAL", 1);
+	ic_knob = ImageclassFind("ICONBOX_SCROLLBAR_KNOB_VERTICAL", 1);
+	if (WinIsMapped(ct->scrollbarknob_win))
+	   ic_snob = ImageclassFind("ICONBOX_SCROLLKNOB_VERTICAL", 0);
+	if (WinIsMapped(ct->arrow1_win))
+	  {
+	     ic_arr1 = ImageclassFind("ICONBOX_ARROW_UP", 1);
+	     ic_arr2 = ImageclassFind("ICONBOX_ARROW_DOWN", 1);
+	  }
+     }
+   else
+     {
+	ic_base = ImageclassFind("ICONBOX_SCROLLBAR_BASE_HORIZONTAL", 1);
+	ic_knob = ImageclassFind("ICONBOX_SCROLLBAR_KNOB_HORIZONTAL", 1);
+	if (WinIsMapped(ct->scrollbarknob_win))
+	   ic_snob = ImageclassFind("ICONBOX_SCROLLKNOB_HORIZONTAL", 0);
+	if (WinIsMapped(ct->arrow1_win))
+	  {
+	     ic_arr1 = ImageclassFind("ICONBOX_ARROW_LEFT", 1);
+	     ic_arr2 = ImageclassFind("ICONBOX_ARROW_RIGHT", 1);
+	  }
+     }
+
+   ImageclassApply(ic_base, ct->scroll_win, 0, 0, STATE_NORMAL, ST_ICONBOX);
+   ImageclassApply(ic_knob, ct->scrollbar_win, 0, 0, ST(ct->scrollbar_state),
+		   ST_ICONBOX);
+   if (WinIsMapped(ct->scrollbarknob_win))
+      ImageclassApply(ic_snob, ct->scrollbarknob_win, 0, 0,
+		      ST(ct->scrollbar_state), ST_ICONBOX);
+
+   if (WinIsMapped(ct->arrow1_win))
+     {
+	ImageclassApply(ic_arr1, ct->arrow1_win, 0, 0, ST(ct->arrow1_state),
+			ST_ICONBOX);
+	ImageclassApply(ic_arr2, ct->arrow2_win, 0, 0, ST(ct->arrow2_state),
+			ST_ICONBOX);
+     }
+}
+
 static void
 ContainerFixPos(Container * ct)
 {
    int                 v;
 
    if (ct->orientation)
-      v = ct->max - ct->h;
+      v = ct->iwin_maxl - ct->h;
    else
-      v = ct->max - ct->w;
+      v = ct->iwin_maxl - ct->w;
 
    if (ct->pos > v)
       ct->pos = v;
@@ -990,53 +947,76 @@ ContainerLayout(Container * ct, int *px, int *py, int *pw, int *ph)
    y = *py;
    w = *pw;
    h = *ph;
-   ICCCM_SizeMatch(ewin, w, h, &w, &h);
 
-   ContainerLayoutImageWin(ct);
+   ContainerLayoutImageWin(ct);	/* Find iwin_maxl, iwin_fixh */
 
+   /* Possibly change "length" if autosizing */
    if (ct->auto_resize)
      {
-	int                 add = 0;
 	int                 bl, br, bt, bb;
 
-	EwinBorderGetSize(ct->ewin, &bl, &br, &bt, &bb);
+	EwinBorderGetSize(ewin, &bl, &br, &bt, &bb);
 
 	if (ct->orientation)
 	  {
-	     add = ct->max;
-	     if (ct->ewin->border)
+	     ct->h = h = ct->iwin_maxl;
+	     if (ewin->border)
 	       {
-		  if ((bt + bb + add) > WinGetH(VROOT))
-		     add = WinGetH(VROOT) - (bt + bb);
+		  if ((bt + bb + h) > WinGetH(VROOT))
+		     h = WinGetH(VROOT) - (bt + bb);
 	       }
-	     y += (((ct->ewin->client.h - add) * ct->auto_resize_anchor) >> 10);
-	     h = add;
-	     if (ct->ewin->border)
+	     y += (((ewin->client.h - h) * ct->auto_resize_anchor) >> 10);
+	     if (ewin->border)
 	       {
-		  if ((EoGetY(ct->ewin) + bt + bb + add) > WinGetH(VROOT))
-		     y = WinGetH(VROOT) - (bt + bb + add);
+		  if ((EoGetY(ewin) + bt + bb + h) > WinGetH(VROOT))
+		     y = WinGetH(VROOT) - (bt + bb + h);
 	       }
 	  }
 	else
 	  {
-	     add = ct->max;
-	     if (ct->ewin->border)
+	     ct->w = w = ct->iwin_maxl;
+	     if (ewin->border)
 	       {
-		  if ((bl + br + add) > WinGetW(VROOT))
-		     add = WinGetW(VROOT) - (bl + br);
+		  if ((bl + br + w) > WinGetW(VROOT))
+		     w = WinGetW(VROOT) - (bl + br);
 	       }
-	     x += (((ct->ewin->client.w - add) * ct->auto_resize_anchor) >> 10);
-	     w = add;
-	     if (ct->ewin->border)
+	     x += (((ewin->client.w - w) * ct->auto_resize_anchor) >> 10);
+	     if (ewin->border)
 	       {
-		  if ((EoGetX(ct->ewin) + bl + br + add) > WinGetW(VROOT))
-		     x = WinGetW(VROOT) - (bl + br + add);
+		  if ((EoGetX(ewin) + bl + br + w) > WinGetW(VROOT))
+		     x = WinGetW(VROOT) - (bl + br + w);
 	       }
 	  }
      }
+   else
+     {
+	if (ct->orientation)
+	  {
+	     if (h < ct->iwin_maxl_min)
+		h = ct->iwin_maxl_min;
+	     ct->h = h;
+	  }
+	else
+	  {
+	     if (w < ct->iwin_maxl_min)
+		w = ct->iwin_maxl_min;
+	     ct->w = w;
+	  }
+     }
 
+   ContainerLayoutScroll(ct);	/* Find scroll_thickness_set, uses h/w, iwin_maxl */
    ContainerFixPos(ct);
 
+   /* Possibly change "height" if hiding scrollbar */
+   if (ct->orientation)
+     {
+	ct->w = w = ct->iwin_fixh + ct->scroll_thickness_set;
+     }
+   else
+     {
+	ct->h = h = ct->iwin_fixh + ct->scroll_thickness_set;
+     }
+
    *px = x;
    *py = y;
    *pw = w;
@@ -1053,9 +1033,6 @@ ContainerDraw(Container * ct)
    EImage             *im;
    int                 ww, hh;
 
-   if (!ct->ic_box)
-      ContainerLayoutImageWin(ct);
-
    w = ct->w;
    h = ct->h;
 
@@ -1070,30 +1047,30 @@ ContainerDraw(Container * ct)
      {
 	ib_ic_cover = ImageclassFind("ICONBOX_COVER_VERTICAL", 0);
 	if (ct->scrollbar_side == 0)
-	   ib_xlt = ct->scroll_thickness;
-	ib_ww -= ct->scroll_thickness;
+	   ib_xlt = ct->scroll_thickness_set;
+	ib_ww -= ct->scroll_thickness_set;
 
 	/* Geometry of icon window (including invisible parts) */
 	ib_x0 = ib_xlt;
 	ib_y0 = ib_ylt - ct->pos;
 	ib_w0 = ib_ww;
 	ib_h0 = ib_hh;
-	if (ib_h0 < ct->max)
-	   ib_h0 = ct->max;
+	if (ib_h0 < ct->iwin_maxl)
+	   ib_h0 = ct->iwin_maxl;
      }
    else
      {
 	ib_ic_cover = ImageclassFind("ICONBOX_COVER_HORIZONTAL", 0);
 	if (ct->scrollbar_side == 0)
-	   ib_ylt = ct->scroll_thickness;
-	ib_hh -= ct->scroll_thickness;
+	   ib_ylt = ct->scroll_thickness_set;
+	ib_hh -= ct->scroll_thickness_set;
 
 	/* Geometry of icon window (including invisible parts) */
 	ib_x0 = ib_xlt - ct->pos;
 	ib_y0 = ib_ylt;
 	ib_w0 = ib_ww;
-	if (ib_w0 < ct->max)
-	   ib_w0 = ct->max;
+	if (ib_w0 < ct->iwin_maxl)
+	   ib_w0 = ct->iwin_maxl;
 	ib_h0 = ib_hh;
      }
 
@@ -1128,7 +1105,7 @@ ContainerDraw(Container * ct)
 	/* Start out with blank image */
 	im = EImageCreate(ib_w0, ib_h0);
 	EImageSetHasAlpha(im, 1);
-	EImageFill(im, 0, 0, ib_w0, ib_h0, 0, 0, 0, 0);
+	EImageFill(im, 0, 0, ib_w0, ib_h0, 0);
      }
 
    for (i = 0; i < ct->num_objs; i++)
@@ -1192,32 +1169,33 @@ ContainerScroll(Container * ct, int dir)
 }
 
 static void
-ContainerShowMenu(Container * ct, int x __UNUSED__, int y __UNUSED__)
+ContainerShowMenu(Container * ct)
 {
-   static Menu        *p_menu = NULL;
+   Menu               *m;
    MenuItem           *mi;
    char                s[1024];
 
-   if (p_menu)
-      MenuDestroy(p_menu);
+   m = MenuCreate("__ct", ct->menu_title, NULL, NULL);
+   if (!m)
+      return;
 
-   p_menu = MenuCreate("__CT_MENU", ct->menu_title, NULL, NULL);
+   MenuSetTransient(m);		/* Destroy when hidden */
 
    Esnprintf(s, sizeof(s), "ibox cfg %s", ct->name);
    mi = MenuItemCreate(_("Settings..."), NULL, s, NULL);
-   MenuAddItem(p_menu, mi);
+   MenuAddItem(m, mi);
 
    Esnprintf(s, sizeof(s), "wop %#lx cl", WinGetXwin(ct->win));
    mi = MenuItemCreate(_("Close"), NULL, s, NULL);
-   MenuAddItem(p_menu, mi);
+   MenuAddItem(m, mi);
 
    if (ct->type == IB_TYPE_ICONBOX)
      {
 	mi = MenuItemCreate(_("Create New Iconbox"), NULL, "ibox new", NULL);
-	MenuAddItem(p_menu, mi);
+	MenuAddItem(m, mi);
      }
 
-   EFunc(NULL, "menus show __CT_MENU");
+   EFunc(NULL, "menus show __ct");
 }
 
 static void
@@ -1257,7 +1235,7 @@ ContainerEventScrollWin(Win win __UNUSED__, XEvent * ev, void *prm)
 	if (ev->xbutton.button == 1)
 	   ct->scrollbox_clicked = 1;
 	else if (ev->xbutton.button == 3)
-	   ContainerShowMenu(ct, ev->xbutton.x, ev->xbutton.y);
+	   ContainerShowMenu(ct);
 	break;
 
      case ButtonRelease:
@@ -1300,27 +1278,26 @@ ContainerEventScrollbarWin(Win win __UNUSED__, XEvent * ev, void *prm)
 	     px = ev->xbutton.x_root;
 	     py = ev->xbutton.y_root;
 	     pos0 = ct->pos;
-	     ct->scrollbar_clicked = 1;
+	     ct->scrollbar_state |= WS_CLICK;
 	  }
 	else if (ev->xbutton.button == 3)
-	   ContainerShowMenu(ct, ev->xbutton.x, ev->xbutton.y);
-	break;
+	   ContainerShowMenu(ct);
+	goto draw_scoll;
 
      case ButtonRelease:
-	if (ct->scrollbar_clicked)
-	   ct->scrollbar_clicked = 0;
-	break;
+	ct->scrollbar_state &= ~WS_CLICK;
+	goto draw_scoll;
 
      case EnterNotify:
-	ct->scrollbar_hilited = 1;
-	break;
+	ct->scrollbar_state |= WS_HILITE;
+	goto draw_scoll;
 
      case LeaveNotify:
-	ct->scrollbar_hilited = 0;
-	break;
+	ct->scrollbar_state &= ~WS_HILITE;
+	goto draw_scoll;
 
      case MotionNotify:
-	if (!ct->scrollbar_clicked)
+	if (!(ct->scrollbar_state & WS_CLICK))
 	   break;
 
 	if (ct->orientation)
@@ -1345,12 +1322,15 @@ ContainerEventScrollbarWin(Win win __UNUSED__, XEvent * ev, void *prm)
 		bs = 1;
 	     dp = ev->xmotion.x_root - px;
 	  }
-	dp = pos0 + (dp * ct->max) / bs - ct->pos;
+	dp = pos0 + (dp * ct->iwin_maxl) / bs - ct->pos;
 	if (dp)
 	   ContainerScroll(ct, dp);
-	return;
+	break;
+
+      draw_scoll:
+	ContainerDrawScroll(ct);
+	break;
      }
-   ContainerDrawScroll(ct);
 }
 
 static void
@@ -1361,7 +1341,7 @@ ContainerEventCoverWin(Win win __UNUSED__, XEvent * ev, void *prm)
    switch (ev->type)
      {
      case ButtonPress:
-	ContainerShowMenu(ct, ev->xbutton.x, ev->xbutton.y);
+	ContainerShowMenu(ct);
 	break;
      case ButtonRelease:
 	break;
@@ -1377,28 +1357,31 @@ ContainerEventArrow1Win(Win win __UNUSED__, XEvent * ev, void *prm)
      {
      case ButtonPress:
 	if (ev->xbutton.button == 1)
-	   ct->arrow1_clicked = 1;
+	   ct->arrow1_state |= WS_CLICK;
 	else if (ev->xbutton.button == 3)
-	   ContainerShowMenu(ct, ev->xbutton.x, ev->xbutton.y);
-	break;
+	   ContainerShowMenu(ct);
+	goto draw_scoll;
 
      case ButtonRelease:
-	if (!ct->arrow1_clicked)
-	   break;
-	ct->arrow1_clicked = 0;
+	if (!(ct->arrow1_state & WS_CLICK))
+	   goto draw_scoll;
+	ct->arrow1_state &= ~WS_CLICK;
 	if (ContainerScroll(ct, -8))
 	   return;
-	break;
+	goto draw_scoll;
 
      case EnterNotify:
-	ct->arrow1_hilited = 1;
-	break;
+	ct->arrow1_state |= WS_HILITE;
+	goto draw_scoll;
 
      case LeaveNotify:
-	ct->arrow1_hilited = 0;
+	ct->arrow1_state &= ~WS_HILITE;
+	goto draw_scoll;
+
+      draw_scoll:
+	ContainerDrawScroll(ct);
 	break;
      }
-   ContainerDrawScroll(ct);
 }
 
 static void
@@ -1410,28 +1393,31 @@ ContainerEventArrow2Win(Win win __UNUSED__, XEvent * ev, void *prm)
      {
      case ButtonPress:
 	if (ev->xbutton.button == 1)
-	   ct->arrow2_clicked = 1;
+	   ct->arrow2_state |= WS_CLICK;
 	else if (ev->xbutton.button == 3)
-	   ContainerShowMenu(ct, ev->xbutton.x, ev->xbutton.y);
-	break;
+	   ContainerShowMenu(ct);
+	goto draw_scoll;
 
      case ButtonRelease:
-	if (!ct->arrow2_clicked)
-	   break;
-	ct->arrow2_clicked = 0;
+	if (!(ct->arrow2_state & WS_CLICK))
+	   goto draw_scoll;
+	ct->arrow2_state &= ~WS_CLICK;
 	if (ContainerScroll(ct, 8))
 	   return;
-	break;
+	goto draw_scoll;
 
      case EnterNotify:
-	ct->arrow2_hilited = 1;
-	break;
+	ct->arrow2_state |= WS_HILITE;
+	goto draw_scoll;
 
      case LeaveNotify:
-	ct->arrow2_hilited = 0;
+	ct->arrow2_state &= ~WS_HILITE;
+	goto draw_scoll;
+
+      draw_scoll:
+	ContainerDrawScroll(ct);
 	break;
      }
-   ContainerDrawScroll(ct);
 }
 
 static void
@@ -1444,7 +1430,7 @@ ContainerEventIconWin(Win win __UNUSED__, XEvent * ev, void *prm)
      case ButtonPress:
 	if (ev->xbutton.button != 3)
 	   break;
-	ContainerShowMenu(ct, ev->xbutton.x, ev->xbutton.y);
+	ContainerShowMenu(ct);
 	return;
      }
 
@@ -1456,91 +1442,100 @@ ContainerEventIconWin(Win win __UNUSED__, XEvent * ev, void *prm)
 /*
  * Configuration dialog
  */
-static char        *tmp_ib_name = NULL;
-static char         tmp_ib_nobg;
-static char         tmp_ib_shownames;
-static int          tmp_ib_vert;
-static int          tmp_ib_side;
-static int          tmp_ib_arrows;
-static int          tmp_ib_iconsize;
-static int          tmp_ib_mode;
-static char         tmp_ib_auto_resize;
-static char         tmp_ib_draw_icon_base;
-static char         tmp_ib_scrollbar_hide;
-static char         tmp_ib_cover_hide;
-static int          tmp_ib_autoresize_anchor;
-static char         tmp_ib_anim_mode;
+
+typedef struct {
+   Container          *ct;
+   char                nobg;
+   char                shownames;
+   int                 vert;
+   int                 side;
+   int                 arrows;
+   int                 iconsize;
+   int                 icon_mode;
+   char                auto_resize;
+   char                draw_icon_base;
+   char                scrollbar_hide;
+   char                cover_hide;
+   int                 autoresize_anchor;
+   int                 anim_mode;
+   int                 anim_speed;
+} ContainerDlgData;
 
 static void
-CB_ConfigureContainer(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
+CB_ConfigureContainer(Dialog * d, int val, void *data __UNUSED__)
 {
-   if (val < 2)
-     {
-	Container          *ct;
+   ContainerDlgData   *dd = DLG_DATA_GET(d, ContainerDlgData);
+   Container          *ct;
 
-	if (!tmp_ib_name)
-	   return;
-	ct = ContainerFind(tmp_ib_name);
-	if (!ct)
-	   return;
+   if (val >= 2)
+      return;
 
-	ct->nobg = tmp_ib_nobg;
-	ct->shownames = tmp_ib_shownames;
-	ct->orientation = tmp_ib_vert;
-	ct->scrollbar_side = tmp_ib_side;
-	ct->arrow_side = tmp_ib_arrows;
-	ct->iconsize = tmp_ib_iconsize;
-	ct->icon_mode = tmp_ib_mode;
-	ct->auto_resize = tmp_ib_auto_resize;
-	ct->draw_icon_base = tmp_ib_draw_icon_base;
-	ct->scrollbar_hide = tmp_ib_scrollbar_hide;
-	ct->cover_hide = tmp_ib_cover_hide;
-	ct->auto_resize_anchor = tmp_ib_autoresize_anchor;
-	ct->anim_mode = tmp_ib_anim_mode;
+   ct = (Container *) ecore_list_goto(container_list, dd->ct);
+   if (!ct)
+      return;
 
-	ContainerReconfigure(ct);
-	ContainerRedraw(ct);
-	ContainersConfigSave();
-     }
+   ct->nobg = dd->nobg;
+   ct->shownames = dd->shownames;
+   ct->orientation = dd->vert;
+   ct->scrollbar_side = dd->side;
+   ct->arrow_side = dd->arrows;
+   ct->iconsize = dd->iconsize;
+   ct->icon_mode = dd->icon_mode;
+   ct->auto_resize = dd->auto_resize;
+   ct->draw_icon_base = dd->draw_icon_base;
+   ct->scrollbar_hide = dd->scrollbar_hide;
+   ct->cover_hide = dd->cover_hide;
+   ct->auto_resize_anchor = dd->autoresize_anchor;
+   ct->anim_mode = dd->anim_mode;
+   Conf_containers.anim_time = 1000 - dd->anim_speed;
+
+   ContainerRedraw(ct);
+
+   ContainersConfigSave();
+   autosave();
 }
 
 static void
 CB_IconSizeSlider(Dialog * d, int val __UNUSED__, void *data)
 {
+   ContainerDlgData   *dd = DLG_DATA_GET(d, ContainerDlgData);
    DItem              *di = (DItem *) data;
    char                s[256];
 
-   Esnprintf(s, sizeof(s), _("Icon size: %2d"), tmp_ib_iconsize);
+   Esnprintf(s, sizeof(s), _("Icon size: %2d"), dd->iconsize);
    DialogItemSetText(di, s);
-   DialogDrawItems(d, di, 0, 0, 99999, 99999);
 }
 
 static void
 _DlgFillContainer(Dialog * d, DItem * table, void *data)
 {
+   ContainerDlgData   *dd;
    Container          *ct = (Container *) data;
-   DItem              *di, *table2;
-   DItem              *radio1, *radio2, *radio3, *radio4, *label;
+   DItem              *di, *table2, *radio, *label;
    char                s[256];
 
+   dd = DLG_DATA_SET(d, ContainerDlgData);
+   if (!dd)
+      return;
+
    if (!ct)
       return;
 
-   tmp_ib_nobg = ct->nobg;
-   tmp_ib_shownames = ct->shownames;
-   tmp_ib_vert = ct->orientation;
-   tmp_ib_side = ct->scrollbar_side;
-   tmp_ib_arrows = ct->arrow_side;
-   tmp_ib_iconsize = ct->iconsize;
-   tmp_ib_mode = ct->icon_mode;
-   tmp_ib_auto_resize = ct->auto_resize;
-   tmp_ib_draw_icon_base = ct->draw_icon_base;
-   tmp_ib_scrollbar_hide = ct->scrollbar_hide;
-   tmp_ib_cover_hide = ct->cover_hide;
-   tmp_ib_autoresize_anchor = ct->auto_resize_anchor;
-   tmp_ib_anim_mode = ct->anim_mode;
-   Efree(tmp_ib_name);
-   tmp_ib_name = Estrdup(ct->name);
+   dd->ct = ct;
+   dd->nobg = ct->nobg;
+   dd->shownames = ct->shownames;
+   dd->vert = ct->orientation;
+   dd->side = ct->scrollbar_side;
+   dd->arrows = ct->arrow_side;
+   dd->iconsize = ct->iconsize;
+   dd->icon_mode = ct->icon_mode;
+   dd->auto_resize = ct->auto_resize;
+   dd->draw_icon_base = ct->draw_icon_base;
+   dd->scrollbar_hide = ct->scrollbar_hide;
+   dd->cover_hide = ct->cover_hide;
+   dd->autoresize_anchor = ct->auto_resize_anchor;
+   dd->anim_mode = ct->anim_mode;
+   dd->anim_speed = 1000 - Conf_containers.anim_time;
 
    DialogSetTitle(d, ct->dlg_title);
 
@@ -1548,23 +1543,23 @@ _DlgFillContainer(Dialog * d, DItem * table, void *data)
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetText(di, _("Transparent background"));
-   DialogItemCheckButtonSetPtr(di, &tmp_ib_nobg);
+   DialogItemCheckButtonSetPtr(di, &dd->nobg);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetText(di, _("Hide inner border"));
-   DialogItemCheckButtonSetPtr(di, &tmp_ib_cover_hide);
+   DialogItemCheckButtonSetPtr(di, &dd->cover_hide);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetText(di, _("Draw base image behind Icons"));
-   DialogItemCheckButtonSetPtr(di, &tmp_ib_draw_icon_base);
+   DialogItemCheckButtonSetPtr(di, &dd->draw_icon_base);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetText(di, _("Hide scrollbar when not needed"));
-   DialogItemCheckButtonSetPtr(di, &tmp_ib_scrollbar_hide);
+   DialogItemCheckButtonSetPtr(di, &dd->scrollbar_hide);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetText(di, _("Automatically resize to fit Icons"));
-   DialogItemCheckButtonSetPtr(di, &tmp_ib_auto_resize);
+   DialogItemCheckButtonSetPtr(di, &dd->auto_resize);
 
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetFill(di, 0, 0);
@@ -1576,91 +1571,91 @@ _DlgFillContainer(Dialog * d, DItem * table, void *data)
    DialogItemSliderSetBounds(di, 0, 1024);
    DialogItemSliderSetUnits(di, 1);
    DialogItemSliderSetJump(di, 8);
-   DialogItemSliderSetValPtr(di, &tmp_ib_autoresize_anchor);
+   DialogItemSliderSetValPtr(di, &dd->autoresize_anchor);
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
 
    label = di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetFill(di, 0, 0);
    DialogItemSetAlign(di, 0, 512);
-   Esnprintf(s, sizeof(s), _("Icon size: %2d"), tmp_ib_iconsize);
+   Esnprintf(s, sizeof(s), _("Icon size: %2d"), dd->iconsize);
    DialogItemSetText(di, s);
 
    di = DialogAddItem(table, DITEM_SLIDER);
    DialogItemSliderSetBounds(di, 4, 128);
    DialogItemSliderSetUnits(di, 1);
    DialogItemSliderSetJump(di, 8);
-   DialogItemSliderSetValPtr(di, &tmp_ib_iconsize);
+   DialogItemSliderSetValPtr(di, &dd->iconsize);
    DialogItemSetCallback(di, CB_IconSizeSlider, 0, label);
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
 
    table2 = DialogAddItem(table, DITEM_TABLE);
-   DialogItemTableSetOptions(table2, 3, 0, 0, 0);
+   DialogItemTableSetOptions(table2, 5, 0, 0, 0);
 
    di = DialogAddItem(table2, DITEM_TEXT);
    DialogItemSetFill(di, 0, 0);
    DialogItemSetAlign(di, 0, 512);
    DialogItemSetText(di, _("Orientation:"));
 
+   radio = di = DialogAddItem(table2, DITEM_RADIOBUTTON);
+   DialogItemSetText(di, _("Horizontal"));
+   DialogItemRadioButtonSetFirst(di, radio);
+   DialogItemRadioButtonGroupSetVal(di, 0);
+
+   di = DialogAddItem(table2, DITEM_RADIOBUTTON);
+   DialogItemSetText(di, _("Vertical"));
+   DialogItemRadioButtonSetFirst(di, radio);
+   DialogItemRadioButtonGroupSetVal(di, 1);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->vert);
+
+   di = DialogAddItem(table2, DITEM_NONE);
+   di = DialogAddItem(table2, DITEM_NONE);
+
    di = DialogAddItem(table2, DITEM_TEXT);
    DialogItemSetFill(di, 0, 0);
    DialogItemSetAlign(di, 0, 512);
    DialogItemSetText(di, _("Scrollbar side:"));
 
+   radio = di = DialogAddItem(table2, DITEM_RADIOBUTTON);
+   DialogItemSetText(di, _("Left / Top"));
+   DialogItemRadioButtonSetFirst(di, radio);
+   DialogItemRadioButtonGroupSetVal(di, 0);
+
+   di = DialogAddItem(table2, DITEM_RADIOBUTTON);
+   DialogItemSetText(di, _("Right / Bottom"));
+   DialogItemRadioButtonSetFirst(di, radio);
+   DialogItemRadioButtonGroupSetVal(di, 1);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->side);
+
+   di = DialogAddItem(table2, DITEM_NONE);
+   di = DialogAddItem(table2, DITEM_NONE);
+
    di = DialogAddItem(table2, DITEM_TEXT);
    DialogItemSetFill(di, 0, 0);
    DialogItemSetAlign(di, 0, 512);
    DialogItemSetText(di, _("Scrollbar arrows:"));
 
-   radio1 = di = DialogAddItem(table2, DITEM_RADIOBUTTON);
-   DialogItemSetText(di, _("Horizontal"));
-   DialogItemRadioButtonSetFirst(di, radio1);
-   DialogItemRadioButtonGroupSetVal(di, 0);
-
-   radio2 = di = DialogAddItem(table2, DITEM_RADIOBUTTON);
-   DialogItemSetText(di, _("Left / Top"));
-   DialogItemRadioButtonSetFirst(di, radio2);
-   DialogItemRadioButtonGroupSetVal(di, 0);
-
-   radio3 = di = DialogAddItem(table2, DITEM_RADIOBUTTON);
+   radio = di = DialogAddItem(table2, DITEM_RADIOBUTTON);
    DialogItemSetText(di, _("Start"));
-   DialogItemRadioButtonSetFirst(di, radio3);
+   DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 0);
 
-   di = DialogAddItem(table2, DITEM_RADIOBUTTON);
-   DialogItemSetText(di, _("Vertical"));
-   DialogItemRadioButtonSetFirst(di, radio1);
-   DialogItemRadioButtonGroupSetVal(di, 1);
-   DialogItemRadioButtonGroupSetValPtr(radio1, &tmp_ib_vert);
-
-   di = DialogAddItem(table2, DITEM_RADIOBUTTON);
-   DialogItemSetText(di, _("Right / Bottom"));
-   DialogItemRadioButtonSetFirst(di, radio2);
-   DialogItemRadioButtonGroupSetVal(di, 1);
-   DialogItemRadioButtonGroupSetValPtr(radio2, &tmp_ib_side);
-
    di = DialogAddItem(table2, DITEM_RADIOBUTTON);
    DialogItemSetText(di, _("Both ends"));
-   DialogItemRadioButtonSetFirst(di, radio3);
+   DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 1);
 
-   di = DialogAddItem(table2, DITEM_NONE);
-   di = DialogAddItem(table2, DITEM_NONE);
-
    di = DialogAddItem(table2, DITEM_RADIOBUTTON);
    DialogItemSetText(di, _("End"));
-   DialogItemRadioButtonSetFirst(di, radio3);
+   DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 2);
 
-   di = DialogAddItem(table2, DITEM_NONE);
-   di = DialogAddItem(table2, DITEM_NONE);
-
    di = DialogAddItem(table2, DITEM_RADIOBUTTON);
    DialogItemSetText(di, _("None"));
-   DialogItemRadioButtonSetFirst(di, radio3);
+   DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 3);
-   DialogItemRadioButtonGroupSetValPtr(radio3, &tmp_ib_arrows);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->arrows);
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
 
@@ -1668,11 +1663,42 @@ _DlgFillContainer(Dialog * d, DItem * table, void *data)
      {
 	di = DialogAddItem(table, DITEM_CHECKBUTTON);
 	DialogItemSetText(di, _("Show icon names"));
-	DialogItemCheckButtonSetPtr(di, &tmp_ib_shownames);
+	DialogItemCheckButtonSetPtr(di, &dd->shownames);
 
-	di = DialogAddItem(table, DITEM_CHECKBUTTON);
-	DialogItemSetText(di, _("Animate when iconifying to this Iconbox"));
-	DialogItemCheckButtonSetPtr(di, &tmp_ib_anim_mode);
+	table2 = DialogAddItem(table, DITEM_TABLE);
+	DialogItemTableSetOptions(table2, 4, 0, 1, 0);
+
+	di = DialogAddItem(table2, DITEM_TEXT);
+	DialogItemSetFill(di, 0, 0);
+	DialogItemSetAlign(di, 0, 512);
+	DialogItemSetText(di, _("Animation mode:"));
+
+	radio = di = DialogAddItem(table2, DITEM_RADIOBUTTON);
+	DialogItemSetText(di, _("Off"));
+	DialogItemRadioButtonSetFirst(di, radio);
+	DialogItemRadioButtonGroupSetVal(di, 0);
+
+	di = DialogAddItem(table2, DITEM_RADIOBUTTON);
+	DialogItemSetText(di, _("Whirl"));
+	DialogItemRadioButtonSetFirst(di, radio);
+	DialogItemRadioButtonGroupSetVal(di, 1);
+
+	di = DialogAddItem(table2, DITEM_RADIOBUTTON);
+	DialogItemSetText(di, _("Box"));
+	DialogItemRadioButtonSetFirst(di, radio);
+	DialogItemRadioButtonGroupSetVal(di, 2);
+	DialogItemRadioButtonGroupSetValPtr(radio, &dd->anim_mode);
+
+	di = DialogAddItem(table, DITEM_TEXT);
+	DialogItemSetFill(di, 0, 0);
+	DialogItemSetAlign(di, 0, 512);
+	DialogItemSetText(di, _("Animation speed:"));
+
+	di = DialogAddItem(table, DITEM_SLIDER);
+	DialogItemSliderSetBounds(di, 0, 1000);
+	DialogItemSliderSetUnits(di, 10);
+	DialogItemSliderSetJump(di, 50);
+	DialogItemSliderSetValPtr(di, &dd->anim_speed);
 
 	di = DialogAddItem(table, DITEM_SEPARATOR);
 
@@ -1683,25 +1709,25 @@ _DlgFillContainer(Dialog * d, DItem * table, void *data)
 			  _
 			  ("Icon image display policy (if one operation fails, try the next):"));
 
-	radio4 = di = DialogAddItem(table, DITEM_RADIOBUTTON);
+	radio = di = DialogAddItem(table, DITEM_RADIOBUTTON);
 	DialogItemSetText(di,
 			  _
 			  ("Snapshot Windows, Use application icon, Use Enlightenment Icon"));
-	DialogItemRadioButtonSetFirst(di, radio4);
+	DialogItemRadioButtonSetFirst(di, radio);
 	DialogItemRadioButtonGroupSetVal(di, 0);
 
 	di = DialogAddItem(table, DITEM_RADIOBUTTON);
 	DialogItemSetText(di,
 			  _
 			  ("Use application icon, Use Enlightenment Icon, Snapshot Window"));
-	DialogItemRadioButtonSetFirst(di, radio4);
+	DialogItemRadioButtonSetFirst(di, radio);
 	DialogItemRadioButtonGroupSetVal(di, 1);
 
 	di = DialogAddItem(table, DITEM_RADIOBUTTON);
 	DialogItemSetText(di, _("Use Enlightenment Icon, Snapshot Window"));
-	DialogItemRadioButtonSetFirst(di, radio4);
+	DialogItemRadioButtonSetFirst(di, radio);
 	DialogItemRadioButtonGroupSetVal(di, 2);
-	DialogItemRadioButtonGroupSetValPtr(radio4, &tmp_ib_mode);
+	DialogItemRadioButtonGroupSetValPtr(radio, &dd->icon_mode);
      }
 }
 
@@ -1772,7 +1798,7 @@ ContainersConfigLoad(void)
 	  case CONFIG_ICON_SIZE:	/* __ICON_SIZE %i */
 	     ct->iconsize = i2;
 	     break;
-	  case CONFIG_ICON_MODE:	/* __ICON_MODE [ 0 | 1 | 2 | 3 | 4 ] */
+	  case CONFIG_ICON_MODE:	/* __ICON_MODE [ 0 | 1 | 2 ] */
 	     ct->icon_mode = i2;
 	     break;
 	  case CONFIG_SCROLLBAR_SIDE:	/* __SCROLLBAR_SIDE [ __BAR_LEFT/__BAR_TOP | __BAR_RIGHT/__BAR_BOTTOM ] */
