@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2006-2007 Kim Woelders
+ * Copyright (C) 2006-2009 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -27,19 +27,18 @@
 #include "etypes.h"
 
 /* cursors.c */
-#define ECSR_NONE           0
-#define ECSR_ROOT           1
-#define ECSR_GRAB           2
-#define ECSR_PGRAB          3
-#define ECSR_ACT_MOVE       4
-#define ECSR_ACT_RESIZE     5
-#define ECSR_ACT_RESIZE_H   6
-#define ECSR_ACT_RESIZE_V   7
-#define ECSR_ACT_RESIZE_TL  8
-#define ECSR_ACT_RESIZE_TR  9
-#define ECSR_ACT_RESIZE_BL 10
-#define ECSR_ACT_RESIZE_BR 11
-#define ECSR_COUNT         12
+#define ECSR_ROOT           0
+#define ECSR_GRAB           1
+#define ECSR_PGRAB          2
+#define ECSR_ACT_MOVE       3
+#define ECSR_ACT_RESIZE     4
+#define ECSR_ACT_RESIZE_H   5
+#define ECSR_ACT_RESIZE_V   6
+#define ECSR_ACT_RESIZE_TL  7
+#define ECSR_ACT_RESIZE_TR  8
+#define ECSR_ACT_RESIZE_BL  9
+#define ECSR_ACT_RESIZE_BR 10
+#define ECSR_COUNT         11
 
 ECursor            *ECursorAlloc(const char *name);
 void                ECursorFree(ECursor * ec);
@@ -47,4 +46,6 @@ void                ECursorApply(ECursor * ec, Win win);
 Cursor              ECsrGet(int which);
 void                ECsrApply(int which, Window win);
 
+int                 ECursorConfigLoad(FILE * fs);
+
 #endif /* _CURSORS_H_ */
