@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -386,7 +386,7 @@ DeskCreate(int desk, int configure)
    EoSetNoRedirect(dsk, 1);
    EoInit(dsk, EOBJ_TYPE_DESK, win,
 	  0, 0, WinGetW(VROOT), WinGetH(VROOT), 0, buf);
-   EventCallbackRegister(EoGetWin(dsk), 0, DeskHandleEvents, dsk);
+   EventCallbackRegister(EoGetWin(dsk), DeskHandleEvents, dsk);
    dsk->bg.o = EoObj(dsk);
    if (desk == 0)
      {
@@ -399,7 +399,7 @@ DeskCreate(int desk, int configure)
 			      0, "Root-bg");
 	eo->floating = 0;
 	EobjSetLayer(eo, 0);
-	EventCallbackRegister(EobjGetWin(eo), 0, DeskHandleEvents, dsk);
+	EventCallbackRegister(EobjGetWin(eo), DeskHandleEvents, dsk);
 	dsk->bg.o_bg = eo;
 	ESelectInput(EobjGetWin(eo), EnterWindowMask);
 #endif
@@ -423,7 +423,7 @@ DeskCreate(int desk, int configure)
    EobjReparent(eo, EoObj(dsk), 0, 0);
    EobjSetLayer(eo, 0);
    dsk->bg.o_bg = eo;
-   EventCallbackRegister(EobjGetWin(eo), 0, DeskHandleEvents, dsk);
+   EventCallbackRegister(EobjGetWin(eo), DeskHandleEvents, dsk);
 #endif
 #endif
 
@@ -448,12 +448,12 @@ DeskDestroy(Desk * dsk)
 #if USE_COMPOSITE
    if (dsk->bg.o_bg)
      {
-	EventCallbackUnregister(EobjGetWin(dsk->bg.o_bg), 0, DeskHandleEvents,
+	EventCallbackUnregister(EobjGetWin(dsk->bg.o_bg), DeskHandleEvents,
 				dsk);
 	EobjWindowDestroy(dsk->bg.o_bg);
      }
 #endif
-   EventCallbackUnregister(EoGetWin(dsk), 0, DeskHandleEvents, dsk);
+   EventCallbackUnregister(EoGetWin(dsk), DeskHandleEvents, dsk);
 
    DeskControlsDestroy(dsk, 1);
    DeskControlsDestroy(dsk, 2);
@@ -521,9 +521,10 @@ DeskBackgroundConfigure(Desk * dsk)
 
    if (dsk->viewable || !dsk->bg.bg)
      {
-#if !USE_BG_WIN_ON_ALL_DESKS
-	if (ECompMgrDeskConfigure(dsk))
+#if USE_COMPOSITE && !USE_BG_WIN_ON_ALL_DESKS
+	if (ECompMgrIsActive())
 	  {
+	     ECompMgrDeskConfigure(dsk);
 	     ESetWindowBackgroundPixmap(win, None);
 	  }
 	else
@@ -717,6 +718,15 @@ DeskGet(unsigned int desk)
    return _DeskGet(desk);
 }
 
+Desk               *
+DeskGetValid(unsigned int desk)
+{
+   if (desk >= Conf.desks.num)
+      desk = 0;
+
+   return _DeskGet(desk);
+}
+
 Desk               *
 DeskGetRelative(Desk * dsk, int inc)
 {
@@ -846,7 +856,7 @@ ChangeNumberOfDesktops(unsigned int quantity)
 	for (j = 0; j < num; j++)
 	  {
 	     if (EoGetDeskNum(lst[j]) >= quantity)
-		EwinMoveToDesktop(lst[j], _DeskGet(quantity - 1));
+		EwinMoveToDesktop(lst[j], _DeskGet(0));
 	  }
 
 	while (Conf.desks.num > quantity)
@@ -1173,7 +1183,7 @@ DeskGoto(Desk * dsk)
 
    ModulesSignal(ESIGNAL_DESK_SWITCH_START, NULL);
 
-   ActionsSuspend();
+   MoveResizeSuspend();
    DeskSwitchStart();
 
    if (dsk->num > 0)
@@ -1227,7 +1237,7 @@ DeskGoto(Desk * dsk)
      }
 
    DeskSwitchDone();
-   ActionsResume();
+   MoveResizeResume();
 
    ModulesSignal(ESIGNAL_DESK_SWITCH_DONE, NULL);
 
@@ -1340,10 +1350,6 @@ DeskRestackSimple(Desk * dsk)
    unsigned int        value_mask;
 
    eo = dsk->stack.latest;
-   eo->stacked = 1;
-
-   if (EDebug(EDBUG_TYPE_STACKING))
-      Eprintf("DeskRestackSimple %#lx %s\n", EobjGetXwin(eo), EobjGetName(eo));
 
    lst = EobjListStackGetForDesk(&num, dsk);
    if (num < 2)
@@ -1355,6 +1361,11 @@ DeskRestackSimple(Desk * dsk)
    if (i >= num)
       return;
 
+   eo->stacked = 1;
+
+   if (EDebug(EDBUG_TYPE_STACKING))
+      Eprintf("DeskRestackSimple %#lx %s\n", EobjGetXwin(eo), EobjGetName(eo));
+
    if (i < num - 1)
      {
 	xwc.stack_mode = Above;
@@ -1443,8 +1454,6 @@ DeskGotoByEwin(EWin * ewin)
 /*
  * Areas
  */
-static int          area_w = 3;
-static int          area_h = 3;
 
 void
 DesksFixArea(int *ax, int *ay)
@@ -1452,31 +1461,31 @@ DesksFixArea(int *ax, int *ay)
    if (*ax < 0)
      {
 	if (Conf.desks.areas_wraparound)
-	   *ax = area_w - 1;
+	   *ax = Conf.desks.areas_nx - 1;
 	else
 	   *ax = 0;
      }
-   else if (*ax >= area_w)
+   else if (*ax >= Conf.desks.areas_nx)
      {
 	if (Conf.desks.areas_wraparound)
 	   *ax = 0;
 	else
-	   *ax = area_w - 1;
+	   *ax = Conf.desks.areas_nx - 1;
      }
 
    if (*ay < 0)
      {
 	if (Conf.desks.areas_wraparound)
-	   *ay = area_h - 1;
+	   *ay = Conf.desks.areas_ny - 1;
 	else
 	   *ay = 0;
      }
-   else if (*ay >= area_h)
+   else if (*ay >= Conf.desks.areas_ny)
      {
 	if (Conf.desks.areas_wraparound)
 	   *ay = 0;
 	else
-	   *ay = area_h - 1;
+	   *ay = Conf.desks.areas_ny - 1;
      }
 }
 
@@ -1484,7 +1493,7 @@ static int
 AreaXYToLinear(int ax, int ay)
 {
    DesksFixArea(&ax, &ay);
-   return (ay * area_w) + ax;
+   return (ay * Conf.desks.areas_nx) + ax;
 }
 
 static void
@@ -1492,10 +1501,10 @@ AreaLinearToXY(int a, int *ax, int *ay)
 {
    if (a < 0)
       a = 0;
-   else if (a >= (area_w * area_h))
-      a = (area_w * area_h) - 1;
-   *ay = a / area_w;
-   *ax = a - (*ay * area_w);
+   else if (a >= (Conf.desks.areas_nx * Conf.desks.areas_ny))
+      a = (Conf.desks.areas_nx * Conf.desks.areas_ny) - 1;
+   *ay = a / Conf.desks.areas_nx;
+   *ax = a - (*ay * Conf.desks.areas_nx);
 }
 
 static void
@@ -1505,8 +1514,8 @@ SetAreaSize(int aw, int ah)
       aw = 1;
    if (ah < 1)
       ah = 1;
-   Conf.desks.areas_nx = area_w = aw;
-   Conf.desks.areas_ny = area_h = ah;
+   Conf.desks.areas_nx = aw;
+   Conf.desks.areas_ny = ah;
    HintsSetViewportConfig();
    EdgeWindowsShow();
    ModulesSignal(ESIGNAL_AREA_CONFIGURED, NULL);
@@ -1515,8 +1524,8 @@ SetAreaSize(int aw, int ah)
 void
 DesksGetAreaSize(int *aw, int *ah)
 {
-   *aw = area_w;
-   *ah = area_h;
+   *aw = Conf.desks.areas_nx;
+   *ah = Conf.desks.areas_ny;
 }
 
 static void
@@ -1586,6 +1595,28 @@ MoveCurrentLinearAreaBy(int a)
    SetCurrentLinearArea(GetCurrentLinearArea() + a);
 }
 
+/* Return 1 to disable area switch */
+static int
+_DeskAreaSwitchCheckEwins(void)
+{
+   EWin               *const *lst, *ewin;
+   int                 i, num;
+
+   lst = EwinListGetForDesk(&num, desks.current);
+   for (i = 0; i < num; i++)
+     {
+	ewin = lst[i];
+
+	/* Disable if there are non-sticky shading windows */
+	if (EoIsSticky(ewin) || ewin->state.iconified)
+	   continue;
+	if (ewin->state.shading)
+	   return 1;
+     }
+
+   return 0;
+}
+
 void
 DeskCurrentGotoArea(int ax, int ay)
 {
@@ -1596,6 +1627,9 @@ DeskCurrentGotoArea(int ax, int ay)
        || (Mode.mode == MODE_RESIZE_V))
       return;
 
+   if (_DeskAreaSwitchCheckEwins())
+      return;
+
    DesksFixArea(&ax, &ay);
    DeskCurrentGetArea(&pax, &pay);
 
@@ -1619,7 +1653,7 @@ DeskCurrentGotoArea(int ax, int ay)
    else if (dy > 0)
       SoundPlay(SOUND_MOVE_AREA_DOWN);
 
-   ActionsSuspend();
+   MoveResizeSuspend();
 
    /* remove lots of event masks from windows.. we dont want to bother */
    /* handling events as a result of our playing wiht windows */
@@ -1686,7 +1720,7 @@ DeskCurrentGotoArea(int ax, int ay)
    /* set hints up for it */
    HintsSetDesktopViewport();
 
-   ActionsResume();
+   MoveResizeResume();
 
    /* re-focus on a new ewin on that new desktop area */
    DeskSwitchDone();
@@ -1723,7 +1757,8 @@ DeskDragStart(int desk)
 {
    Desk               *dsk;
 
-   if (!(dsk = DeskGet(desk)))
+   dsk = DeskGet(desk);
+   if (!dsk)
       return;
 
    desks.drag_x0 = Mode.events.cx - EoGetX(dsk);
@@ -2147,34 +2182,41 @@ DesksSighan(int sig, void *prm __UNUSED__)
 /*
  * Dialogs
  */
-static int          tmp_desktops;
-static DItem       *tmp_desk_text;
-static char         tmp_desktop_slide;
-static int          tmp_desktop_slide_speed;
-static char         tmp_desktop_wraparound;
-static char         tmp_dragbar;
-static int          tmp_dragdir;
+typedef struct {
+   int                 desktops;
+   int                 prev_desktops;
+   DItem              *desk_text;
+   char                desktop_slide;
+   int                 desktop_slide_speed;
+   char                desktop_wraparound;
+   char                dragbar;
+   int                 dragdir;
+   char                initialised;
+   Win                 wins[ENLIGHTENMENT_CONF_NUM_DESKTOPS];
+} DeskDlgData;
 
 static void
-CB_ConfigureDesktops(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
+CB_ConfigureDesktops(Dialog * d, int val, void *data __UNUSED__)
 {
+   DeskDlgData        *dd = DLG_DATA_GET(d, DeskDlgData);
+
    if (val >= 2)
       return;
 
-   ChangeNumberOfDesktops(tmp_desktops);
-   Conf.desks.slidein = tmp_desktop_slide;
-   Conf.desks.slidespeed = tmp_desktop_slide_speed;
-   Conf.desks.desks_wraparound = tmp_desktop_wraparound;
+   ChangeNumberOfDesktops(dd->desktops);
+   Conf.desks.slidein = dd->desktop_slide;
+   Conf.desks.slidespeed = dd->desktop_slide_speed;
+   Conf.desks.desks_wraparound = dd->desktop_wraparound;
 
-   if ((Conf.desks.dragdir != tmp_dragdir) ||
-       ((tmp_dragbar) && (Conf.desks.dragbar_width < 1)) ||
-       ((!tmp_dragbar) && (Conf.desks.dragbar_width > 0)))
+   if ((Conf.desks.dragdir != dd->dragdir) ||
+       ((dd->dragbar) && (Conf.desks.dragbar_width < 1)) ||
+       ((!dd->dragbar) && (Conf.desks.dragbar_width > 0)))
      {
-	if (tmp_dragbar)
+	if (dd->dragbar)
 	   Conf.desks.dragbar_width = 16;
 	else
 	   Conf.desks.dragbar_width = 0;
-	Conf.desks.dragdir = tmp_dragdir;
+	Conf.desks.dragdir = dd->dragdir;
 	DesksControlsRefresh();
      }
 }
@@ -2182,9 +2224,7 @@ CB_ConfigureDesktops(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
 static void
 CB_DesktopDisplayRedraw(Dialog * d, int val, void *data)
 {
-   static char         called = 0;
-   static int          prev_desktops = -1;
-   static Win          wins[ENLIGHTENMENT_CONF_NUM_DESKTOPS];
+   DeskDlgData        *dd = DLG_DATA_GET(d, DeskDlgData);
    DItem              *di;
    int                 i;
    int                 w, h;
@@ -2192,74 +2232,69 @@ CB_DesktopDisplayRedraw(Dialog * d, int val, void *data)
    char                s[64];
    ImageClass         *ic;
 
-   if (val == 1)
-      called = 0;
-
-   if ((val != 1) && (prev_desktops == tmp_desktops))
+   if ((val != 1) && (dd->prev_desktops == dd->desktops))
       return;
 
-   prev_desktops = tmp_desktops;
+   dd->prev_desktops = dd->desktops;
    di = (DItem *) data;
    win = DialogItemAreaGetWindow(di);
    DialogItemAreaGetSize(di, &w, &h);
 
-   if (!called)
+   if (!dd->initialised)
      {
 	ic = ImageclassFind("SETTINGS_DESKTOP_AREA", 1);
 	ImageclassApply(ic, win, 0, 0, STATE_NORMAL, ST_SOLID);
-	for (i = 0; i < ENLIGHTENMENT_CONF_NUM_DESKTOPS; i++)
-	   wins[i] = 0;
-	called = 1;
+	dd->initialised = 1;
      }
 
-   for (i = 0; i < tmp_desktops; i++)
+   for (i = 0; i < dd->desktops; i++)
      {
-	if (!wins[i])
+	if (!dd->wins[i])
 	  {
 	     Background         *bg;
 
-	     wins[i] = ECreateWindow(win, 0, 0, 64, 48, 0);
-	     ESetWindowBorderWidth(wins[i], 1);
+	     dd->wins[i] = ECreateWindow(win, 0, 0, 64, 48, 0);
+	     ESetWindowBorderWidth(dd->wins[i], 1);
 
 	     bg = DeskBackgroundGet(DeskGet(i));
 	     if (bg)
 	       {
 		  Pixmap              pmap;
 
-		  pmap = EGetWindowBackgroundPixmap(wins[i]);
-		  BackgroundApplyPmap(bg, wins[i], pmap, 64, 48);
+		  pmap = EGetWindowBackgroundPixmap(dd->wins[i]);
+		  BackgroundApplyPmap(bg, dd->wins[i], pmap, 64, 48);
 	       }
 	     else
 	       {
 		  ic = ImageclassFind("SETTINGS_DESKTOP_AREA", 1);
-		  ImageclassApply(ic, wins[i], 0, 0, STATE_NORMAL, ST_SOLID);
+		  ImageclassApply(ic, dd->wins[i], 0, 0, STATE_NORMAL,
+				  ST_SOLID);
 	       }
 	  }
      }
 
-   for (i = tmp_desktops - 1; i >= 0; i--)
+   for (i = dd->desktops - 1; i >= 0; i--)
      {
 	int                 num;
 
-	num = tmp_desktops - 1;
+	num = dd->desktops - 1;
 	if (num < 1)
 	   num = 1;
-	EMoveWindow(wins[i], (i * (w - 64 - 2)) / num,
+	EMoveWindow(dd->wins[i], (i * (w - 64 - 2)) / num,
 		    (i * (h - 48 - 2)) / num);
-	ERaiseWindow(wins[i]);
-	EMapWindow(wins[i]);
+	ERaiseWindow(dd->wins[i]);
+	EMapWindow(dd->wins[i]);
      }
 
-   for (i = tmp_desktops; i < ENLIGHTENMENT_CONF_NUM_DESKTOPS; i++)
+   for (i = dd->desktops; i < ENLIGHTENMENT_CONF_NUM_DESKTOPS; i++)
      {
-	if (!wins[i])
+	if (!dd->wins[i])
 	   continue;
-	EUnmapWindow(wins[i]);
+	EUnmapWindow(dd->wins[i]);
      }
 
-   Esnprintf(s, sizeof(s), "%i", tmp_desktops);
-   DialogItemSetText(tmp_desk_text, s);
-   DialogDrawItems(d, tmp_desk_text, 0, 0, 99999, 99999);
+   Esnprintf(s, sizeof(s), "%i", dd->desktops);
+   DialogItemSetText(dd->desk_text, s);
 }
 
 static void
@@ -2270,19 +2305,25 @@ CB_DesktopDisplayAreaRedraw(DItem * di, int val __UNUSED__,
 }
 
 static void
-_DlgFillDesks(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
+_DlgFillDesks(Dialog * d, DItem * table, void *data __UNUSED__)
 {
    DItem              *di, *slider, *radio;
+   DeskDlgData        *dd;
 
-   tmp_desktops = Conf.desks.num;
-   tmp_desktop_slide = Conf.desks.slidein;
-   tmp_desktop_slide_speed = Conf.desks.slidespeed;
-   tmp_desktop_wraparound = Conf.desks.desks_wraparound;
+   dd = DLG_DATA_SET(d, DeskDlgData);
+   if (!dd)
+      return;
+
+   dd->desktops = Conf.desks.num;
+   dd->prev_desktops = -1;
+   dd->desktop_slide = Conf.desks.slidein;
+   dd->desktop_slide_speed = Conf.desks.slidespeed;
+   dd->desktop_wraparound = Conf.desks.desks_wraparound;
    if (Conf.desks.dragbar_width < 1)
-      tmp_dragbar = 0;
+      dd->dragbar = 0;
    else
-      tmp_dragbar = 1;
-   tmp_dragdir = Conf.desks.dragdir;
+      dd->dragbar = 1;
+   dd->dragdir = Conf.desks.dragdir;
 
    DialogItemTableSetOptions(table, 2, 0, 0, 0);
 
@@ -2290,7 +2331,7 @@ _DlgFillDesks(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Number of virtual desktops:\n"));
 
-   di = tmp_desk_text = DialogAddItem(table, DITEM_TEXT);
+   di = dd->desk_text = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, "X");
 
@@ -2299,7 +2340,7 @@ _DlgFillDesks(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSliderSetUnits(di, 1);
    DialogItemSliderSetJump(di, 1);
    DialogItemSetColSpan(di, 2);
-   DialogItemSliderSetValPtr(di, &tmp_desktops);
+   DialogItemSliderSetValPtr(di, &dd->desktops);
 
    di = DialogAddItem(table, DITEM_AREA);
    DialogItemSetColSpan(di, 2);
@@ -2314,7 +2355,7 @@ _DlgFillDesks(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Slide desktops around when changing"));
-   DialogItemCheckButtonSetPtr(di, &tmp_desktop_slide);
+   DialogItemCheckButtonSetPtr(di, &dd->desktop_slide);
 
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetColSpan(di, 2);
@@ -2326,7 +2367,7 @@ _DlgFillDesks(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSliderSetBounds(di, 0, 20000);
    DialogItemSliderSetUnits(di, 500);
    DialogItemSliderSetJump(di, 1000);
-   DialogItemSliderSetValPtr(di, &tmp_desktop_slide_speed);
+   DialogItemSliderSetValPtr(di, &dd->desktop_slide_speed);
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
    DialogItemSetColSpan(di, 2);
@@ -2334,7 +2375,7 @@ _DlgFillDesks(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Wrap desktops around"));
-   DialogItemCheckButtonSetPtr(di, &tmp_desktop_wraparound);
+   DialogItemCheckButtonSetPtr(di, &dd->desktop_wraparound);
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
    DialogItemSetColSpan(di, 2);
@@ -2342,7 +2383,7 @@ _DlgFillDesks(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Display desktop dragbar"));
-   DialogItemCheckButtonSetPtr(di, &tmp_dragbar);
+   DialogItemCheckButtonSetPtr(di, &dd->dragbar);
 
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetColSpan(di, 2);
@@ -2372,7 +2413,7 @@ _DlgFillDesks(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSetText(di, _("Right"));
    DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 1);
-   DialogItemRadioButtonGroupSetValPtr(radio, &tmp_dragdir);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->dragdir);
 }
 
 const DialogDef     DlgDesks = {
@@ -2386,42 +2427,47 @@ const DialogDef     DlgDesks = {
    DLG_OAC, CB_ConfigureDesktops,
 };
 
-static int          tmp_area_x;
-static int          tmp_area_y;
-static int          tmp_edge_flip;
-static int          tmp_edge_resist;
-static DItem       *tmp_area_text;
-static char         tmp_area_wraparound;
+typedef struct {
+   int                 area_x;
+   int                 area_y;
+   int                 edge_flip;
+   int                 edge_resist;
+   DItem              *area_text;
+   char                area_wraparound;
+   int                 prev_ax, prev_ay;
+   Win                 awin;
+} AreaDlgData;
 
 static void
-CB_ConfigureAreas(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
+CB_ConfigureAreas(Dialog * d, int val, void *data __UNUSED__)
 {
+   AreaDlgData        *dd = DLG_DATA_GET(d, AreaDlgData);
+
    if (val >= 2)
       return;
 
-   SetNewAreaSize(tmp_area_x, tmp_area_y);
-   Conf.desks.areas_wraparound = tmp_area_wraparound;
-   Conf.desks.edge_flip_mode = tmp_edge_flip;
-   if (tmp_edge_resist < 1)
-      tmp_edge_resist = 1;
-   Conf.desks.edge_flip_resistance = tmp_edge_resist;
+   SetNewAreaSize(dd->area_x, dd->area_y);
+   Conf.desks.areas_wraparound = dd->area_wraparound;
+   Conf.desks.edge_flip_mode = dd->edge_flip;
+   if (dd->edge_resist < 1)
+      dd->edge_resist = 1;
+   Conf.desks.edge_flip_resistance = dd->edge_resist;
 }
 
 static void
-CB_AreaDisplayRedraw(Dialog * d __UNUSED__, int val, void *data)
+CB_AreaDisplayRedraw(Dialog * d, int val, void *data)
 {
-   static int          prev_ax = 0, prev_ay = 0;
-   static Win          awin;
+   AreaDlgData        *dd = DLG_DATA_GET(d, AreaDlgData);
    char                s[64];
    DItem              *di;
    Win                 win;
    int                 w, h, ww, hh;
 
-   if ((val != 1) && (prev_ax == tmp_area_x) && (prev_ay == tmp_area_y))
+   if ((val != 1) && (dd->prev_ax == dd->area_x) && (dd->prev_ay == dd->area_y))
       return;
 
-   prev_ax = tmp_area_x;
-   prev_ay = tmp_area_y;
+   dd->prev_ax = dd->area_x;
+   dd->prev_ay = dd->area_y;
 
    di = (DItem *) data;
    win = DialogItemAreaGetWindow(di);
@@ -2436,19 +2482,18 @@ CB_AreaDisplayRedraw(Dialog * d __UNUSED__, int val, void *data)
 	ImageclassApply(ic, win, 0, 0, STATE_NORMAL, ST_SOLID);
 
 	/* Note: awin is destroyed when the dialog is destroyed */
-	awin = ECreateWindow(win, 0, 0, 18, 14, 0);
+	dd->awin = ECreateWindow(win, 0, 0, 18, 14, 0);
 	ic = ImageclassFind("SETTINGS_AREADESK_AREA", 1);
-	pmap = EGetWindowBackgroundPixmap(awin);
-	ImageclassApplySimple(ic, awin, pmap, STATE_NORMAL, 0, 0, 18, 14);
+	pmap = EGetWindowBackgroundPixmap(dd->awin);
+	ImageclassApplySimple(ic, dd->awin, pmap, STATE_NORMAL, 0, 0, 18, 14);
      }
-   ww = 18 * prev_ax;
-   hh = 14 * prev_ay;
-   EMoveResizeWindow(awin, (w - ww) / 2, (h - hh) / 2, ww, hh);
-   EMapWindow(awin);
+   ww = 18 * dd->prev_ax;
+   hh = 14 * dd->prev_ay;
+   EMoveResizeWindow(dd->awin, (w - ww) / 2, (h - hh) / 2, ww, hh);
+   EMapWindow(dd->awin);
 
-   Esnprintf(s, sizeof(s), "%i x %i", prev_ax, prev_ay);
-   DialogItemSetText(tmp_area_text, s);
-   DialogDrawItems(d, tmp_area_text, 0, 0, 99999, 99999);
+   Esnprintf(s, sizeof(s), "%i x %i", dd->prev_ax, dd->prev_ay);
+   DialogItemSetText(dd->area_text, s);
 }
 
 static void
@@ -2458,23 +2503,28 @@ CB_AreaDisplayAreaRedraw(DItem * di, int val __UNUSED__, void *data __UNUSED__)
 }
 
 static void
-_DlgFillAreas(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
+_DlgFillAreas(Dialog * d, DItem * table, void *data __UNUSED__)
 {
    DItem              *di, *slider, *slider2, *table2, *radio;
+   AreaDlgData        *dd;
 
-   tmp_area_wraparound = Conf.desks.areas_wraparound;
+   dd = DLG_DATA_SET(d, AreaDlgData);
+   if (!dd)
+      return;
 
-   tmp_edge_flip = Conf.desks.edge_flip_mode;
-   tmp_edge_resist = Conf.desks.edge_flip_resistance;
+   dd->area_wraparound = Conf.desks.areas_wraparound;
 
-   DesksGetAreaSize(&tmp_area_x, &tmp_area_y);
+   dd->edge_flip = Conf.desks.edge_flip_mode;
+   dd->edge_resist = Conf.desks.edge_flip_resistance;
+
+   DesksGetAreaSize(&dd->area_x, &dd->area_y);
 
    DialogItemTableSetOptions(table, 1, 0, 0, 0);
 
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetText(di, _("Virtual Desktop size:\n"));
 
-   di = tmp_area_text = DialogAddItem(table, DITEM_TEXT);
+   di = dd->area_text = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetText(di, "X");
 
    table2 = DialogAddItem(table, DITEM_TABLE);
@@ -2487,7 +2537,7 @@ _DlgFillAreas(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSliderSetBounds(di, 1, 8);
    DialogItemSliderSetUnits(di, 1);
    DialogItemSliderSetJump(di, 1);
-   DialogItemSliderSetValPtr(di, &tmp_area_x);
+   DialogItemSliderSetValPtr(di, &dd->area_x);
 
    di = slider2 = DialogAddItem(table2, DITEM_SLIDER);
    DialogItemSliderSetMinLength(di, 10);
@@ -2496,7 +2546,7 @@ _DlgFillAreas(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSliderSetBounds(di, 1, 8);
    DialogItemSliderSetUnits(di, 1);
    DialogItemSliderSetJump(di, 1);
-   DialogItemSliderSetValPtr(di, &tmp_area_y);
+   DialogItemSliderSetValPtr(di, &dd->area_y);
 
    di = DialogAddItem(table2, DITEM_AREA);
    DialogItemAreaSetSize(di, 160, 120);
@@ -2509,7 +2559,7 @@ _DlgFillAreas(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetText(di, _("Wrap virtual desktops around"));
-   DialogItemCheckButtonSetPtr(di, &tmp_area_wraparound);
+   DialogItemCheckButtonSetPtr(di, &dd->area_wraparound);
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
 
@@ -2531,7 +2581,7 @@ _DlgFillAreas(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSetText(di, _("Only when moving window"));
    DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, EDGE_FLIP_MOVE);
-   DialogItemRadioButtonGroupSetValPtr(radio, &tmp_edge_flip);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->edge_flip);
 
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetText(di, _("Resistance at edge of screen:\n"));
@@ -2541,7 +2591,7 @@ _DlgFillAreas(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSliderSetBounds(di, 1, 100);
    DialogItemSliderSetUnits(di, 1);
    DialogItemSliderSetJump(di, 10);
-   DialogItemSliderSetValPtr(di, &tmp_edge_resist);
+   DialogItemSliderSetValPtr(di, &dd->edge_resist);
 }
 
 const DialogDef     DlgAreas = {
@@ -2600,7 +2650,7 @@ DesksIpcDesk(const char *params)
 {
    const char         *p;
    char                cmd[128], prm[128];
-   int                 len;
+   int                 len, value;
    unsigned int        desk;
 
    cmd[0] = prm[0] = '\0';
@@ -2674,7 +2724,13 @@ DesksIpcDesk(const char *params)
      }
    else if (!strcmp(cmd, "clear"))
      {
-	EwinsShowDesktop(!Mode.showing_desktop);
+	if (!strcmp(prm, "on"))
+	   value = 1;
+	else if (!strcmp(prm, "off"))
+	   value = 0;
+	else
+	   value = !Mode.showing_desktop;
+	EwinsShowDesktop(value);
      }
    else if (!strncmp(cmd, "arrange", 3))
      {
@@ -2750,7 +2806,7 @@ static const IpcItem DesksIpcArray[] = {
     "  desk lower <d>       Lower desktop\n"
     "  desk raise <d>       Raise desktop\n"
     "  desk arrange         Arrange windows on desktop\"\n"
-    "  desk clear           \"Show Desktop\"\n"}
+    "  desk clear [on/off]  \"Show Desktop\"\n"}
    ,
    {
     DesksIpcArea,
