@@ -39,13 +39,11 @@ typedef struct {
    char                horizontal;
 
    char                numeric;
-   char                numeric_side;
 
    int                 upper;
    int                 lower;
    int                 unit;
    int                 jump;
-   int                 val;
    int                *val_ptr;
 
    int                 min_length;
@@ -186,6 +184,8 @@ static void         DItemHandleEvents(Win win, XEvent * ev, void *prm);
 static void         MoveTableBy(Dialog * d, DItem * di, int dx, int dy);
 static void         DialogItemsRealize(Dialog * d);
 static void         DialogItemDestroy(DItem * di, int clean);
+static void         DialogDrawItems(Dialog * d, DItem * di, int x, int y, int w,
+				    int h);
 
 static int          DialogItemCheckButtonGetState(DItem * di);
 
@@ -235,7 +235,7 @@ DialogCreate(const char *name)
 
    d->name = Estrdup(name);
    d->win = ECreateClientWindow(VROOT, -20, -20, 2, 2);
-   EventCallbackRegister(d->win, 0, DialogHandleEvents, d);
+   EventCallbackRegister(d->win, DialogHandleEvents, d);
 
    d->iclass = ImageclassAlloc("DIALOG", 1);
    d->tclass = TextclassAlloc("DIALOG", 1);
@@ -253,6 +253,7 @@ DialogDestroy(Dialog * d)
 
    Efree(d->name);
    Efree(d->title);
+   Efree(d->data);
    DialogKeybindingsDestroy(d);
    if (d->item)
       DialogItemDestroy(d->item, 0);
@@ -298,133 +299,19 @@ DialogCallExitFunction(Dialog * d)
       d->exit_func(d, d->exit_val, NULL);
 }
 
-void
-DialogSetData(Dialog * d, void *data)
-{
-   d->data = data;
-}
-
 void               *
-DialogGetData(Dialog * d)
+DialogDataSet(Dialog * d, unsigned int size)
 {
+   d->data = Ecalloc(size, 1);
    return d->data;
 }
 
-#if 0				/* FIXME - Merge/remove */
-void
-DialogAddButton(Dialog * d, const char *text, DialogCallbackFunc * func,
-		char doclose, int image)
+void               *
+DialogDataGet(Dialog * d)
 {
-   DButton            *db;
-   int                 w, h;
-   EImageBorder       *pad;
-
-   db = EMALLOC(DButton, 1);
-
-   d->num_buttons++;
-   d->button = EREALLOC(DButton *, d->button, d->num_buttons);
-   d->button[d->num_buttons - 1] = db;
-
-   db->parent = d;
-   db->text = Estrdup(text);
-   db->func = func;
-   db->image = image;
-   db->win = ECreateWindow(d->win, -20, -20, 2, 2, 0);
-   EventCallbackRegister(db->win, 0, DButtonHandleEvents, db);
-   EMapWindow(db->win);
-   db->x = -1;
-   db->y = -1;
-   db->w = -1;
-   db->h = -1;
-   db->hilited = 0;
-   db->clicked = 0;
-   db->close = doclose;
-
-   db->tclass = TextclassAlloc("DIALOG_BUTTON", 1);
-   db->iclass = ImageclassAlloc("DIALOG_BUTTON", 1);
-
-   TextSize(db->tclass, 0, 0, STATE_NORMAL, text, &w, &h, 17);
-   pad = ImageclassGetPadding(db->iclass);
-   db->h = h + pad->top + pad->bottom;
-   db->w = w + pad->left + pad->right;
-   if (Conf.dialogs.button_image && db->image)
-      db->w += h + 2;
-
-   ESelectInput(db->win,
-		EnterWindowMask | LeaveWindowMask | ButtonPressMask |
-		ButtonReleaseMask);
+   return d->data;
 }
 
-static void
-DialogDrawButton(Dialog * d __UNUSED__, DButton * db)
-{
-   int                 state;
-   EImage             *im;
-
-   state = STATE_NORMAL;
-   if ((db->hilited) && (db->clicked))
-     {
-	state = STATE_CLICKED;
-     }
-   else if ((db->hilited) && (!db->clicked))
-     {
-	state = STATE_HILITED;
-     }
-   else if (!(db->hilited) && (db->clicked))
-     {
-	state = STATE_CLICKED;
-     }
-
-   im = NULL;
-   if (Conf.dialogs.button_image)
-     {
-	switch (db->image)
-	  {
-	  case DLG_BUTTON_OK:
-	     im = ThemeImageLoad("pix/ok.png");
-	     break;
-	  case DLG_BUTTON_CANCEL:
-	     im = ThemeImageLoad("pix/cancel.png");
-	     break;
-	  case DLG_BUTTON_APPLY:
-	     im = ThemeImageLoad("pix/apply.png");
-	     break;
-	  case DLG_BUTTON_CLOSE:
-	     im = ThemeImageLoad("pix/close.png");
-	     break;
-	  default:
-	     break;
-	  }
-     }
-
-   if (im)
-     {
-	ImageClass         *ic = db->iclass;
-	EImageBorder       *pad;
-	int                 h;
-
-	ImageclassApply(db->iclass, db->win, 0, 0, state, ST_WIDGET);
-
-	pad = ImageclassGetPadding(ic);
-	h = db->h - (pad->top + pad->bottom);
-	TextDraw(db->tclass, db->win, None, 0, 0, state, db->text,
-		 h + 2 + pad->left, pad->top,
-		 db->w - (h + 2 + pad->left + pad->right),
-		 h, h, TextclassGetJustification(db->tclass));
-
-	EImageRenderOnDrawable(im, db->win, None,
-			       EIMAGE_BLEND | EIMAGE_ANTI_ALIAS,
-			       pad->left, pad->top, h, h);
-	EImageFree(im);
-     }
-   else
-     {
-	ITApply(db->win, db->iclass, NULL, state, 0, 0,
-		ST_WIDGET, db->tclass, NULL, db->text, 0);
-     }
-}
-#endif
-
 DItem              *
 DialogItemAddButton(DItem * parent, const char *text, DialogCallbackFunc * func,
 		    int val, char doclose, int image __UNUSED__)
@@ -561,8 +448,6 @@ DialogShowArranged(Dialog * d, int center)
    ewin->client.event_mask |= KeyPressMask;
    ESelectInput(d->win, ewin->client.event_mask);
 
-   EwinMoveToDesktop(ewin, EoGetDesk(ewin));
-
    if (ewin->state.placed)
      {
 	EwinMoveResize(ewin, EoGetX(ewin), EoGetY(ewin), d->w, d->h);
@@ -628,6 +513,9 @@ DialogFill(Dialog * d, DItem * parent, const DialogDef * dd, void *data)
    if (!content)
       return;
 
+   Efree(d->data);
+   d->data = NULL;
+
    dd->fill(d, content, data);
 
    if (dd->func)
@@ -677,11 +565,6 @@ DialogItemCreate(int type)
    di->row_span = 1;
    di->col_span = 1;
    di->item.table.num_columns = 1;
-   di->item.table.border = 0;
-   di->item.table.homogenous_h = 0;
-   di->item.table.homogenous_v = 0;
-   di->item.table.num_items = 0;
-   di->item.table.items = NULL;
 
    DialogItemSetPadding(di, 2, 2, 2, 2);
    DialogItemSetFill(di, 1, 0);
@@ -726,7 +609,6 @@ DialogAddItem(DItem * dii, int type)
 	di->item.area.h = 32;
 	break;
      case DITEM_CHECKBUTTON:
-	di->item.check_button.check_win = 0;
 	di->item.check_button.onoff = 0;
 	di->item.check_button.onoff_ptr = &(di->item.check_button.onoff);
 	di->item.check_button.check_orig_w = 10;
@@ -734,11 +616,6 @@ DialogAddItem(DItem * dii, int type)
 	break;
      case DITEM_TABLE:
 	di->item.table.num_columns = 1;
-	di->item.table.border = 0;
-	di->item.table.homogenous_h = 0;
-	di->item.table.homogenous_v = 0;
-	di->item.table.num_items = 0;
-	di->item.table.items = NULL;
 	break;
      case DITEM_IMAGE:
 	di->item.image.image = NULL;
@@ -747,48 +624,20 @@ DialogAddItem(DItem * dii, int type)
 	di->item.separator.horizontal = 0;
 	break;
      case DITEM_RADIOBUTTON:
-	di->item.radio_button.radio_win = 0;
-	di->item.radio_button.onoff = 0;
-	di->item.radio_button.val = 0;
-	di->item.radio_button.val_ptr = 0;
-	di->item.radio_button.next = NULL;
-	di->item.radio_button.first = NULL;
 	di->item.radio_button.radio_orig_w = 10;
 	di->item.radio_button.radio_orig_h = 10;
-	di->item.radio_button.event_func = NULL;
 	break;
      case DITEM_SLIDER:
 	di->item.slider.horizontal = 1;
-	di->item.slider.numeric = 0;
-	di->item.slider.numeric_side = 0;
 	di->item.slider.upper = 100;
 	di->item.slider.lower = 0;
 	di->item.slider.unit = 10;
 	di->item.slider.jump = 20;
-	di->item.slider.val = 0;
-	di->item.slider.val_ptr = NULL;
 	di->item.slider.min_length = 64;
-	di->item.slider.ic_base = NULL;
-	di->item.slider.ic_knob = NULL;
-	di->item.slider.base_win = 0;
-	di->item.slider.knob_win = 0;
 	di->item.slider.base_orig_w = 10;
 	di->item.slider.base_orig_h = 10;
 	di->item.slider.knob_orig_w = 6;
 	di->item.slider.knob_orig_h = 6;
-	di->item.slider.base_x = 0;
-	di->item.slider.base_y = 0;
-	di->item.slider.base_w = 0;
-	di->item.slider.base_h = 0;
-	di->item.slider.knob_x = 0;
-	di->item.slider.knob_y = 0;
-	di->item.slider.knob_w = 0;
-	di->item.slider.knob_h = 0;
-	di->item.slider.numeric_x = 0;
-	di->item.slider.numeric_y = 0;
-	di->item.slider.numeric_w = 0;
-	di->item.slider.numeric_h = 0;
-	di->item.slider.in_drag = 0;
 	break;
      }
 
@@ -1016,13 +865,11 @@ DialogRealizeItem(Dialog * d, DItem * di)
 	ESelectInput(di->item.slider.base_win,
 		     EnterWindowMask | LeaveWindowMask |
 		     ButtonPressMask | ButtonReleaseMask);
-	EventCallbackRegister(di->item.slider.base_win, 0, DItemHandleEvents,
-			      di);
+	EventCallbackRegister(di->item.slider.base_win, DItemHandleEvents, di);
 	ESelectInput(di->item.slider.knob_win,
 		     EnterWindowMask | LeaveWindowMask |
 		     ButtonPressMask | ButtonReleaseMask | PointerMotionMask);
-	EventCallbackRegister(di->item.slider.knob_win, 0, DItemHandleEvents,
-			      di);
+	EventCallbackRegister(di->item.slider.knob_win, DItemHandleEvents, di);
 
 	if (!di->item.slider.ic_base)
 	  {
@@ -1098,7 +945,7 @@ DialogRealizeItem(Dialog * d, DItem * di)
 	ESelectInput(di->item.area.area_win,
 		     EnterWindowMask | LeaveWindowMask | ButtonPressMask |
 		     ButtonReleaseMask | PointerMotionMask);
-	EventCallbackRegister(di->item.area.area_win, 0, DItemHandleEvents, di);
+	EventCallbackRegister(di->item.area.area_win, DItemHandleEvents, di);
 	di->w = iw;
 	di->h = ih;
 	break;
@@ -1136,9 +983,6 @@ DialogRealizeItem(Dialog * d, DItem * di)
 	if (im)
 	  {
 	     EImageGetSize(im, &iw, &ih);
-	     di->win = ECreateWindow(d->win, 0, 0, iw, ih, 0);
-	     EMapWindow(di->win);
-	     EImageApplyToWin(im, di->win, 0, 0, 0);
 	     EImageFree(im);
 	  }
 	di->w = iw;
@@ -1409,7 +1253,7 @@ DialogRealizeItem(Dialog * d, DItem * di)
      }
 
    if (di->win && register_win_callback)
-      EventCallbackRegister(di->win, 0, DItemHandleEvents, di);
+      EventCallbackRegister(di->win, DItemHandleEvents, di);
 }
 
 static void
@@ -1475,7 +1319,7 @@ MoveTableBy(Dialog * d, DItem * di, int dx, int dy)
      }
 }
 
-void
+static void
 DialogDrawItems(Dialog * d, DItem * di, int x, int y, int w, int h)
 {
    d->update = 1;
@@ -1503,8 +1347,9 @@ DialogDrawItems(Dialog * d, DItem * di, int x, int y, int w, int h)
 static void
 DialogDrawItem(Dialog * d, DItem * di)
 {
-   int                 state, x, w;
+   int                 state, x, w, val;
    EImageBorder       *pad;
+   EImage             *im;
 
    if (!di->update && di->type != DITEM_TABLE)
       return;
@@ -1551,11 +1396,13 @@ DialogDrawItem(Dialog * d, DItem * di)
 	break;
 
      case DITEM_SLIDER:
+	val = (di->item.slider.val_ptr) ?
+	   *(di->item.slider.val_ptr) : di->item.slider.lower;
 	if (di->item.slider.horizontal)
 	  {
 	     di->item.slider.knob_x = di->item.slider.base_x +
 		(((di->item.slider.base_w - di->item.slider.knob_w) *
-		  (di->item.slider.val - di->item.slider.lower)) /
+		  (val - di->item.slider.lower)) /
 		 (di->item.slider.upper - di->item.slider.lower));
 	     di->item.slider.knob_y = di->item.slider.base_y +
 		((di->item.slider.base_h - di->item.slider.knob_h) / 2);
@@ -1564,7 +1411,7 @@ DialogDrawItem(Dialog * d, DItem * di)
 	  {
 	     di->item.slider.knob_y = di->item.slider.base_y +
 		(((di->item.slider.base_h - di->item.slider.knob_h) *
-		  (di->item.slider.val - di->item.slider.lower)) /
+		  (val - di->item.slider.lower)) /
 		 (di->item.slider.upper - di->item.slider.lower));
 	     di->item.slider.knob_x = di->item.slider.base_x +
 		((di->item.slider.base_w - di->item.slider.knob_w) / 2);
@@ -1625,6 +1472,17 @@ DialogDrawItem(Dialog * d, DItem * di)
 	w = di->w;
 	goto draw_text;
 
+     case DITEM_IMAGE:
+	im = ThemeImageLoad(di->item.image.image);
+	if (im)
+	  {
+	     EImageRenderOnDrawable(im, d->win, WinGetPmap(d->win),
+				    EIMAGE_BLEND | EIMAGE_ANTI_ALIAS,
+				    di->x, di->y, di->w, di->h);
+	     EImageFree(im);
+	  }
+	break;
+
      case DITEM_CHECKBUTTON:
 	state = STATE_NORMAL;
 	if ((di->hilited) && (di->clicked))
@@ -1742,6 +1600,9 @@ DialogItemSetText(DItem * di, const char *text)
 {
    Efree(di->text);
    di->text = Estrdup(text);
+
+   if (di->realized)
+      DialogDrawItems(di->dlg, di, di->x, di->y, di->w, di->h);
 }
 
 void
@@ -1782,7 +1643,12 @@ DialogItemRadioButtonGroupSetVal(DItem * di, int val)
 void
 DialogItemCheckButtonSetState(DItem * di, char onoff)
 {
+   if (*(di->item.check_button.onoff_ptr) == onoff)
+      return;
    *(di->item.check_button.onoff_ptr) = onoff;
+
+   if (di->realized)
+      DialogDrawItems(di->dlg, di, di->x, di->y, di->w, di->h);
 }
 
 void
@@ -1829,9 +1695,11 @@ DialogItemSliderSetVal(DItem * di, int val)
       val = di->item.slider.lower;
    else if (val > di->item.slider.upper)
       val = di->item.slider.upper;
-   di->item.slider.val = val;
    if (di->item.slider.val_ptr)
       *di->item.slider.val_ptr = val;
+
+   if (di->realized)
+      DialogDrawItems(di->dlg, di, di->x, di->y, di->w, di->h);
 }
 
 void
@@ -1856,6 +1724,9 @@ DialogItemSliderSetBounds(DItem * di, int lower, int upper)
      }
    if (di->item.slider.upper <= di->item.slider.lower)
       di->item.slider.upper = di->item.slider.lower + 1;
+
+   if (di->realized)
+      DialogDrawItems(di->dlg, di, di->x, di->y, di->w, di->h);
 }
 
 void
@@ -1882,12 +1753,6 @@ DialogItemSliderSetOrientation(DItem * di, char horizontal)
    di->item.slider.horizontal = horizontal;
 }
 
-int
-DialogItemSliderGetVal(DItem * di)
-{
-   return di->item.slider.val;
-}
-
 void
 DialogItemSliderGetBounds(DItem * di, int *lower, int *upper)
 {
@@ -2096,6 +1961,8 @@ DialogHandleEvents(Win win __UNUSED__, XEvent * ev, void *prm)
 static void
 DItemEventMotion(Win win __UNUSED__, DItem * di, XEvent * ev)
 {
+   int                 val;
+
    switch (di->type)
      {
      case DITEM_AREA:
@@ -2110,7 +1977,7 @@ DItemEventMotion(Win win __UNUSED__, DItem * di, XEvent * ev)
 	  {
 	     if (di->item.slider.horizontal)
 	       {
-		  di->item.slider.val =
+		  val =
 		     _DlgPixToVal(di,
 				  ev->xbutton.x + di->item.slider.knob_x -
 				  di->item.slider.knob_w / 2,
@@ -2119,7 +1986,7 @@ DItemEventMotion(Win win __UNUSED__, DItem * di, XEvent * ev)
 	       }
 	     else
 	       {
-		  di->item.slider.val =
+		  val =
 		     _DlgPixToVal(di,
 				  ev->xbutton.y + di->item.slider.knob_y -
 				  di->item.slider.knob_h / 2,
@@ -2127,7 +1994,7 @@ DItemEventMotion(Win win __UNUSED__, DItem * di, XEvent * ev)
 				  di->item.slider.knob_h);
 	       }
 	     if (di->item.slider.val_ptr)
-		*di->item.slider.val_ptr = di->item.slider.val;
+		*di->item.slider.val_ptr = val;
 	     if (di->func)
 		(di->func) (di->dlg, di->val, di->data);
 	  }
@@ -2140,7 +2007,7 @@ DItemEventMotion(Win win __UNUSED__, DItem * di, XEvent * ev)
 static void
 DItemEventMouseDown(Win win, DItem * di, XEvent * ev)
 {
-   int                 x, y, wheel_jump;
+   int                 x, y, wheel_jump, val;
 
    switch (di->type)
      {
@@ -2159,6 +2026,9 @@ DItemEventMouseDown(Win win, DItem * di, XEvent * ev)
 	       }
 	  }
 
+	val = (di->item.slider.val_ptr) ?
+	   *(di->item.slider.val_ptr) : di->item.slider.lower;
+
 	/* Coords -> item.slider.base_win */
 	ETranslateCoordinates(win, di->item.slider.base_win,
 			      ev->xbutton.x, ev->xbutton.y, &x, &y, NULL);
@@ -2171,24 +2041,24 @@ DItemEventMouseDown(Win win, DItem * di, XEvent * ev)
 	       {
 		  if (ev->xbutton.x >
 		      (di->item.slider.knob_x + (di->item.slider.knob_w / 2)))
-		     di->item.slider.val += di->item.slider.jump;
+		     val += di->item.slider.jump;
 		  else
-		     di->item.slider.val -= di->item.slider.jump;
+		     val -= di->item.slider.jump;
 	       }
 	     else
 	       {
 		  if (ev->xbutton.y >
 		      (di->item.slider.knob_y + (di->item.slider.knob_h / 2)))
-		     di->item.slider.val += di->item.slider.jump;
+		     val += di->item.slider.jump;
 		  else
-		     di->item.slider.val -= di->item.slider.jump;
+		     val -= di->item.slider.jump;
 	       }
 	     break;
 
 	  case 2:
 	     if (di->item.slider.horizontal)
 	       {
-		  di->item.slider.val =
+		  val =
 		     _DlgPixToVal(di,
 				  ev->xbutton.x - di->item.slider.knob_w / 2,
 				  di->item.slider.base_w -
@@ -2196,7 +2066,7 @@ DItemEventMouseDown(Win win, DItem * di, XEvent * ev)
 	       }
 	     else
 	       {
-		  di->item.slider.val =
+		  val =
 		     _DlgPixToVal(di,
 				  ev->xbutton.y - di->item.slider.knob_h / 2,
 				  di->item.slider.base_h -
@@ -2211,21 +2081,17 @@ DItemEventMouseDown(Win win, DItem * di, XEvent * ev)
 		wheel_jump++;
 
 	     if (ev->xbutton.button == 5)
-	       {
-		  di->item.slider.val -= wheel_jump;
-	       }
+		val -= wheel_jump;
 	     else if (ev->xbutton.button == 4)
-	       {
-		  di->item.slider.val += wheel_jump;
-	       }
+		val += wheel_jump;
 	     break;
 	  }
-	if (di->item.slider.val < di->item.slider.lower)
-	   di->item.slider.val = di->item.slider.lower;
-	if (di->item.slider.val > di->item.slider.upper)
-	   di->item.slider.val = di->item.slider.upper;
+	if (val < di->item.slider.lower)
+	   val = di->item.slider.lower;
+	if (val > di->item.slider.upper)
+	   val = di->item.slider.upper;
 	if (di->item.slider.val_ptr)
-	   *di->item.slider.val_ptr = di->item.slider.val;
+	   *di->item.slider.val_ptr = val;
 #if 0				/* Remove? */
 	if (di->func)
 	   (di->func) (d, di->val, di->data);
