@@ -83,8 +83,12 @@ void                DialogSetTitle(Dialog * d, const char *title);
 void                DialogSetExitFunction(Dialog * d, DialogCallbackFunc * func,
 					  int val);
 void                DialogCallExitFunction(Dialog * d);
-void                DialogSetData(Dialog * d, void *data);
-void               *DialogGetData(Dialog * d);
+
+void               *DialogDataSet(Dialog * d, unsigned int size);
+void               *DialogDataGet(Dialog * d);
+
+#define DLG_DATA_SET(dlg, type) (type*)DialogDataSet(dlg, sizeof(type))
+#define DLG_DATA_GET(dlg, type) (type*)DialogDataGet(dlg)
 
 void                DialogShow(Dialog * d);
 void                DialogShowCentered(Dialog * d);
@@ -112,8 +116,6 @@ void                DialogItemSetFill(DItem * di, char fill_h, char fill_v);
 void                DialogItemSetAlign(DItem * di, int align_h, int align_v);
 void                DialogItemSetText(DItem * di, const char *text);
 void                DialogItemCallCallback(Dialog * d, DItem * di);
-void                DialogDrawItems(Dialog * d, DItem * di, int x, int y, int w,
-				    int h);
 void                DialogItemRadioButtonSetEventFunc(DItem * di,
 						      DialogItemCallbackFunc *
 						      func);
@@ -139,7 +141,6 @@ void                DialogItemSliderSetJump(DItem * di, int jump);
 void                DialogItemSliderSetMinLength(DItem * di, int min);
 void                DialogItemSliderSetValPtr(DItem * di, int *val_ptr);
 void                DialogItemSliderSetOrientation(DItem * di, char horizontal);
-int                 DialogItemSliderGetVal(DItem * di);
 void                DialogItemSliderGetBounds(DItem * di, int *lower,
 					      int *upper);
 
