@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -241,230 +241,6 @@ static int          ECompMgrDetermineOrder(EObj * const *lst, int num,
 #define REGION_DESTROY(rgn) \
    if (rgn != None) { ERegionDestroy(rgn); rgn = None; }
 
-/*
- * Regions
- */
-#define DEBUG_REGIONS 0
-
-#if DEBUG_REGIONS
-static int          n_rgn_c = 0;
-static int          n_rgn_d = 0;
-#endif
-
-static              XserverRegion
-ERegionCreate(void)
-{
-   XserverRegion       rgn;
-
-   rgn = XFixesCreateRegion(disp, NULL, 0);
-
-#if DEBUG_REGIONS
-   n_rgn_c++;
-   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
-	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
-#endif
-   return rgn;
-}
-
-static              XserverRegion
-ERegionCreateRect(int x, int y, int w, int h)
-{
-   XserverRegion       rgn;
-   XRectangle          rct;
-
-   rct.x = x;
-   rct.y = y;
-   rct.width = w;
-   rct.height = h;
-   rgn = XFixesCreateRegion(disp, &rct, 1);
-
-#if DEBUG_REGIONS
-   n_rgn_c++;
-   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
-	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
-#endif
-   return rgn;
-}
-
-#if USE_DESK_EXPOSE
-static              XserverRegion
-ERegionCreateFromRects(XRectangle * rectangles, int nrectangles)
-{
-   XserverRegion       rgn;
-
-   rgn = XFixesCreateRegion(disp, rectangles, nrectangles);
-
-#if DEBUG_REGIONS
-   n_rgn_c++;
-   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
-	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
-#endif
-   return rgn;
-}
-#endif
-
-static              XserverRegion
-ERegionCreateFromWindow(Win win)
-{
-   XserverRegion       rgn;
-
-   rgn =
-      XFixesCreateRegionFromWindow(disp, WinGetXwin(win), WindowRegionBounding);
-
-#if DEBUG_REGIONS
-   n_rgn_c++;
-   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
-	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
-#endif
-   return rgn;
-}
-
-static              XserverRegion
-ERegionCopy(XserverRegion rgn, XserverRegion src)
-{
-   XFixesCopyRegion(disp, rgn, src);
-   return rgn;
-}
-
-static              XserverRegion
-ERegionClone(XserverRegion src)
-{
-   XserverRegion       rgn;
-
-   rgn = ERegionCreate();
-   ERegionCopy(rgn, src);
-
-   return rgn;
-}
-
-static void
-ERegionDestroy(XserverRegion rgn)
-{
-#if DEBUG_REGIONS
-   n_rgn_d++;
-   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
-	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
-#endif
-   XFixesDestroyRegion(disp, rgn);
-}
-
-static void
-ERegionEmpty(XserverRegion rgn)
-{
-   XFixesSetRegion(disp, rgn, NULL, 0);
-}
-
-static void
-ERegionSetRect(XserverRegion rgn, int x, int y, int w, int h)
-{
-   XRectangle          rct;
-
-   rct.x = x;
-   rct.y = y;
-   rct.width = w;
-   rct.height = h;
-   XFixesSetRegion(disp, rgn, &rct, 1);
-}
-
-static void
-ERegionTranslate(XserverRegion rgn, int dx, int dy)
-{
-   if (dx == 0 && dy == 0)
-      return;
-   XFixesTranslateRegion(disp, rgn, dx, dy);
-}
-
-static void
-ERegionIntersect(XserverRegion dst, XserverRegion src)
-{
-   XFixesIntersectRegion(disp, dst, dst, src);
-}
-
-static void
-ERegionUnion(XserverRegion dst, XserverRegion src)
-{
-   XFixesUnionRegion(disp, dst, dst, src);
-}
-
-static void
-ERegionSubtract(XserverRegion dst, XserverRegion src)
-{
-   XFixesSubtractRegion(disp, dst, dst, src);
-}
-
-static void
-ERegionSubtractOffset(XserverRegion dst, int dx, int dy, XserverRegion src)
-{
-   Display            *dpy = disp;
-   XserverRegion       rgn;
-
-   rgn = src;
-   if (dx != 0 || dy != 0)
-     {
-	rgn = ERegionCopy(rgn_tmp, src);
-	XFixesTranslateRegion(dpy, rgn, dx, dy);
-     }
-   XFixesSubtractRegion(dpy, dst, dst, rgn);
-}
-
-static void
-ERegionUnionOffset(XserverRegion dst, int dx, int dy, XserverRegion src)
-{
-   Display            *dpy = disp;
-   XserverRegion       rgn;
-
-   rgn = src;
-   if (dx != 0 || dy != 0)
-     {
-	rgn = ERegionCopy(rgn_tmp, src);
-	XFixesTranslateRegion(dpy, rgn, dx, dy);
-     }
-   XFixesUnionRegion(dpy, dst, dst, rgn);
-}
-
-#if 0				/* Unused (for debug) */
-static int
-ERegionIsEmpty(XserverRegion rgn)
-{
-   int                 nr;
-   XRectangle         *pr;
-
-   pr = XFixesFetchRegion(disp, rgn, &nr);
-   if (pr)
-      XFree(pr);
-   return nr == 0;
-}
-#endif
-
-static void
-ERegionShow(const char *txt, XserverRegion rgn)
-{
-   int                 i, nr;
-   XRectangle         *pr;
-
-   if (rgn == None)
-     {
-	Eprintf(" - region: %s %#lx is None\n", txt, rgn);
-	return;
-     }
-
-   pr = XFixesFetchRegion(disp, rgn, &nr);
-   if (!pr || nr <= 0)
-     {
-	Eprintf(" - region: %s %#lx is empty\n", txt, rgn);
-	goto done;
-     }
-
-   Eprintf(" - region: %s %#lx:\n", txt, rgn);
-   for (i = 0; i < nr; i++)
-      Eprintf("%4d: %4d+%4d %4dx%4d\n", i, pr[i].x, pr[i].y, pr[i].width,
-	      pr[i].height);
-
- done:
-   if (pr)
-      XFree(pr);
-}
-
 void
 ECompMgrWinClipToGC(EObj * eo, GC gc)
 {
@@ -478,74 +254,6 @@ ECompMgrWinClipToGC(EObj * eo, GC gc)
    XFixesSetGCClipRegion(disp, gc, 0, 0, rgn);
 }
 
-/*
- * Pictures
- */
-
-#define _R(x) (((x) >> 16) & 0xff)
-#define _G(x) (((x) >>  8) & 0xff)
-#define _B(x) (((x)      ) & 0xff)
-
-static              Picture
-EPictureCreateSolid(Bool argb, unsigned int a, unsigned int rgb)
-{
-   Display            *dpy = disp;
-   Pixmap              pmap;
-   Picture             pict;
-   XRenderPictFormat  *pictfmt;
-   XRenderPictureAttributes pa;
-   XRenderColor        c;
-
-   pmap = XCreatePixmap(dpy, Mode_compmgr.root, 1, 1, argb ? 32 : 8);
-   pictfmt = XRenderFindStandardFormat(dpy,
-				       argb ? PictStandardARGB32 :
-				       PictStandardA8);
-   pa.repeat = True;
-   pict = XRenderCreatePicture(dpy, pmap, pictfmt, CPRepeat, &pa);
-
-   c.alpha = (unsigned short)(a * 0x101);
-   c.red = (unsigned short)(_R(rgb) * 0x101);
-   c.green = (unsigned short)(_G(rgb) * 0x101);
-   c.blue = (unsigned short)(_B(rgb) * 0x101);
-   XRenderFillRectangle(dpy, PictOpSrc, pict, &c, 0, 0, 1, 1);
-
-   XFreePixmap(dpy, pmap);
-
-   return pict;
-}
-
-static              Picture
-EPictureCreateBuffer(Win win, int w, int h, Pixmap * ppmap)
-{
-   Picture             pict;
-   Pixmap              pmap;
-   XRenderPictFormat  *pictfmt;
-
-   pmap = XCreatePixmap(disp, WinGetXwin(win), w, h, WinGetDepth(win));
-   pictfmt = XRenderFindVisualFormat(disp, WinGetVisual(win));
-   pict = XRenderCreatePicture(disp, pmap, pictfmt, 0, 0);
-   if (ppmap)
-      *ppmap = pmap;
-   else
-      XFreePixmap(disp, pmap);
-
-   return pict;
-}
-
-#if 0
-static              Picture
-EPictureCreate(Window win, int depth, Visual * vis)
-{
-   Picture             pict;
-   XRenderPictFormat  *pictfmt;
-
-   pictfmt = XRenderFindVisualFormat(disp, vis);
-   pict = XRenderCreatePicture(disp, win, pictfmt, 0, 0);
-
-   return pict;
-}
-#endif
-
 /* Hack to fix redirected window resize bug(?) */
 void
 ECompMgrMoveResizeFix(EObj * eo, int x, int y, int w, int h)
@@ -591,7 +299,7 @@ ECompMgrMoveResizeFix(EObj * eo, int x, int y, int w, int h)
  * Desk background
  */
 
-int
+void
 ECompMgrDeskConfigure(Desk * dsk)
 {
    EObj               *eo;
@@ -601,17 +309,8 @@ ECompMgrDeskConfigure(Desk * dsk)
    XRenderPictureAttributes pa;
    Pixmap              pmap;
 
-   if (!Mode_compmgr.active)
-      return 0;
-
    eo = dsk->bg.o;
-   if (!eo)
-      return 1;
-
-   ECompMgrWinInvalidate(eo, INV_PICTURE);
-
-   if (!dsk->viewable && dsk->bg.bg)
-      return 1;
+   cw = eo->cmhook;
 
    if (dsk->bg.pmap == None)
      {
@@ -630,6 +329,8 @@ ECompMgrDeskConfigure(Desk * dsk)
 	pmap = dsk->bg.pmap;
      }
 
+   ECompMgrWinInvalidate(eo, INV_PICTURE);
+
    pa.repeat = True;
    pictfmt = XRenderFindVisualFormat(disp, WinGetVisual(VROOT));
    pict = XRenderCreatePicture(disp, pmap, pictfmt, CPRepeat, &pa);
@@ -637,17 +338,14 @@ ECompMgrDeskConfigure(Desk * dsk)
    if (pmap != dsk->bg.pmap)
       XFreePixmap(disp, pmap);
 
-   /* New background, all must be repainted */
-   ECompMgrDamageAll();
-
-   cw = eo->cmhook;
    cw->picture = pict;
 
    D1printf
       ("ECompMgrDeskConfigure: Desk %d: using pixmap %#lx picture=%#lx\n",
        dsk->num, pmap, cw->picture);
 
-   return 1;
+   /* New background, all must be repainted */
+   ECompMgrDamageAll();
 }
 #endif
 
@@ -697,7 +395,7 @@ ECompMgrDamageMerge(XserverRegion damage)
    if (Mode_compmgr.got_damage)
      {
 	if (EDebug(EDBUG_TYPE_COMPMGR3))
-	   ERegionShow("ECompMgrDamageMerge add:", damage);
+	   ERegionShow("ECompMgrDamageMerge add:", damage, NULL);
 
 	ERegionUnion(Mode_compmgr.damage, damage);
      }
@@ -712,7 +410,7 @@ ECompMgrDamageMerge(XserverRegion damage)
    Mode_compmgr.got_damage = 1;
 
    if (EDebug(EDBUG_TYPE_COMPMGR3))
-      ERegionShow("ECompMgrDamageMerge all:", Mode_compmgr.damage);
+      ERegionShow("ECompMgrDamageMerge all:", Mode_compmgr.damage, NULL);
 }
 
 static void
@@ -1127,7 +825,7 @@ ECompMgrWinSetExtents(EObj * eo)
 	    r.height);
 
    if (EDebug(EDBUG_TYPE_COMPMGR))
-      ERegionShow("extents", cw->extents);
+      ERegionShow("extents", cw->extents, NULL);
 }
 
 /* Region of shaped window in screen coordinates */
@@ -1164,19 +862,7 @@ ECompMgrWinSetShape(EObj * eo)
 
    D1printf("shape %#lx: %d %d\n", EobjGetXwin(eo), cw->shape_x, cw->shape_y);
    if (EDebug(EDBUG_TYPE_COMPMGR))
-      ERegionShow("shape", cw->shape);
-}
-
-static              Pixmap
-EWindowGetNamePixmap(const struct _xwin *win)
-{
-   XWindowAttributes   xwa;
-
-   if (XGetWindowAttributes(disp, win->xwin, &xwa) == 0 ||
-       xwa.map_state == IsUnmapped)
-      return None;
-
-   return XCompositeNameWindowPixmap(disp, WinGetXwin(win));
+      ERegionShow("shape", cw->shape, NULL);
 }
 
 Pixmap
@@ -1193,7 +879,7 @@ ECompMgrWinGetPixmap(const EObj * eo)
    if (eo->noredir)
       return None;
 
-   cw->pixmap = EWindowGetNamePixmap(EobjGetWin(eo));
+   cw->pixmap = EWindowGetPixmap(EobjGetWin(eo));
 
    return cw->pixmap;
 }
@@ -1390,7 +1076,6 @@ ECompMgrWinFadeOut(EObj * eo)
 	   0x10000000);
 #endif
    cw->fadeout = 1;
-   ECompMgrWinInvalidate(eo, INV_PICTURE);
    ECompMgrWinFade(eo, cw->opacity, 0x10000000);
 }
 
@@ -1468,7 +1153,10 @@ ECompMgrWinUnmap(EObj * eo)
    ECompMgrDamageMergeObject(eo, cw->extents);
    _ECM_SET_STACK_CHANGED();
    if (Conf_compmgr.fading.enable && eo->fade && !eo->gone)
-      ECompMgrWinFadeOut(eo);
+     {
+	ECompMgrWinInvalidate(eo, INV_PICTURE);
+	ECompMgrWinFadeOut(eo);
+     }
    else
       ECompMgrWinInvalidate(eo, INV_PIXMAP);
 }
@@ -1481,7 +1169,7 @@ ECompMgrWinSetPicts(EObj * eo)
    if (cw->pixmap == None && eo->shown && !eo->noredir &&
        (Mode_compmgr.use_pixmap || (eo->fade && Conf_compmgr.fading.enable)))
      {
-	cw->pixmap = EWindowGetNamePixmap(EobjGetWin(eo));
+	cw->pixmap = EWindowGetPixmap(EobjGetWin(eo));
 	D2printf("ECompMgrWinSetPicts %#lx: Pmap=%#lx\n", EobjGetXwin(eo),
 		 cw->pixmap);
      }
@@ -1504,12 +1192,13 @@ ECompMgrWinSetPicts(EObj * eo)
 	D2printf("ECompMgrWinSetPicts %#lx: Pict=%#lx (drawable=%#lx)\n",
 		 EobjGetXwin(eo), cw->picture, draw);
 
-	if (draw == cw->pixmap)
+	/* Hmm.. What good does this actually do? */
+	if (draw == cw->pixmap && EobjIsShaped(eo))
 	  {
 	     XserverRegion       clip;
 
 	     clip = ERegionCreateFromWindow(EobjGetWin(eo));
-	     XFixesSetPictureClipRegion(disp, cw->picture, 0, 0, clip);
+	     EPictureSetClip(cw->picture, clip);
 	     ERegionDestroy(clip);
 	  }
      }
@@ -1583,7 +1272,7 @@ ECompMgrWinNew(EObj * eo)
    cw->opacity = 0xdeadbeef;
    ECompMgrWinSetOpacity(eo, eo->opacity);
 
-   EventCallbackRegister(EobjGetWin(eo), 0, ECompMgrHandleWindowEvent, eo);
+   EventCallbackRegister(EobjGetWin(eo), ECompMgrHandleWindowEvent, eo);
 }
 
 void
@@ -1614,7 +1303,7 @@ ECompMgrWinMoveResize(EObj * eo, int change_xy, int change_wh, int change_bw)
      }
 
    if (EDebug(EDBUG_TYPE_COMPMGR3))
-      ERegionShow("old-extents:", cw->extents);
+      ERegionShow("old-extents:", cw->extents, NULL);
 
 #if 0				/* FIXME - We shouldn't have to update clip if transparent */
    if (cw->mode == WINDOW_UNREDIR || cw->mode == WINDOW_SOLID)
@@ -1715,7 +1404,7 @@ ECompMgrWinReparent(EObj * eo, Desk * dsk, int change_xy)
 
    /* Invalidate old window region */
    if (EDebug(EDBUG_TYPE_COMPMGR3))
-      ERegionShow("old-extents:", cw->extents);
+      ERegionShow("old-extents:", cw->extents, NULL);
    ECompMgrDamageMergeObject(eo, cw->extents);
    if (change_xy)
      {
@@ -1782,7 +1471,7 @@ ECompMgrWinDel(EObj * eo)
    if (eo->fading)
       ECompMgrWinFadeEnd(eo, 1);
 
-   EventCallbackUnregister(EobjGetWin(eo), 0, ECompMgrHandleWindowEvent, eo);
+   EventCallbackUnregister(EobjGetWin(eo), ECompMgrHandleWindowEvent, eo);
 
    if (!eo->gone)
      {
@@ -1849,28 +1538,30 @@ ECompMgrWinDamage(EObj * eo, XEvent * ev __UNUSED__)
 }
 
 static void
-ECompMgrWinDumpInfo(const char *txt, EObj * eo, XserverRegion rgn, int force)
+ECompMgrWinDumpInfo(const char *txt, EObj * eo, XserverRegion rgn, int ipc)
 {
+   void                (*prf) (const char *fmt, ...);
    ECmWinInfo         *cw = eo->cmhook;
 
-   Eprintf("%s %#lx: %d,%d %dx%d: %s\n", txt, EobjGetXwin(eo),
-	   EobjGetX(eo), EobjGetY(eo), EobjGetW(eo), EobjGetH(eo),
-	   EobjGetName(eo));
+   prf = (ipc) ? IpcPrintf : Eprintf;
+
+   prf("%s %#lx: %d,%d %dx%d: %s\n", txt, EobjGetXwin(eo),
+       EobjGetX(eo), EobjGetY(eo), EobjGetW(eo), EobjGetH(eo), EobjGetName(eo));
    if (!cw)
      {
-	Eprintf("Not managed\n");
+	prf("Not managed\n");
 	return;
      }
 
-   if (force || EDebug(EDBUG_TYPE_COMPMGR3))
+   if (ipc || EDebug(EDBUG_TYPE_COMPMGR3))
      {
-	Eprintf(" - pict=%#lx pmap=%#lx\n", cw->picture, cw->pixmap);
+	prf(" - pict=%#lx pmap=%#lx\n", cw->picture, cw->pixmap);
 
-	ERegionShow("win extents", cw->extents);
-	ERegionShow("win shape  ", cw->shape);
-	ERegionShow("win clip   ", cw->clip);
+	ERegionShow("win extents", cw->extents, prf);
+	ERegionShow("win shape  ", cw->shape, prf);
+	ERegionShow("win clip   ", cw->clip, prf);
 	if (rgn != None)
-	   ERegionShow("region", rgn);
+	   ERegionShow("region", rgn, prf);
      }
 }
 
@@ -2016,9 +1707,10 @@ ECompMgrDetermineOrder(EObj * const *lst, int num, EObj ** first,
 		      cw->clip, EobjGetX(eo), EobjGetY(eo), EobjGetW(eo),
 		      EobjGetH(eo), EobjGetName(eo));
 #if USE_CLIP_RELATIVE_TO_DESK
-	     ERegionUnionOffset(clip, 0, 0, cw->shape);
+	     ERegionUnion(clip, cw->shape);
 #else
-	     ERegionUnionOffset(clip, EoGetX(dsk), EoGetY(dsk), cw->shape);
+	     ERegionUnionOffset(clip, EoGetX(dsk), EoGetY(dsk), cw->shape,
+				rgn_tmp);
 #endif
 	     break;
 
@@ -2054,9 +1746,9 @@ ECompMgrRepaintObjSetClip(XserverRegion rgn, XserverRegion damage,
 {
    ERegionCopy(rgn, damage);
 #if USE_CLIP_RELATIVE_TO_DESK
-   ERegionSubtractOffset(rgn, x, y, clip);
+   ERegionSubtractOffset(rgn, x, y, clip, rgn_tmp);
 #else
-   ERegionSubtractOffset(rgn, 0, 0, clip);
+   ERegionSubtract(rgn, clip);
    x = y = 0;
 #endif
    return rgn;
@@ -2099,7 +1791,7 @@ ECompMgrRepaintObj(Picture pbuf, XserverRegion region, EObj * eo, int mode)
 	     clip = ECompMgrRepaintObjSetClip(rgn_clip, region, cw->clip, x, y);
 	     if (EDebug(EDBUG_TYPE_COMPMGR2))
 		ECompMgrWinDumpInfo("ECompMgrRepaintObj solid", eo, clip, 0);
-	     XFixesSetPictureClipRegion(dpy, pbuf, 0, 0, clip);
+	     EPictureSetClip(pbuf, clip);
 	     XRenderComposite(dpy, PictOpSrc, cw->picture, None, pbuf,
 			      0, 0, 0, 0, x + cw->rcx, y + cw->rcy, cw->rcw,
 			      cw->rch);
@@ -2121,10 +1813,11 @@ ECompMgrRepaintObj(Picture pbuf, XserverRegion region, EObj * eo, int mode)
 	     clip = ECompMgrRepaintObjSetClip(rgn_clip, region, cw->clip, x, y);
 	     if (EDebug(EDBUG_TYPE_COMPMGR2))
 		ECompMgrWinDumpInfo("ECompMgrRepaintObj trans", eo, clip, 0);
-	     XFixesSetPictureClipRegion(dpy, pbuf, 0, 0, clip);
+	     EPictureSetClip(pbuf, clip);
 	     if (cw->opacity != OPAQUE && !cw->pict_alpha)
 		cw->pict_alpha =
-		   EPictureCreateSolid(True, OP32To8(cw->opacity),
+		   EPictureCreateSolid(Mode_compmgr.root, True,
+				       OP32To8(cw->opacity),
 				       Conf_compmgr.shadows.color);
 	     XRenderComposite(dpy, PictOpOver, cw->picture, cw->pict_alpha,
 			      pbuf, 0, 0, 0, 0, x + cw->rcx, y + cw->rcy,
@@ -2138,8 +1831,8 @@ ECompMgrRepaintObj(Picture pbuf, XserverRegion region, EObj * eo, int mode)
 
 	if (clip == None)
 	   clip = ECompMgrRepaintObjSetClip(rgn_clip, region, cw->clip, x, y);
-	ERegionSubtractOffset(clip, x, y, cw->shape);
-	XFixesSetPictureClipRegion(dpy, pbuf, 0, 0, clip);
+	ERegionSubtractOffset(clip, x, y, cw->shape, rgn_tmp);
+	EPictureSetClip(pbuf, clip);
 
 	switch (Mode_compmgr.shadow_mode)
 	  {
@@ -2147,7 +1840,7 @@ ECompMgrRepaintObj(Picture pbuf, XserverRegion region, EObj * eo, int mode)
 	  case ECM_SHADOWS_ECHO:
 	     if (cw->opacity != OPAQUE && !cw->shadow_alpha)
 		cw->shadow_alpha =
-		   EPictureCreateSolid(True,
+		   EPictureCreateSolid(Mode_compmgr.root, True,
 				       OP32To8(cw->opacity *
 					       Mode_compmgr.opac_sharp),
 				       Conf_compmgr.shadows.color);
@@ -2172,7 +1865,8 @@ ECompMgrRepaintObj(Picture pbuf, XserverRegion region, EObj * eo, int mode)
 
 	     if (cw->opacity != OPAQUE && !cw->pict_alpha)
 		cw->pict_alpha =
-		   EPictureCreateSolid(True, OP32To8(cw->opacity),
+		   EPictureCreateSolid(Mode_compmgr.root, True,
+				       OP32To8(cw->opacity),
 				       Conf_compmgr.shadows.color);
 	     alpha = (cw->pict_alpha) ? cw->pict_alpha : transBlackPicture;
 	     XRenderComposite(dpy, PictOpOver, alpha, cw->shadow_pict, pbuf,
@@ -2212,7 +1906,7 @@ ECompMgrPaintGhosts(Picture pict, XserverRegion damage)
 	  }
 
 	/* Subtract window region from damage region */
-	ERegionSubtractOffset(damage, 0, 0, eo->cmhook->shape);
+	ERegionSubtract(damage, eo->cmhook->shape);
      }
 }
 
@@ -2232,7 +1926,7 @@ ECompMgrRepaint(void)
    D2printf("ECompMgrRepaint rootBuffer=%#lx rootPicture=%#lx\n",
 	    rootBuffer, rootPicture);
    if (EDebug(EDBUG_TYPE_COMPMGR2))
-      ERegionShow("damage", Mode_compmgr.damage);
+      ERegionShow("damage", Mode_compmgr.damage, NULL);
 
    pbuf = rootBuffer;
 
@@ -2252,12 +1946,12 @@ ECompMgrRepaint(void)
    Picture             pict;
 
    if (EDebug(EDBUG_TYPE_COMPMGR2))
-      ERegionShow("after opaque", region);
+      ERegionShow("after opaque", region, NULL);
 
    /* Repaint background, clipped by damage region and opaque windows */
    pict = dsk->o.cmhook->picture;
    D1printf("ECompMgrRepaint desk picture=%#lx\n", pict);
-   XFixesSetPictureClipRegion(dpy, pbuf, 0, 0, region);
+   EPictureSetClip(pbuf, region);
    XRenderComposite(dpy, PictOpSrc, pict, None, pbuf,
 		    0, 0, 0, 0, 0, 0, WinGetW(VROOT), WinGetH(VROOT));
 #endif
@@ -2272,7 +1966,7 @@ ECompMgrRepaint(void)
 
    if (pbuf != rootPicture)
      {
-	XFixesSetPictureClipRegion(dpy, pbuf, 0, 0, Mode_compmgr.damage);
+	EPictureSetClip(pbuf, Mode_compmgr.damage);
 	XRenderComposite(dpy, PictOpSrc, pbuf, None, rootPicture,
 			 0, 0, 0, 0, 0, 0, WinGetW(VROOT), WinGetH(VROOT));
      }
@@ -2385,10 +2079,11 @@ ECompMgrShadowsInit(int mode, int cleanup)
      {
 	if (mode == ECM_SHADOWS_BLURRED)
 	   transBlackPicture =
-	      EPictureCreateSolid(True, 255, Conf_compmgr.shadows.color);
+	      EPictureCreateSolid(Mode_compmgr.root, True, 255,
+				  Conf_compmgr.shadows.color);
 	else
 	   transBlackPicture =
-	      EPictureCreateSolid(True,
+	      EPictureCreateSolid(Mode_compmgr.root, True,
 				  OP32To8(Mode_compmgr.opac_sharp * OPAQUE),
 				  Conf_compmgr.shadows.color);
      }
@@ -2495,7 +2190,7 @@ ECompMgrStart(void)
    rgn_tmp = ERegionCreate();
    rgn_tmp2 = ERegionCreate();
 
-   EventCallbackRegister(VROOT, 0, ECompMgrHandleRootEvent, NULL);
+   EventCallbackRegister(VROOT, ECompMgrHandleRootEvent, NULL);
 
    wm_cm_sel = SelectionAcquire("_NET_WM_CM_S", NULL, NULL);
 
@@ -2563,7 +2258,7 @@ ECompMgrStop(void)
       XCompositeUnredirectSubwindows(disp, WinGetXwin(VROOT),
 				     CompositeRedirectManual);
 
-   EventCallbackUnregister(VROOT, 0, ECompMgrHandleRootEvent, NULL);
+   EventCallbackUnregister(VROOT, ECompMgrHandleRootEvent, NULL);
 
 #if USE_COMPOSITE_OVERLAY_WINDOW
    if (Mode_compmgr.cow != None)
