@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2008 Kim Woelders
+ * Copyright (C) 2003-2009 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -66,11 +66,12 @@ typedef enum {
 void                EModuleRegister(EModule * em);
 #endif
 
-extern const EModule *p_modules[];
+extern const EModule *const p_modules[];
 extern int          n_modules;
 
-const EModule     **ModuleListGet(int *num);
-void                ModuleListFree(const EModule ** lst);
+#define MODULE_LIST_GET(lst, num) \
+    do { lst = p_modules; num = n_modules; } while (0)
+#define MODULE_LIST_FREE(lst)
 
 int                 ModuleConfigSet(const char *name, const char *item,
 				    const char *params);
