@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -49,6 +49,9 @@
 #if USE_GLX
 #include <GL/glx.h>
 #endif
+#if USE_XI2
+#include <X11/extensions/XInput2.h>
+#endif
 
 #if ENABLE_DEBUG_EVENTS
 static const char  *EventName(unsigned int type);
@@ -153,6 +156,64 @@ ExtInitRR(int available)
 }
 #endif
 
+#if USE_XI2
+static int          xi_major_op = 0;
+
+static              Bool
+EInputQueryExtension(Display * dpy,
+		     int *event_base_return, int *error_base_return)
+{
+   return XQueryExtension(dpy, "XInputExtension", &xi_major_op,
+			  event_base_return, error_base_return);
+}
+
+static              Status
+EInputQueryVersion(Display * dpy,
+		   int *major_version_return, int *minor_version_return)
+{
+   *major_version_return = XI_2_Major;
+   *minor_version_return = XI_2_Minor;
+
+   return XIQueryVersion(dpy, major_version_return, minor_version_return);
+}
+
+#include "X11/extensions/XInput.h"
+
+static void
+ExtInitInput(int available)
+{
+   int                 i, j, nd;
+   XDeviceInfo        *dvi, *dvis;
+   char               *p;
+
+   if (!available)
+      return;
+
+   if (!EDebug(EDBUG_TYPE_VERBOSE))
+      return;
+
+   dvis = XListInputDevices(disp, &nd);
+   Eprintf("Dev  id nc use name\n");
+   for (i = 0; i < nd; i++)
+     {
+	dvi = dvis + i;
+	Eprintf(" %2d %#3lx  %d  %d %-32s", i,
+		dvi->id, dvi->num_classes, dvi->use, dvi->name);
+	p = (char *)dvi->inputclassinfo;
+	for (j = 0; j < dvi->num_classes; j++)
+	  {
+	     XAnyClassPtr        pcany = (XAnyClassPtr) p;
+
+	     printf("  %#3lx %3d", pcany->class, pcany->length);
+	     p += pcany->length;
+	  }
+	printf("\n");
+
+     }
+   XFreeDeviceList(dvis);
+}
+#endif
+
 static const EServerExt Extensions[] = {
    {"Shape", XEXT_SHAPE, XShapeQueryExtension, XShapeQueryVersion,
     ExtInitShape},
@@ -176,6 +237,9 @@ static const EServerExt Extensions[] = {
 #if USE_GLX
    {"GLX", XEXT_GLX, glXQueryExtension, glXQueryVersion, NULL},
 #endif
+#if USE_XI2
+   {"Input", XEXT_XI, EInputQueryExtension, EInputQueryVersion, ExtInitInput},
+#endif
 };
 
 static void
@@ -273,14 +337,12 @@ EventsGetXY(int *px, int *py)
 }
 
 static void
-ModeGetXY(Window rwin, int rx, int ry)
+ModeGetXY(int rx, int ry)
 {
-   Window              child;
-
    if (Mode.wm.window)
      {
-	XTranslateCoordinates(disp, rwin, WinGetXwin(VROOT), rx, ry,
-			      &Mode.events.cx, &Mode.events.cy, &child);
+	ETranslateCoordinates(RROOT, VROOT, rx, ry,
+			      &Mode.events.cx, &Mode.events.cy, NULL);
      }
    else
      {
@@ -308,7 +370,7 @@ HandleEvent(XEvent * ev)
 	Mode.events.last_keystate = ev->xkey.state;
      case KeyRelease:
 	Mode.events.time = ev->xkey.time;
-	ModeGetXY(ev->xbutton.root, ev->xkey.x_root, ev->xkey.y_root);
+	ModeGetXY(ev->xkey.x_root, ev->xkey.y_root);
 #if 0				/* FIXME - Why? */
 	if (ev->xkey.root != WinGetXwin(VROOT))
 	  {
@@ -326,7 +388,7 @@ HandleEvent(XEvent * ev)
      case ButtonPress:
      case ButtonRelease:
 	Mode.events.time = ev->xbutton.time;
-	ModeGetXY(ev->xbutton.root, ev->xbutton.x_root, ev->xbutton.y_root);
+	ModeGetXY(ev->xbutton.x_root, ev->xbutton.y_root);
 	Mode.events.on_screen = ev->xbutton.same_screen;
 	TooltipsHide();
 	goto do_stuff;
@@ -335,7 +397,7 @@ HandleEvent(XEvent * ev)
 	Mode.events.time = ev->xmotion.time;
 	Mode.events.px = Mode.events.mx;
 	Mode.events.py = Mode.events.my;
-	ModeGetXY(ev->xmotion.root, ev->xmotion.x_root, ev->xmotion.y_root);
+	ModeGetXY(ev->xmotion.x_root, ev->xmotion.y_root);
 	Mode.events.mx = Mode.events.cx;
 	Mode.events.my = Mode.events.cy;
 	Mode.events.on_screen = ev->xmotion.same_screen;
@@ -352,8 +414,7 @@ HandleEvent(XEvent * ev)
 	     if (!Mode.grabs.pointer_grab_active)
 		Mode.grabs.pointer_grab_active = 2;
 	  }
-	ModeGetXY(ev->xcrossing.root, ev->xcrossing.x_root,
-		  ev->xcrossing.y_root);
+	ModeGetXY(ev->xcrossing.x_root, ev->xcrossing.y_root);
 	TooltipsHide();
 	goto do_stuff;
 
@@ -366,8 +427,7 @@ HandleEvent(XEvent * ev)
 	     Mode.grabs.pointer_grab_window = None;
 	     Mode.grabs.pointer_grab_active = 0;
 	  }
-	ModeGetXY(ev->xcrossing.root, ev->xcrossing.x_root,
-		  ev->xcrossing.y_root);
+	ModeGetXY(ev->xcrossing.x_root, ev->xcrossing.y_root);
 	TooltipsHide();
 	goto do_stuff;
 
@@ -415,7 +475,6 @@ HandleEvent(XEvent * ev)
      {
      case ButtonRelease:	/*  5 */
 	Mode.events.last_bpress = 0;
-	Mode.action_inhibit = 0;
 	break;
 
 #if 1				/* Do this here? */
@@ -638,6 +697,47 @@ EventsCompress(XEvent * evq, int count)
 #endif
 }
 
+#if USE_XI2
+static void
+EventFetchXI2(XEvent * ev)
+{
+   XIDeviceEvent      *xie;
+
+   if (!XGetEventData(disp, &ev->xcookie))
+      return;
+
+   xie = (XIDeviceEvent *) ev->xcookie.data;
+
+   if (EDebug(EDBUG_TYPE_XI2))
+      Eprintf("%s: %#lx: XI2 ext=%d type=%d devid=%d srcid=%d\n",
+	      "EventsFetch", xie->event, xie->extension,
+	      xie->evtype, xie->deviceid, xie->sourceid);
+
+   switch (ev->xcookie.evtype)
+     {
+     default:
+	break;
+     case XI_ButtonPress:
+     case XI_ButtonRelease:
+     case XI_Motion:
+	ev->type = ev->xcookie.evtype;
+	ev->xbutton.window = xie->event;
+	ev->xbutton.root = xie->root;
+	ev->xbutton.subwindow = xie->child;
+	ev->xbutton.time = xie->time;
+	ev->xbutton.x = (int)xie->event_x;
+	ev->xbutton.y = (int)xie->event_y;
+	ev->xbutton.x_root = (int)xie->root_x;
+	ev->xbutton.y_root = (int)xie->root_y;
+	ev->xbutton.button = xie->detail;
+	ev->xbutton.state = xie->mods.effective;
+	ev->xbutton.same_screen = 1;	/* FIXME */
+	break;
+     }
+   XFreeEventData(disp, &ev->xcookie);
+}
+#endif
+
 static int
 EventsFetch(XEvent ** evq_p, int *evq_n)
 {
@@ -658,6 +758,14 @@ EventsFetch(XEvent ** evq_p, int *evq_n)
 	for (; i < count; i++, ev++)
 	  {
 	     XNextEvent(disp, ev);
+#if USE_XI2
+	     if (ev->type == GenericEvent)
+	       {
+		  if (ev->xcookie.extension == xi_major_op)
+		     EventFetchXI2(ev);
+		  continue;
+	       }
+#endif /* USE_XI2 */
 
 	     /* Map some event types to E internals */
 	     if (ev->type == event_base_shape + ShapeNotify)
