@@ -182,40 +182,40 @@ doEwinMoveResize(EWin * ewin, Desk * dsk, int x, int y, int w, int h, int flags)
    reparent = move = resize = raise = 0;
    floating = EoIsFloating(ewin);
 
-   if (flags & (MRF_DESK | MRF_FLOAT | MRF_UNFLOAT))
+   if (flags & MRF_FLOAT)
      {
-	if (flags & MRF_FLOAT)
+	if (floating == 0)
 	  {
-	     if (EoIsFloating(ewin) == 0)
-	       {
-		  dsk = (pdesk == NULL) ? EoGetDesk(ewin) : pdesk;
-		  floating = 1;
-	       }
-	     else if (EoIsFloating(ewin) == 1)
-	       {
-		  dsk = DeskGet(0);
-		  floating = 2;
-	       }
-	     flags |= MRF_RAISE;
+	     dsk = (pdesk == NULL) ? EoGetDesk(ewin) : pdesk;
+	     floating = 1;
 	  }
-	else if (flags & MRF_UNFLOAT)
+	else if (floating == 1)
 	  {
-	     floating = 0;
-	     flags |= MRF_RAISE;
+	     dsk = DeskGet(0);
+	     floating = 2;
 	  }
-	else
-	  {
-	     if (EoIsSticky(ewin) && !EoIsFloating(ewin))
-		dsk = DesksGetCurrent();
-	  }
-	if (dsk != pdesk)
-	   reparent = 1;
+	flags |= MRF_RAISE;
      }
-   else
+   else if (flags & MRF_UNFLOAT)
+     {
+	floating = 0;
+	flags |= MRF_RAISE;
+     }
+   else if (EoIsSticky(ewin) && !floating)
+     {
+	dsk = DesksGetCurrent();
+     }
+   else if (!dsk)
      {
 	dsk = EoGetDesk(ewin);
      }
 
+   if (dsk != pdesk)
+     {
+	reparent = 1;
+	flags |= MRF_DESK;
+     }
+
    if (Mode.mode == MODE_NONE && Mode.move.check)
      {
 	/* Don't throw windows offscreen */
@@ -279,6 +279,10 @@ doEwinMoveResize(EWin * ewin, Desk * dsk, int x, int y, int w, int h, int flags)
      }
    else
      {
+	if (w > 32000)
+	   w = 32000;
+	if (h > 32000)
+	   h = 32000;
 	if (ewin->ops && ewin->ops->Layout)
 	  {
 	     ewin->ops->Layout(ewin, &x, &y, &w, &h);
@@ -297,11 +301,12 @@ doEwinMoveResize(EWin * ewin, Desk * dsk, int x, int y, int w, int h, int flags)
 	ewin->client.w = w;
 	ewin->client.h = h;
 
-	/* Don't touch frame size while shaded */
 	if (ewin->state.shaded)
 	  {
 	     w = EoGetW(ewin);
 	     h = EoGetH(ewin);
+	     if (resize)
+		EwinBorderMinShadeSize(ewin, &w, &h);
 	  }
 	else
 	  {
@@ -327,9 +332,15 @@ doEwinMoveResize(EWin * ewin, Desk * dsk, int x, int y, int w, int h, int flags)
       ModulesSignal(ESIGNAL_EWIN_CHANGE, ewin);
 
    if (reparent)
-      EoReparent(ewin, EoObj(dsk), x, y);
+     {
+	EoReparent(ewin, EoObj(dsk), x, y);
+	if (flags & MRF_RESIZE)
+	   EoResize(ewin, w, h);
+     }
    else
-      EoMoveResize(ewin, x, y, w, h);
+     {
+	EoMoveResize(ewin, x, y, w, h);
+     }
 
    configure = 0;
    if (Mode.mode == MODE_NONE || resize || Conf.movres.update_while_moving)
@@ -404,7 +415,8 @@ doEwinMoveResize(EWin * ewin, Desk * dsk, int x, int y, int w, int h, int flags)
    if (Mode.op_source == OPSRC_USER)
       EwinSetPlacementGravity(ewin, x, y);
 
-   if ((flags & (MRF_MOVE | MRF_RESIZE)) && ewin->ops && ewin->ops->MoveResize)
+   if ((flags & (MRF_DESK | MRF_MOVE | MRF_RESIZE)) &&
+       ewin->ops && ewin->ops->MoveResize)
       ewin->ops->MoveResize(ewin, resize);
 
    if (Mode.mode == MODE_NONE)
@@ -1380,12 +1392,12 @@ EwinsShowDesktop(int on)
 
    lst = EwinListGetForDesk(&num, DesksGetCurrent());
 
-   for (i = 0; i < num; i++)
+   if (on)
      {
-	ewin = lst[i];
-
-	if (on)
+	for (i = 0; i < num; i++)
 	  {
+	     ewin = lst[i];
+
 	     if (EwinIsTransient(ewin) ||
 		 ewin->state.iconified || ewin->state.donthide)
 		continue;
@@ -1393,8 +1405,13 @@ EwinsShowDesktop(int on)
 	     ewin->state.showingdesk = 1;
 	     EwinIconify(ewin);
 	  }
-	else
+     }
+   else
+     {
+	for (i = num - 1; i >= 0; i--)
 	  {
+	     ewin = lst[i];
+
 	     if (!ewin->state.showingdesk)
 		continue;
 
