@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -221,10 +221,18 @@ EwinManage(EWin * ewin)
    Win                 frame;
    int                 type;
 
+   /* There seems to be a shape related problem when window dimensions
+    * are >= 32768. Also, it looks like it is not possible(?) to create
+    * pixmaps with dimensions >= 32768.
+    * So, limit to 32000 leaving room for borders. */
    if (ewin->client.w <= 0)
       ewin->client.w = 100;
+   else if (ewin->client.w > 32000)
+      ewin->client.w = 32000;
    if (ewin->client.h <= 0)
       ewin->client.h = 100;
+   else if (ewin->client.h > 32000)
+      ewin->client.h = 32000;
 
    if (ewin->state.docked)
       ewin->inh_wm.b.border = 1;
@@ -246,12 +254,11 @@ EwinManage(EWin * ewin)
 	EobjListFocusAdd(&ewin->o, 1);
 	EobjListOrderAdd(&ewin->o);
 
-	EventCallbackRegister(EoGetWin(ewin), 0,
-			      EwinHandleEventsToplevel, ewin);
-	EventCallbackRegister(ewin->win_container, 0,
-			      EwinHandleEventsContainer, ewin);
-	EventCallbackRegister(EwinGetClientWin(ewin), 0,
-			      EwinHandleEventsClient, ewin);
+	EventCallbackRegister(EoGetWin(ewin), EwinHandleEventsToplevel, ewin);
+	EventCallbackRegister(ewin->win_container, EwinHandleEventsContainer,
+			      ewin);
+	EventCallbackRegister(EwinGetClientWin(ewin), EwinHandleEventsClient,
+			      ewin);
      }
 
    att.event_mask = EWIN_CONTAINER_EVENT_MASK;
@@ -262,7 +269,8 @@ EwinManage(EWin * ewin)
 
    att.event_mask = EWIN_TOP_EVENT_MASK;
    att.do_not_propagate_mask =
-      ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
+      KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask |
+      PointerMotionMask;
    EChangeWindowAttributes(EoGetWin(ewin), CWEventMask | CWDontPropagate, &att);
 
    ewin->client.event_mask = EWIN_CLIENT_EVENT_MASK;
@@ -285,6 +293,9 @@ EwinManage(EWin * ewin)
    /* We must reparent after getting original window position */
    EReparentWindow(EwinGetClientWin(ewin), ewin->win_container, 0, 0);
 
+   /* Something (e.g. a match) may have changed the window size */
+   EResizeWindow(EwinGetClientWin(ewin), ewin->client.w, ewin->client.h);
+
    EwinUpdateShapeInfo(ewin);
 
    ModulesSignal(ESIGNAL_EWIN_CREATE, ewin);
@@ -375,10 +386,10 @@ EwinDestroy(EWin * ewin)
       Eprintf("EwinDestroy %#lx st=%d: %s\n", EwinGetClientXwin(ewin),
 	      ewin->state.state, EwinGetTitle(ewin));
 
-   EventCallbackUnregister(EoGetWin(ewin), 0, EwinHandleEventsToplevel, ewin);
-   EventCallbackUnregister(ewin->win_container, 0, EwinHandleEventsContainer,
+   EventCallbackUnregister(EoGetWin(ewin), EwinHandleEventsToplevel, ewin);
+   EventCallbackUnregister(ewin->win_container, EwinHandleEventsContainer,
 			   ewin);
-   EventCallbackUnregister(EwinGetClientWin(ewin), 0, EwinHandleEventsClient,
+   EventCallbackUnregister(EwinGetClientWin(ewin), EwinHandleEventsClient,
 			   ewin);
    if (!EwinIsInternal(ewin))
       EUnregisterWindow(EwinGetClientWin(ewin));
@@ -669,7 +680,7 @@ EwinStateUpdate(EWin * ewin)
 }
 
 static void
-AddToFamily(EWin * ewin, Window xwin)
+AddToFamily(EWin * ewin, Window xwin, int startup)
 {
    EWin               *ewin2;
    EWin              **lst;
@@ -698,6 +709,9 @@ AddToFamily(EWin * ewin, Window xwin)
    EwinManage(ewin);
    EwinConfigure(ewin);
 
+   if (startup)
+      ewin->state.placed = 1;
+
    /* if it hasn't been planted on a desktop - assign it the current desktop */
    dsk = EoGetDesk(ewin);
 
@@ -904,15 +918,14 @@ AddToFamily(EWin * ewin, Window xwin)
 	Mode.events.cx = rx;
 	Mode.events.cy = ry;
 	ewin->state.placed = 1;
-	x = Mode.events.cx + 1;
-	y = Mode.events.cy + 1;
-	EwinMoveToDesktopAt(ewin, dsk, x, y);
-	EwinMove(ewin, x, y);
-	EwinShow(ewin);
+	x = Mode.events.cx - 8;
+	y = Mode.events.cy - 8;
 	GrabPointerSet(VROOT, ECSR_GRAB, 0);
-	Mode.place.doing_manual = 1;
 	EoSetFloating(ewin, 1);	/* Causes reparenting to root */
-	ActionMoveStart(ewin, 0, 0, 0);
+	EwinOpFloatAt(ewin, OPSRC_USER, x, y);
+	EwinShow(ewin);
+	Mode.place.doing_manual = 1;
+	MoveResizeMoveStart(ewin, 0, 0, 0);
 	goto done;
      }
    else if (doslide)
@@ -1007,20 +1020,7 @@ EwinUnmap1(EWin * ewin)
    if (GetZoomEWin() == ewin)
       Zoom(NULL);
 
-   ActionsEnd(ewin);
-
-   if (Mode.place.doing_slide)
-     {
-#if 0				/* No event processing during slides - No use doing this here */
-	DrawEwinShape(ewin, Conf.slidemode, ewin->shape_x, ewin->shape_y,
-		      ewin->client.w, ewin->client.h, 2);
-#endif
-#if 0				/* FIXME - Do this right */
-	RemoveTimerEvent("Slide");
-	Mode.place.doing_slide = 0;
-	FocusEnable(1);
-#endif
-     }
+   MoveResizeEnd(ewin);
 }
 
 static void
@@ -1079,7 +1079,7 @@ EwinEventMapRequest(EWin * ewin, Window win)
 	if (ewin->state.state == EWIN_STATE_ICONIC)
 	   EwinDeIconify(ewin);
 	else if (ewin->state.state == EWIN_STATE_WITHDRAWN)
-	   AddToFamily(ewin, win);
+	   AddToFamily(ewin, win, 0);
 	else
 	  {
 	     Eprintf("AddToFamily: Already managing %s %#lx\n", "A",
@@ -1101,7 +1101,7 @@ EwinEventMapRequest(EWin * ewin, Window win)
 	     EwinShow(ewin);
 	  }
 	else
-	   AddToFamily(NULL, win);
+	   AddToFamily(NULL, win, 0);
      }
 }
 
@@ -1938,7 +1938,7 @@ EwinSlideIn(void *data)
  */
 static struct {
    unsigned int        flags;
-   EWin                ewin_old;
+   Desk               *desk;
 } EWinChanges;
 
 void
@@ -1951,8 +1951,7 @@ void
 EwinChangesStart(EWin * ewin)
 {
    EWinChanges.flags = 0;
-   /* Brute force :) */
-   EWinChanges.ewin_old = *ewin;
+   EWinChanges.desk = EoGetDesk(ewin);
 }
 
 void
@@ -1972,7 +1971,7 @@ EwinChangesProcess(EWin * ewin)
 	Desk               *desk, *pdesk;
 
 	desk = EoGetDesk(ewin);
-	pdesk = EoGetDesk(&EWinChanges.ewin_old);
+	pdesk = EWinChanges.desk;
 	if (desk != pdesk && !EoIsSticky(ewin))
 	  {
 	     EoSetDesk(ewin, pdesk);
@@ -2189,7 +2188,7 @@ EwinsManage(void)
 	  }
 	else
 	  {
-	     AddToFamily(NULL, xwin);
+	     AddToFamily(NULL, xwin, 1);
 	  }
      }
    XFree(xwins);
@@ -2232,9 +2231,6 @@ ActionsCheck(const char *which, EWin * ewin, XEvent * ev)
 {
    ActionClass        *ac;
 
-   if (Mode.action_inhibit)	/* Probably not here */
-      return 0;
-
    ac = ActionclassFind(which);
    if (!ac)
       return 0;
@@ -2260,16 +2256,11 @@ EwinHandleEventsToplevel(Win win __UNUSED__, XEvent * ev, void *prm)
 
    switch (ev->type)
      {
-     case KeyPress:
-	ActionsHandleKey(XLookupKeysym(&ev->xkey, 0));
-	break;
      case ButtonPress:
 	ActionsCheck("BUTTONBINDINGS", ewin, ev);
 	break;
      case ButtonRelease:
-	ActionsEnd(NULL);
 	ActionsCheck("BUTTONBINDINGS", ewin, ev);
-	BorderCheckState(ewin, ev);
 	break;
      case EnterNotify:
 	FocusHandleEnter(ewin, ev);
@@ -2277,9 +2268,6 @@ EwinHandleEventsToplevel(Win win __UNUSED__, XEvent * ev, void *prm)
      case LeaveNotify:
 	FocusHandleLeave(ewin, ev);
 	break;
-     case MotionNotify:
-	ActionsHandleMotion();
-	break;
      default:
 #if DEBUG_EWIN_EVENTS
 	Eprintf("EwinHandleEventsToplevel: type=%2d win=%#lx: %s\n",
@@ -2495,7 +2483,7 @@ EwinHandleEventsRoot(Win win __UNUSED__, XEvent * ev, void *prm __UNUSED__)
 static void
 EwinsInit(void)
 {
-   EventCallbackRegister(VROOT, 0, EwinHandleEventsRoot, NULL);
+   EventCallbackRegister(VROOT, EwinHandleEventsRoot, NULL);
 }
 
 /*
