@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -88,10 +89,8 @@ FX_ripple_timeout(void *data __UNUSED__)
 	fx_ripple_above =
 	   ECreatePixmap(fx_ripple_win, WinGetW(VROOT),
 			 fx_ripple_waterh * 2, 0);
-	if (gc)
-	   EXFreeGC(gc);
-	if (gc1)
-	   EXFreeGC(gc1);
+	EXFreeGC(gc);
+	EXFreeGC(gc1);
 	gcv.subwindow_mode = IncludeInferiors;
 	gc = EXCreateGC(WinGetXwin(fx_ripple_win), GCSubwindowMode, &gcv);
 	gc1 = EXCreateGC(WinGetXwin(fx_ripple_win), 0L, &gcv);
@@ -154,6 +153,8 @@ static void
 FX_Ripple_Quit(void)
 {
    TIMER_DEL(fx_ripple_timer);
+   if (!fx_ripple_win)
+      return;
    EClearArea(fx_ripple_win, 0, WinGetH(VROOT) - fx_ripple_waterh,
 	      WinGetW(VROOT), fx_ripple_waterh);
 }
@@ -175,293 +176,6 @@ FX_Ripple_Pause(void)
      }
 }
 
-#if E_FX_RAINDROPS
-#include "piximg.h"
-
-/****************************** RAIN DROPS **********************************/
-
-#define fx_raindrop_size 96
-#define fx_raindrop_size2 (fx_raindrop_size / 2)
-#define fx_raindrop_duration 32
-#define fx_frequency 4
-#define fx_amplitude 48
-static Win          fx_raindrops_win = NULL;
-static int          fx_raindrops_number = 4;
-static PixImg      *fx_raindrops_draw = NULL;
-static Timer       *fx_raindrops_timer = NULL;
-
-typedef struct {
-   int                 x, y;
-   int                 count;
-   PixImg             *buf;
-} DropContext;
-
-static DropContext  fx_raindrops[4];
-
-static int
-FX_raindrops_timeout(void *data __UNUSED__)
-{
-   static GC           gc1 = 0, gc = 0;
-   int                 i, x, y, xx, yy;
-   int                 percent_done;
-   static char         first = 1;
-   static char         sintab[256];
-   static unsigned char disttab[fx_raindrop_size][fx_raindrop_size];
-   EObj               *bgeo;
-
-   bgeo = DeskGetBackgroundObj(DesksGetCurrent());
-
-   if (fx_raindrops_win == None)
-     {
-	XGCValues           gcv;
-
-	if (first)
-	  {
-	     int                 j;
-
-	     first = 0;
-	     for (i = 0; i < 256; i++)
-		sintab[i] =
-		   (char)(sin(((double)i) * M_PI_2 * 4 / 256) * fx_amplitude);
-	     for (j = 0; j < fx_raindrop_size; j++)
-	       {
-		  for (i = 0; i < fx_raindrop_size; i++)
-		    {
-		       xx = i - fx_raindrop_size2;
-		       yy = j - fx_raindrop_size2;
-		       disttab[i][j] =
-			  (unsigned char)sqrt((double)((xx * xx) + (yy * yy)));
-		    }
-	       }
-	  }
-
-	fx_raindrops_win = EobjGetWin(bgeo);
-
-	if (gc)
-	   EXFreeGC(gc);
-	if (gc1)
-	   EXFreeGC(gc1);
-
-	gcv.subwindow_mode = IncludeInferiors;
-	gc = EXCreateGC(WinGetXwin(fx_raindrops_win), GCSubwindowMode, &gcv);
-	gc1 = EXCreateGC(WinGetXwin(fx_raindrops_win), 0L, &gcv);
-
-	fx_raindrops_draw =
-	   ECreatePixImg(WinGetXwin(fx_raindrops_win), fx_raindrop_size,
-			 fx_raindrop_size);
-	if (!fx_raindrops_draw)
-	   return 0;
-
-	for (i = 0; i < fx_raindrops_number; i++)
-	  {
-	     fx_raindrops[i].buf =
-		ECreatePixImg(WinGetXwin(fx_raindrops_win), fx_raindrop_size,
-			      fx_raindrop_size);
-	     if (fx_raindrops[i].buf)
-		XShmGetImage(disp, WinGetXwin(fx_raindrops_win),
-			     fx_raindrops[i].buf->xim, fx_raindrops[i].x,
-			     fx_raindrops[i].y, 0xffffffff);
-	     if (!fx_raindrops[i].buf)
-		return 0;
-	  }
-     }
-
-   SET_GC_CLIP(bgeo, gc1);
-
-   for (i = 0; i < fx_raindrops_number; i++)
-     {
-	fx_raindrops[i].count++;
-	if (fx_raindrops[i].count == fx_raindrop_duration)
-	  {
-	     int                 j, count = 0;
-	     char                intersect = 1;
-
-	     EClearArea(fx_raindrops_win, fx_raindrops[i].x,
-			fx_raindrops[i].y, fx_raindrop_size, fx_raindrop_size);
-	     fx_raindrops[i].count = 0;
-	     while (intersect)
-	       {
-		  count++;
-		  if (count > 10240)
-		     break;
-		  intersect = 0;
-		  for (j = 0; j < fx_raindrops_number; j++)
-		    {
-		       fx_raindrops[i].x =
-			  rand() % (WinGetW(VROOT) - fx_raindrop_size);
-		       fx_raindrops[i].y =
-			  rand() % (WinGetH(VROOT) - fx_raindrop_size);
-		       if (fx_raindrops[i].x < 0)
-			  fx_raindrops[i].x = 0;
-		       else if (fx_raindrops[i].x >
-				(WinGetW(VROOT) - fx_raindrop_size))
-			  fx_raindrops[i].x = WinGetW(VROOT) - fx_raindrop_size;
-		       if (fx_raindrops[i].y < 0)
-			  fx_raindrops[i].y = 0;
-		       else if (fx_raindrops[i].y >
-				(WinGetH(VROOT) - fx_raindrop_size))
-			  fx_raindrops[i].y = WinGetH(VROOT) - fx_raindrop_size;
-		       if (i != j)
-			 {
-			    if (((fx_raindrops[i].x >= fx_raindrops[j].x)
-				 && (fx_raindrops[i].x <
-				     fx_raindrops[j].x + fx_raindrop_size)
-				 && (fx_raindrops[i].y >= fx_raindrops[j].y)
-				 && (fx_raindrops[i].y <
-				     fx_raindrops[j].y + fx_raindrop_size))
-				||
-				((fx_raindrops
-				  [i].x + fx_raindrop_size >= fx_raindrops[j].x)
-				 && (fx_raindrops[i].x + fx_raindrop_size <
-				     fx_raindrops[j].x + fx_raindrop_size)
-				 && (fx_raindrops[i].y >= fx_raindrops[j].y)
-				 && (fx_raindrops[i].y <
-				     fx_raindrops[j].y + fx_raindrop_size))
-				|| ((fx_raindrops[i].x >= fx_raindrops[j].x)
-				    && (fx_raindrops[i].x <
-					fx_raindrops[j].x + fx_raindrop_size)
-				    && (fx_raindrops[i].y + fx_raindrop_size >=
-					fx_raindrops[j].y)
-				    && (fx_raindrops[i].y + fx_raindrop_size <
-					fx_raindrops[j].y + fx_raindrop_size))
-				||
-				((fx_raindrops
-				  [i].x + fx_raindrop_size >= fx_raindrops[j].x)
-				 && (fx_raindrops[i].x + fx_raindrop_size <
-				     fx_raindrops[j].x + fx_raindrop_size)
-				 && (fx_raindrops[i].y + fx_raindrop_size >=
-				     fx_raindrops[j].y)
-				 && (fx_raindrops[i].y + fx_raindrop_size <
-				     fx_raindrops[j].y + fx_raindrop_size)))
-			       intersect = 1;
-			 }
-		    }
-	       }
-	     XShmGetImage(disp, WinGetXwin(fx_raindrops_win),
-			  fx_raindrops[i].buf->xim,
-			  fx_raindrops[i].x, fx_raindrops[i].y, 0xffffffff);
-	  }
-	percent_done =
-	   1 + ((fx_raindrops[i].count << 8) / fx_raindrop_duration);
-	for (y = 0; y < fx_raindrop_size; y++)
-	  {
-	     for (x = 0; x < fx_raindrop_size; x++)
-	       {
-		  int                 dist;
-
-		  dist = disttab[x][y];
-		  if (dist > fx_raindrop_size2)
-		     XPutPixel(fx_raindrops_draw->xim, x, y,
-			       XGetPixel(fx_raindrops[i].buf->xim, x, y));
-		  else
-		    {
-		       int                 percent;
-
-		       percent = 1 + ((dist << 8) / fx_raindrop_size2);
-		       if (percent > percent_done)
-			  XPutPixel(fx_raindrops_draw->xim, x, y,
-				    XGetPixel(fx_raindrops[i].buf->xim, x, y));
-		       else
-			 {
-			    int                 varx, vary;
-			    int                 phase, divisor, multiplier;
-
-			    phase =
-			       ((percent - percent_done) * fx_frequency) & 0xff;
-			    xx = x - fx_raindrop_size2;
-			    yy = y - fx_raindrop_size2;
-			    divisor = 1 + (dist << 8);
-			    multiplier =
-			       (int)sintab[phase] * (256 - percent_done);
-			    varx = ((-xx) * multiplier) / divisor;
-			    vary = ((-yy) * multiplier) / divisor;
-			    xx = x + varx;
-			    yy = y + vary;
-			    if (xx < 0)
-			       xx = 0;
-			    else if (xx >= fx_raindrop_size)
-			       xx = fx_raindrop_size - 1;
-			    if (yy < 0)
-			       yy = 0;
-			    else if (yy >= fx_raindrop_size)
-			       yy = fx_raindrop_size - 1;
-			    XPutPixel(fx_raindrops_draw->xim, x, y,
-				      XGetPixel(fx_raindrops[i].buf->xim, xx,
-						yy));
-			 }
-		    }
-	       }
-	  }
-	XShmPutImage(disp, WinGetXwin(fx_raindrops_win), gc1,
-		     fx_raindrops_draw->xim, 0, 0,
-		     fx_raindrops[i].x, fx_raindrops[i].y, fx_raindrop_size,
-		     fx_raindrop_size, False);
-	ESync(0);
-     }
-
-   return 1;
-}
-
-static void
-FX_Raindrops_Init(const char *name __UNUSED__)
-{
-   int                 i;
-
-   fx_raindrops_win = None;
-   for (i = 0; i < fx_raindrops_number; i++)
-     {
-	fx_raindrops[i].count = rand() % fx_raindrop_duration;
-	fx_raindrops[i].x = rand() % (WinGetW(VROOT) - fx_raindrop_size);
-	fx_raindrops[i].y = rand() % (WinGetH(VROOT) - fx_raindrop_size);
-     }
-   TIMER_ADD(fx_raindrops_timer, 0.066, FX_raindrops_timeout, NULL);
-}
-
-static void
-FX_Raindrops_Desk(void)
-{
-   fx_raindrops_win = None;
-}
-
-static void
-FX_Raindrops_Quit(void)
-{
-   int                 i;
-
-   TIMER_DEL(fx_raindrops_timer);
-   for (i = 0; i < fx_raindrops_number; i++)
-     {
-	EClearArea(fx_raindrops_win, fx_raindrops[i].x, fx_raindrops[i].y,
-		   fx_raindrop_size, fx_raindrop_size);
-	if (fx_raindrops[i].buf)
-	   EDestroyPixImg(fx_raindrops[i].buf);
-	fx_raindrops[i].buf = NULL;
-     }
-   if (fx_raindrops_draw)
-      EDestroyPixImg(fx_raindrops_draw);
-   fx_raindrops_draw = NULL;
-   fx_raindrops_win = None;
-}
-
-static void
-FX_Raindrops_Pause(void)
-{
-   static char         paused = 0;
-
-   if (!paused)
-     {
-	FX_Raindrops_Quit();
-	paused = 1;
-     }
-   else
-     {
-	FX_Raindrops_Init(NULL);
-	paused = 0;
-     }
-}
-
-#endif /* E_FX_RAINDROPS */
-
 /****************************** WAVES ***************************************/
 /* by tsade :)                                                              */
 /****************************************************************************/
@@ -498,10 +212,9 @@ FX_Wave_timeout(void *data __UNUSED__)
 
 	fx_wave_above =
 	   ECreatePixmap(fx_wave_win, WinGetW(VROOT), FX_WAVE_WATERH * 2, 0);
-	if (gc)
-	   EXFreeGC(gc);
-	if (gc1)
-	   EXFreeGC(gc1);
+
+	EXFreeGC(gc);
+	EXFreeGC(gc1);
 	gcv.subwindow_mode = IncludeInferiors;
 	gc = EXCreateGC(WinGetXwin(fx_wave_win), GCSubwindowMode, &gcv);
 	gc1 = EXCreateGC(WinGetXwin(fx_wave_win), 0L, &gcv);
@@ -607,6 +320,8 @@ static void
 FX_Waves_Quit(void)
 {
    TIMER_DEL(fx_wave_timer);
+   if (!fx_wave_win)
+      return;
    EClearArea(fx_wave_win, 0, WinGetH(VROOT) - FX_WAVE_WATERH,
 	      WinGetW(VROOT), FX_WAVE_WATERH);
 }
@@ -628,6 +343,284 @@ FX_Waves_Pause(void)
      }
 }
 
+#if E_FX_RAINDROPS
+#include "piximg.h"
+
+/****************************** RAIN DROPS **********************************/
+
+#define fx_raindrop_size 96
+#define fx_raindrop_size2 (fx_raindrop_size / 2)
+#define fx_raindrop_duration 32
+#define fx_frequency 4
+#define fx_amplitude 48
+static Win          fx_raindrops_win = NULL;
+static int          fx_raindrops_number = 4;
+static PixImg      *fx_raindrops_draw = NULL;
+static Timer       *fx_raindrops_timer = NULL;
+
+typedef struct {
+   int                 x, y;
+   int                 count;
+   PixImg             *buf;
+} DropContext;
+
+static DropContext  fx_raindrops[4];
+
+static int
+FX_raindrops_timeout(void *data __UNUSED__)
+{
+   static GC           gc1 = 0, gc = 0;
+   int                 i, x, y, xx, yy;
+   int                 percent_done;
+   static char         first = 1;
+   static char         sintab[256];
+   static unsigned char disttab[fx_raindrop_size][fx_raindrop_size];
+   EObj               *bgeo;
+
+   bgeo = DeskGetBackgroundObj(DesksGetCurrent());
+
+   if (fx_raindrops_win == None)
+     {
+	XGCValues           gcv;
+
+	if (first)
+	  {
+	     int                 j;
+
+	     first = 0;
+	     for (i = 0; i < 256; i++)
+		sintab[i] =
+		   (char)(sin(((double)i) * M_PI_2 * 4 / 256) * fx_amplitude);
+	     for (j = 0; j < fx_raindrop_size; j++)
+	       {
+		  for (i = 0; i < fx_raindrop_size; i++)
+		    {
+		       xx = i - fx_raindrop_size2;
+		       yy = j - fx_raindrop_size2;
+		       disttab[i][j] =
+			  (unsigned char)sqrt((double)((xx * xx) + (yy * yy)));
+		    }
+	       }
+	  }
+
+	fx_raindrops_win = EobjGetWin(bgeo);
+
+	EXFreeGC(gc);
+	EXFreeGC(gc1);
+	gcv.subwindow_mode = IncludeInferiors;
+	gc = EXCreateGC(WinGetXwin(fx_raindrops_win), GCSubwindowMode, &gcv);
+	gc1 = EXCreateGC(WinGetXwin(fx_raindrops_win), 0L, &gcv);
+
+	fx_raindrops_draw = PixImgCreate(fx_raindrop_size, fx_raindrop_size);
+	if (!fx_raindrops_draw)
+	   return 0;
+
+	for (i = 0; i < fx_raindrops_number; i++)
+	  {
+	     fx_raindrops[i].buf =
+		PixImgCreate(fx_raindrop_size, fx_raindrop_size);
+	     if (fx_raindrops[i].buf)
+		PixImgFill(fx_raindrops[i].buf, WinGetXwin(fx_raindrops_win),
+			   fx_raindrops[i].x, fx_raindrops[i].y);
+	     if (!fx_raindrops[i].buf)
+		return 0;
+	  }
+     }
+
+   SET_GC_CLIP(bgeo, gc1);
+
+   for (i = 0; i < fx_raindrops_number; i++)
+     {
+	fx_raindrops[i].count++;
+	if (fx_raindrops[i].count == fx_raindrop_duration)
+	  {
+	     int                 j, count = 0;
+	     char                intersect = 1;
+
+	     EClearArea(fx_raindrops_win, fx_raindrops[i].x,
+			fx_raindrops[i].y, fx_raindrop_size, fx_raindrop_size);
+	     fx_raindrops[i].count = 0;
+	     while (intersect)
+	       {
+		  count++;
+		  if (count > 10240)
+		     break;
+		  intersect = 0;
+		  for (j = 0; j < fx_raindrops_number; j++)
+		    {
+		       fx_raindrops[i].x =
+			  rand() % (WinGetW(VROOT) - fx_raindrop_size);
+		       fx_raindrops[i].y =
+			  rand() % (WinGetH(VROOT) - fx_raindrop_size);
+		       if (fx_raindrops[i].x < 0)
+			  fx_raindrops[i].x = 0;
+		       else if (fx_raindrops[i].x >
+				(WinGetW(VROOT) - fx_raindrop_size))
+			  fx_raindrops[i].x = WinGetW(VROOT) - fx_raindrop_size;
+		       if (fx_raindrops[i].y < 0)
+			  fx_raindrops[i].y = 0;
+		       else if (fx_raindrops[i].y >
+				(WinGetH(VROOT) - fx_raindrop_size))
+			  fx_raindrops[i].y = WinGetH(VROOT) - fx_raindrop_size;
+		       if (i != j)
+			 {
+			    if (((fx_raindrops[i].x >= fx_raindrops[j].x)
+				 && (fx_raindrops[i].x <
+				     fx_raindrops[j].x + fx_raindrop_size)
+				 && (fx_raindrops[i].y >= fx_raindrops[j].y)
+				 && (fx_raindrops[i].y <
+				     fx_raindrops[j].y + fx_raindrop_size))
+				||
+				((fx_raindrops
+				  [i].x + fx_raindrop_size >= fx_raindrops[j].x)
+				 && (fx_raindrops[i].x + fx_raindrop_size <
+				     fx_raindrops[j].x + fx_raindrop_size)
+				 && (fx_raindrops[i].y >= fx_raindrops[j].y)
+				 && (fx_raindrops[i].y <
+				     fx_raindrops[j].y + fx_raindrop_size))
+				|| ((fx_raindrops[i].x >= fx_raindrops[j].x)
+				    && (fx_raindrops[i].x <
+					fx_raindrops[j].x + fx_raindrop_size)
+				    && (fx_raindrops[i].y + fx_raindrop_size >=
+					fx_raindrops[j].y)
+				    && (fx_raindrops[i].y + fx_raindrop_size <
+					fx_raindrops[j].y + fx_raindrop_size))
+				||
+				((fx_raindrops
+				  [i].x + fx_raindrop_size >= fx_raindrops[j].x)
+				 && (fx_raindrops[i].x + fx_raindrop_size <
+				     fx_raindrops[j].x + fx_raindrop_size)
+				 && (fx_raindrops[i].y + fx_raindrop_size >=
+				     fx_raindrops[j].y)
+				 && (fx_raindrops[i].y + fx_raindrop_size <
+				     fx_raindrops[j].y + fx_raindrop_size)))
+			       intersect = 1;
+			 }
+		    }
+	       }
+	     PixImgFill(fx_raindrops[i].buf, WinGetXwin(fx_raindrops_win),
+			fx_raindrops[i].x, fx_raindrops[i].y);
+	  }
+	percent_done =
+	   1 + ((fx_raindrops[i].count << 8) / fx_raindrop_duration);
+	for (y = 0; y < fx_raindrop_size; y++)
+	  {
+	     for (x = 0; x < fx_raindrop_size; x++)
+	       {
+		  int                 dist;
+
+		  dist = disttab[x][y];
+		  if (dist > fx_raindrop_size2)
+		     XPutPixel(fx_raindrops_draw->xim, x, y,
+			       XGetPixel(fx_raindrops[i].buf->xim, x, y));
+		  else
+		    {
+		       int                 percent;
+
+		       percent = 1 + ((dist << 8) / fx_raindrop_size2);
+		       if (percent > percent_done)
+			  XPutPixel(fx_raindrops_draw->xim, x, y,
+				    XGetPixel(fx_raindrops[i].buf->xim, x, y));
+		       else
+			 {
+			    int                 varx, vary;
+			    int                 phase, divisor, multiplier;
+
+			    phase =
+			       ((percent - percent_done) * fx_frequency) & 0xff;
+			    xx = x - fx_raindrop_size2;
+			    yy = y - fx_raindrop_size2;
+			    divisor = 1 + (dist << 8);
+			    multiplier =
+			       (int)sintab[phase] * (256 - percent_done);
+			    varx = ((-xx) * multiplier) / divisor;
+			    vary = ((-yy) * multiplier) / divisor;
+			    xx = x + varx;
+			    yy = y + vary;
+			    if (xx < 0)
+			       xx = 0;
+			    else if (xx >= fx_raindrop_size)
+			       xx = fx_raindrop_size - 1;
+			    if (yy < 0)
+			       yy = 0;
+			    else if (yy >= fx_raindrop_size)
+			       yy = fx_raindrop_size - 1;
+			    XPutPixel(fx_raindrops_draw->xim, x, y,
+				      XGetPixel(fx_raindrops[i].buf->xim, xx,
+						yy));
+			 }
+		    }
+	       }
+	  }
+	PixImgPaste(fx_raindrops_draw, WinGetXwin(fx_raindrops_win), gc1,
+		    0, 0, fx_raindrop_size, fx_raindrop_size,
+		    fx_raindrops[i].x, fx_raindrops[i].y);
+	ESync(0);
+     }
+
+   return 1;
+}
+
+static void
+FX_Raindrops_Init(const char *name __UNUSED__)
+{
+   int                 i;
+
+   fx_raindrops_win = None;
+   for (i = 0; i < fx_raindrops_number; i++)
+     {
+	fx_raindrops[i].count = rand() % fx_raindrop_duration;
+	fx_raindrops[i].x = rand() % (WinGetW(VROOT) - fx_raindrop_size);
+	fx_raindrops[i].y = rand() % (WinGetH(VROOT) - fx_raindrop_size);
+     }
+   TIMER_ADD(fx_raindrops_timer, 0.066, FX_raindrops_timeout, NULL);
+}
+
+static void
+FX_Raindrops_Desk(void)
+{
+   fx_raindrops_win = None;
+}
+
+static void
+FX_Raindrops_Quit(void)
+{
+   int                 i;
+
+   TIMER_DEL(fx_raindrops_timer);
+   for (i = 0; i < fx_raindrops_number; i++)
+     {
+	EClearArea(fx_raindrops_win, fx_raindrops[i].x, fx_raindrops[i].y,
+		   fx_raindrop_size, fx_raindrop_size);
+	if (fx_raindrops[i].buf)
+	   PixImgDestroy(fx_raindrops[i].buf);
+	fx_raindrops[i].buf = NULL;
+     }
+   if (fx_raindrops_draw)
+      PixImgDestroy(fx_raindrops_draw);
+   fx_raindrops_draw = NULL;
+   fx_raindrops_win = None;
+}
+
+static void
+FX_Raindrops_Pause(void)
+{
+   static char         paused = 0;
+
+   if (!paused)
+     {
+	FX_Raindrops_Quit();
+	paused = 1;
+     }
+   else
+     {
+	FX_Raindrops_Init(NULL);
+	paused = 0;
+     }
+}
+
+#endif /* E_FX_RAINDROPS */
+
 #if E_FX_IMAGESPINNER
 
 /****************************** IMAGESPINNER ********************************/
@@ -980,9 +973,9 @@ FxIpc(const char *params)
 	else if (!strcmp(word2, "off"))
 	   FX_Op(word1, FX_OP_STOP);
 	else if (!strcmp(word2, "?"))
-	   IpcPrintf("%s: %s", word1, FX_IsOn(word1) ? "on" : "off");
+	   IpcPrintf("%s: %s\n", word1, FX_IsOn(word1) ? "on" : "off");
 	else
-	   IpcPrintf("Error: unknown mode specified");
+	   IpcPrintf("Error: unknown mode specified\n");
      }
 }
 
