@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -45,7 +45,8 @@ struct _imagestate {
    char                rotate;
    EImage             *im;
    EImageBorder       *border;
-   EColor              bg, hi, lo, hihi, lolo;
+   unsigned int        bg, hi, lo, hihi, lolo;
+   unsigned int        bg_pixel, hi_pixel, lo_pixel, hihi_pixel, lolo_pixel;
 };
 
 typedef struct {
@@ -69,8 +70,6 @@ static ImageClass  *ImageclassGetFallback(void);
 #ifdef ENABLE_THEME_TRANSPARENCY
 
 static EImageColorModifier *icm = NULL;
-static unsigned char gray[256];
-static unsigned char alpha[256];
 
 static int          prev_alpha = -1;
 
@@ -90,6 +89,8 @@ static void
 TransparencyMakeColorModifier(void)
 {
    int                 i;
+   unsigned char       gray[256];
+   unsigned char       alpha[256];
 
    for (i = 0; i < 256; i++)
      {
@@ -175,11 +176,11 @@ ImagestateColorsSetGray(ImageState * is,
 			unsigned int hihi, unsigned int hi,
 			unsigned int bg, unsigned int lo, unsigned int lolo)
 {
-   SET_COLOR(&(is->hihi), hihi, hihi, hihi);
-   SET_COLOR(&(is->hi), hi, hi, hi);
-   SET_COLOR(&(is->bg), bg, bg, bg);
-   SET_COLOR(&(is->lo), lo, lo, lo);
-   SET_COLOR(&(is->lolo), lolo, lolo, lolo);
+   COLOR32_FROM_RGB(is->hihi, hihi, hihi, hihi);
+   COLOR32_FROM_RGB(is->hi, hi, hi, hi);
+   COLOR32_FROM_RGB(is->bg, bg, bg, bg);
+   COLOR32_FROM_RGB(is->lo, lo, lo, lo);
+   COLOR32_FROM_RGB(is->lolo, lolo, lolo, lolo);
 }
 
 static ImageState  *
@@ -199,7 +200,6 @@ ImagestateCreate(const char *file)
    return is;
 }
 
-#if ENABLE_DESTROY
 static void
 ImagestateDestroy(ImageState * is)
 {
@@ -217,15 +217,19 @@ ImagestateDestroy(ImageState * is)
    Efree(is);
 }
 
-static void
-FreeImageStateArray(ImageStateArray * isa)
+static ImageState  *
+ImagestateSet(ImageState ** isp, const char *name)
 {
-   ImagestateDestroy(isa->normal);
-   ImagestateDestroy(isa->hilited);
-   ImagestateDestroy(isa->clicked);
-   ImagestateDestroy(isa->disabled);
+   ImageState         *is;
+
+   is = ImagestateCreate(name);
+
+   if (*isp)
+      ImagestateDestroy(*isp);
+   *isp = is;
+
+   return is;
 }
-#endif /* ENABLE_DESTROY */
 
 static void
 ImagestateColorsAlloc(ImageState * is)
@@ -233,11 +237,11 @@ ImagestateColorsAlloc(ImageState * is)
    if (!is || is->got_colors)
       return;
 
-   EAllocColor(WinGetCmap(VROOT), &is->bg);
-   EAllocColor(WinGetCmap(VROOT), &is->hi);
-   EAllocColor(WinGetCmap(VROOT), &is->lo);
-   EAllocColor(WinGetCmap(VROOT), &is->hihi);
-   EAllocColor(WinGetCmap(VROOT), &is->lolo);
+   is->bg_pixel = EAllocColor(WinGetCmap(VROOT), is->bg);
+   is->hi_pixel = EAllocColor(WinGetCmap(VROOT), is->hi);
+   is->lo_pixel = EAllocColor(WinGetCmap(VROOT), is->lo);
+   is->hihi_pixel = EAllocColor(WinGetCmap(VROOT), is->hihi);
+   is->lolo_pixel = EAllocColor(WinGetCmap(VROOT), is->lolo);
 
    is->got_colors = 1;
 }
@@ -308,24 +312,20 @@ ImageclassCreate(const char *name)
    ecore_list_prepend(iclass_list, ic);
 
    ic->name = Estrdup(name);
-   ic->norm.normal = ic->norm.hilited = ic->norm.clicked = ic->norm.disabled =
-      NULL;
-   ic->active.normal = ic->active.hilited = ic->active.clicked =
-      ic->active.disabled = NULL;
-   ic->sticky.normal = ic->sticky.hilited = ic->sticky.clicked =
-      ic->sticky.disabled = NULL;
-   ic->sticky_active.normal = ic->sticky_active.hilited =
-      ic->sticky_active.clicked = ic->sticky_active.disabled = NULL;
-   ic->padding.left = 0;
-   ic->padding.right = 0;
-   ic->padding.top = 0;
-   ic->padding.bottom = 0;
-   ic->ref_count = 0;
 
    return ic;
 }
 
 #if ENABLE_DESTROY
+static void
+FreeImageStateArray(ImageStateArray * isa)
+{
+   ImagestateDestroy(isa->normal);
+   ImagestateDestroy(isa->hilited);
+   ImagestateDestroy(isa->clicked);
+   ImagestateDestroy(isa->disabled);
+}
+
 static void
 ImageclassDestroy(ImageClass * ic)
 {
@@ -415,10 +415,7 @@ ImageclassFind(const char *name, int fallback)
 static void
 ImageclassPopulate(ImageClass * ic)
 {
-   if (!ic)
-      return;
-
-   if (!ic->norm.normal)
+   if (!ic || !ic->norm.normal)
       return;
 
    ISTATE_SET_STATE(norm.hilited, norm.normal);
@@ -496,7 +493,7 @@ ImageclassConfigLoad(FILE * fs)
 		ic->sticky_active = ic2->sticky_active;
 		ic->padding = ic2->padding;
 	     }
-	     break;
+	     continue;
 	  case ICLASS_PADDING:
 	     l = r = t = b = 0;
 	     sscanf(p2, "%i %i %i %i", &l, &r, &t, &b);
@@ -508,52 +505,52 @@ ImageclassConfigLoad(FILE * fs)
 	  case CONFIG_DESKTOP:
 	     /* don't ask... --mandrake */
 	  case ICLASS_NORMAL:
-	     ic->norm.normal = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->norm.normal, s2);
 	     continue;
 	  case ICLASS_CLICKED:
-	     ic->norm.clicked = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->norm.clicked, s2);
 	     continue;
 	  case ICLASS_HILITED:
-	     ic->norm.hilited = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->norm.hilited, s2);
 	     continue;
 	  case ICLASS_DISABLED:
-	     ic->norm.disabled = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->norm.disabled, s2);
 	     continue;
 	  case ICLASS_STICKY_NORMAL:
-	     ic->sticky.normal = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->sticky.normal, s2);
 	     continue;
 	  case ICLASS_STICKY_CLICKED:
-	     ic->sticky.clicked = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->sticky.clicked, s2);
 	     continue;
 	  case ICLASS_STICKY_HILITED:
-	     ic->sticky.hilited = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->sticky.hilited, s2);
 	     continue;
 	  case ICLASS_STICKY_DISABLED:
-	     ic->sticky.disabled = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->sticky.disabled, s2);
 	     continue;
 	  case ICLASS_ACTIVE_NORMAL:
-	     ic->active.normal = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->active.normal, s2);
 	     continue;
 	  case ICLASS_ACTIVE_CLICKED:
-	     ic->active.clicked = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->active.clicked, s2);
 	     continue;
 	  case ICLASS_ACTIVE_HILITED:
-	     ic->active.hilited = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->active.hilited, s2);
 	     continue;
 	  case ICLASS_ACTIVE_DISABLED:
-	     ic->active.disabled = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->active.disabled, s2);
 	     continue;
 	  case ICLASS_STICKY_ACTIVE_NORMAL:
-	     ic->sticky_active.normal = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->sticky_active.normal, s2);
 	     continue;
 	  case ICLASS_STICKY_ACTIVE_CLICKED:
-	     ic->sticky_active.clicked = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->sticky_active.clicked, s2);
 	     continue;
 	  case ICLASS_STICKY_ACTIVE_HILITED:
-	     ic->sticky_active.hilited = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->sticky_active.hilited, s2);
 	     continue;
 	  case ICLASS_STICKY_ACTIVE_DISABLED:
-	     ic->sticky_active.disabled = is = ImagestateCreate(s2);
+	     is = ImagestateSet(&ic->sticky_active.disabled, s2);
 	     continue;
 	  }
 
@@ -564,7 +561,8 @@ ImageclassConfigLoad(FILE * fs)
 	switch (i1)
 	  {
 	  case ICLASS_LRTB:
-	     is->border = EMALLOC(EImageBorder, 1);
+	     if (!is->border)
+		is->border = EMALLOC(EImageBorder, 1);
 	     l = r = t = b = 0;
 	     sscanf(p2, "%i %i %i %i", &l, &r, &t, &b);
 	     is->border->left = l;
@@ -585,26 +583,17 @@ ImageclassConfigLoad(FILE * fs)
 	     is->rotate = strtoul(s2, NULL, 0);
 	     continue;
 	  case ICLASS_BEVEL:
-#define _R(x) (((x) >> 16) & 0xff)
-#define _G(x) (((x) >>  8) & 0xff)
-#define _B(x) (((x)      ) & 0xff)
-#define INT_TO_COLOR(xc, rgb) \
-    do { (&(xc))->red = _R(rgb); (&(xc))->green = _G(rgb); (&(xc))->blue = _B(rgb); } while(0)
 	     {
-		int                 n, bevel, hihi, hi, bg, lo, lolo;
+		int                 n, bevel;
 
 		n = sscanf(p2, "%i %i %i %i %i %i",
-			   &bevel, &hihi, &hi, &bg, &lo, &lolo);
+			   &bevel, &is->hihi, &is->hi, &is->bg, &is->lo,
+			   &is->lolo);
 		if (n < 6)
 		   goto not_ok;
 		is->bevelstyle = bevel;
-		INT_TO_COLOR(is->hihi, hihi);
-		INT_TO_COLOR(is->hi, hi);
-		INT_TO_COLOR(is->bg, bg);
-		INT_TO_COLOR(is->lo, lo);
-		INT_TO_COLOR(is->lolo, lolo);
 	     }
-	     break;
+	     continue;
 	  default:
 	     goto not_ok;
 	  }
@@ -998,87 +987,87 @@ ImagestateDrawBevel(ImageState * is, Drawable win, GC gc,
    switch (is->bevelstyle)
      {
      case BEVEL_AMIGA:
-	XSetForeground(disp, gc, is->hihi.pixel);
+	XSetForeground(disp, gc, is->hihi_pixel);
 	LINE(0, 0, w - 2, 0);
 	LINE(0, 0, 0, h - 2);
-	XSetForeground(disp, gc, is->lolo.pixel);
+	XSetForeground(disp, gc, is->lolo_pixel);
 	LINE(1, h - 1, w - 1, h - 1);
 	LINE(w - 1, 1, w - 1, h - 1);
 	break;
      case BEVEL_MOTIF:
-	XSetForeground(disp, gc, is->hi.pixel);
+	XSetForeground(disp, gc, is->hi_pixel);
 	LINE(0, 0, w - 1, 0);
 	LINE(0, 0, 0, h - 1);
 	LINE(1, 1, w - 2, 1);
 	LINE(1, 1, 1, h - 2);
-	XSetForeground(disp, gc, is->lo.pixel);
+	XSetForeground(disp, gc, is->lo_pixel);
 	LINE(0, h - 1, w - 1, h - 1);
 	LINE(w - 1, 1, w - 1, h - 1);
 	LINE(1, h - 2, w - 2, h - 2);
 	LINE(w - 2, 2, w - 2, h - 2);
 	break;
      case BEVEL_NEXT:
-	XSetForeground(disp, gc, is->hihi.pixel);
+	XSetForeground(disp, gc, is->hihi_pixel);
 	LINE(0, 0, w - 1, 0);
 	LINE(0, 0, 0, h - 1);
-	XSetForeground(disp, gc, is->hi.pixel);
+	XSetForeground(disp, gc, is->hi_pixel);
 	LINE(1, 1, w - 2, 1);
 	LINE(1, 1, 1, h - 2);
-	XSetForeground(disp, gc, is->lolo.pixel);
+	XSetForeground(disp, gc, is->lolo_pixel);
 	LINE(1, h - 1, w - 1, h - 1);
 	LINE(w - 1, 1, w - 1, h - 1);
-	XSetForeground(disp, gc, is->lo.pixel);
+	XSetForeground(disp, gc, is->lo_pixel);
 	LINE(2, h - 2, w - 2, h - 2);
 	LINE(w - 2, 2, w - 2, h - 2);
 	break;
      case BEVEL_DOUBLE:
-	XSetForeground(disp, gc, is->hi.pixel);
+	XSetForeground(disp, gc, is->hi_pixel);
 	LINE(0, 0, w - 2, 0);
 	LINE(0, 0, 0, h - 2);
-	XSetForeground(disp, gc, is->lo.pixel);
+	XSetForeground(disp, gc, is->lo_pixel);
 	LINE(1, 1, w - 3, 1);
 	LINE(1, 1, 1, h - 3);
-	XSetForeground(disp, gc, is->lo.pixel);
+	XSetForeground(disp, gc, is->lo_pixel);
 	LINE(1, h - 1, w - 1, h - 1);
 	LINE(w - 1, 1, w - 1, h - 1);
-	XSetForeground(disp, gc, is->hi.pixel);
+	XSetForeground(disp, gc, is->hi_pixel);
 	LINE(2, h - 2, w - 2, h - 2);
 	LINE(w - 2, 2, w - 2, h - 2);
 	break;
      case BEVEL_WIDEDOUBLE:
-	XSetForeground(disp, gc, is->hihi.pixel);
+	XSetForeground(disp, gc, is->hihi_pixel);
 	LINE(0, 0, w - 1, 0);
 	LINE(0, 0, 0, h - 1);
-	XSetForeground(disp, gc, is->hi.pixel);
+	XSetForeground(disp, gc, is->hi_pixel);
 	LINE(1, 1, w - 2, 1);
 	LINE(1, 1, 1, h - 2);
 	LINE(3, h - 4, w - 4, h - 4);
 	LINE(w - 4, 3, w - 4, h - 4);
-	XSetForeground(disp, gc, is->lolo.pixel);
+	XSetForeground(disp, gc, is->lolo_pixel);
 	LINE(1, h - 1, w - 1, h - 1);
 	LINE(w - 1, 1, w - 1, h - 1);
-	XSetForeground(disp, gc, is->lo.pixel);
+	XSetForeground(disp, gc, is->lo_pixel);
 	LINE(2, h - 2, w - 2, h - 2);
 	LINE(w - 2, 2, w - 2, h - 2);
 	LINE(3, 3, w - 4, 3);
 	LINE(3, 3, 3, h - 4);
 	break;
      case BEVEL_THINPOINT:
-	XSetForeground(disp, gc, is->hi.pixel);
+	XSetForeground(disp, gc, is->hi_pixel);
 	LINE(0, 0, w - 2, 0);
 	LINE(0, 0, 0, h - 2);
-	XSetForeground(disp, gc, is->lo.pixel);
+	XSetForeground(disp, gc, is->lo_pixel);
 	LINE(1, h - 1, w - 1, h - 1);
 	LINE(w - 1, 1, w - 1, h - 1);
-	XSetForeground(disp, gc, is->hihi.pixel);
+	XSetForeground(disp, gc, is->hihi_pixel);
 	LINE(0, 0, 1, 0);
 	LINE(0, 0, 0, 1);
-	XSetForeground(disp, gc, is->lolo.pixel);
+	XSetForeground(disp, gc, is->lolo_pixel);
 	LINE(w - 2, h - 1, w - 1, h - 1);
 	LINE(w - 1, h - 2, w - 1, h - 1);
 	break;
      case BEVEL_THICKPOINT:
-	XSetForeground(disp, gc, is->hi.pixel);
+	XSetForeground(disp, gc, is->hi_pixel);
 	RECT(x, y, w - 1, h - 1);
 	break;
      default:
@@ -1095,7 +1084,7 @@ ImagestateDrawNoImg(ImageState * is, Drawable draw, int x, int y, int w, int h)
 
    gc = EXCreateGC(draw, 0, NULL);
    XSetFillStyle(disp, gc, FillSolid);
-   XSetForeground(disp, gc, is->bg.pixel);
+   XSetForeground(disp, gc, is->bg_pixel);
    XFillRectangle(disp, draw, gc, x, y, w, h);
    if (is->bevelstyle != BEVEL_NONE)
       ImagestateDrawBevel(is, draw, gc, x, y, w, h);
@@ -1143,7 +1132,8 @@ ITApply(Win win, ImageClass * ic, ImageState * is,
 	  {
 	     Pixmap              pmap = pmm.pmap;
 
-	     if ((ts && text) || (is->bevelstyle != BEVEL_NONE))
+	     if ((ts && text) || (is->bevelstyle != BEVEL_NONE) ||
+		 (flags & ITA_BGPMAP))
 	       {
 		  if (pmm.type != 0)
 		    {
@@ -1156,7 +1146,6 @@ ITApply(Win win, ImageClass * ic, ImageState * is,
 		       GC                  gc;
 
 		       gc = EXCreateGC(WinGetXwin(win), 0, NULL);
-		       Eprintf("%s: bevel=%d\n", __func__, is->bevelstyle);
 		       ImagestateDrawBevel(is, pmap, gc, 0, 0, w, h);
 		       EXFreeGC(gc);
 		    }
@@ -1164,7 +1153,8 @@ ITApply(Win win, ImageClass * ic, ImageState * is,
 		  if (ts && text)
 		     TextstateTextDraw(ts, win, pmap, text, 0, 0, w, h,
 				       &(ic->padding), 0,
-				       TextclassGetJustification(tc), flags);
+				       TextclassGetJustification(tc),
+				       flags & ITA_JUSTV);
 	       }
 
 	     /* Set window pixmap */
@@ -1194,7 +1184,7 @@ ITApply(Win win, ImageClass * ic, ImageState * is,
 
 	if (is->bevelstyle == BEVEL_NONE && !text)
 	  {
-	     ESetWindowBackground(win, is->bg.pixel);
+	     ESetWindowBackground(win, is->bg_pixel);
 	  }
 	else
 	  {
@@ -1205,7 +1195,8 @@ ITApply(Win win, ImageClass * ic, ImageState * is,
 	     if (ts && text)
 		TextstateTextDraw(ts, win, pmap, text, 0, 0, w, h,
 				  &(ic->padding), 0,
-				  TextclassGetJustification(tc), flags);
+				  TextclassGetJustification(tc),
+				  flags & ITA_JUSTV);
 	  }
      }
    EClearWindow(win);
@@ -1354,17 +1345,17 @@ ImageclassGetFallback(void)
 
    ic->active.normal = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->active.normal, 255, 255, 0, 0, 0);
-   SET_COLOR(&(ic->active.normal->bg), 180, 140, 160);
+   COLOR32_FROM_RGB(ic->active.normal->bg, 180, 140, 160);
    ic->active.normal->bevelstyle = BEVEL_AMIGA;
 
    ic->active.hilited = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->active.hilited, 255, 255, 0, 0, 0);
-   SET_COLOR(&(ic->active.hilited->bg), 230, 190, 210);
+   COLOR32_FROM_RGB(ic->active.hilited->bg, 230, 190, 210);
    ic->active.hilited->bevelstyle = BEVEL_AMIGA;
 
    ic->active.clicked = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->active.clicked, 0, 0, 0, 255, 255);
-   SET_COLOR(&(ic->active.clicked->bg), 230, 190, 210);
+   COLOR32_FROM_RGB(ic->active.clicked->bg, 230, 190, 210);
    ic->active.clicked->bevelstyle = BEVEL_AMIGA;
 
    ic->padding.left = 4;
@@ -1436,12 +1427,7 @@ ImageclassIpc(const char *params)
 	return;
      }
 
-   if (!strcmp(param2, "create"))
-     {
-	/* Not implemented */
-	return;
-     }
-   else if (!strcmp(param2, "free_pixmap"))
+   if (!strcmp(param2, "free_pixmap"))
      {
 	Pixmap              pmap;
 
@@ -1457,17 +1443,7 @@ ImageclassIpc(const char *params)
 	return;
      }
 
-   if (!strcmp(param2, "delete"))
-     {
-#if ENABLE_DESTROY
-	ImageclassDestroy(ic);
-#endif
-     }
-   else if (!strcmp(param2, "modify"))
-     {
-	/* Not implemented */
-     }
-   else if (!strcmp(param2, "get_padding"))
+   if (!strcmp(param2, "get_padding"))
      {
 	IpcPrintf("%i %i %i %i\n",
 		  ic->padding.left, ic->padding.right,
