@@ -111,11 +111,12 @@ MenuLoadFromDirectory(Menu * m)
    if (stat(dir, &st) < 0)
       return 1;
 
+   Esnprintf(cs, sizeof(cs), "%s/cached/img/%s",
+	     EDirUserCache(), _dircache_filename(ss, sizeof(ss), &st));
+
    if (Mode.backgrounds.force_scan)
       goto skip_dir_cache;
 
-   Esnprintf(cs, sizeof(cs), "%s/cached/img/%s",
-	     EDirUserCache(), _dircache_filename(ss, sizeof(ss), &st));
    if (exists(cs))
      {
 	/* cached dir listing - use it */
@@ -250,11 +251,21 @@ void
 ScanBackgroundMenu(void)
 {
    menu_scan_recursive = 1;
-   MenuLoad(MenuFind("BACKGROUNDS_MENU", NULL));
+   MenuLoad(MenuFind("backgrounds", NULL));
    Mode.backgrounds.force_scan = 0;
    menu_scan_recursive = 0;
 }
 
+static char        *
+find_icon(const char *file)
+{
+   char                s[FILEPATH_LEN_MAX];
+
+   Esnprintf(s, sizeof(s), "%s/icons", EDirUser());
+
+   return FindFile(file, s);
+}
+
 static void
 FillFlatFileMenu(Menu * m, const char *file)
 {
@@ -309,14 +320,18 @@ FillFlatFileMenu(Menu * m, const char *file)
 	     Menu               *mm;
 
 	     txt = icon = act = params = NULL;
+	     wd[0] = '\0';
 	     parse(s, "%S%T%S%S", &txt, &icon, &act, &params);
 
-	     if (icon && exists(icon))
+	     if (icon)
+		icon = find_icon(icon);
+	     if (icon)
 	       {
 		  Esnprintf(wd, sizeof(wd), "__FM.%s", icon);
 		  icc = ImageclassFind(wd, 0);
 		  if (!icc)
 		     icc = ImageclassCreateSimple(wd, icon);
+		  Efree(icon);
 	       }
 	     if ((act) && (!strcmp(act, "exec")) && (params))
 	       {
@@ -407,6 +422,24 @@ MenuCreateFromFlatFile(const char *name, Menu * parent, MenuStyle * ms,
    return m;
 }
 
+static Menu        *
+MenuCreateFromBackgrounds(const char *name, MenuStyle * ms)
+{
+   Menu               *m;
+   char                s[FILEPATH_LEN_MAX];
+
+   Esnprintf(s, sizeof(s), "%s/backgrounds", EDirUser());
+
+   m = MenuCreateFromDirectory(name, NULL, ms, s);
+   if (!m)
+      return NULL;
+
+   MenuSetTitle(m, _("Backgrounds"));
+   MenuSetInternal(m);
+
+   return m;
+}
+
 static int
 MenuLoadFromThemes(Menu * m)
 {
@@ -687,6 +720,10 @@ MenusCreateInternal(const char *type, const char *name, const char *style,
 	SoundPlay(SOUND_SCANNING);
 	m = MenuCreateFromDirectory(name, NULL, ms, prm);
      }
+   else if (!strcmp(type, "backgrounds"))
+     {
+	m = MenuCreateFromBackgrounds(name, ms);
+     }
    else if (!strcmp(type, "borders"))
      {
 	m = MenuCreateFromBorders(name, ms);
