@@ -102,9 +102,9 @@ struct _menu {
    MenuItem          **items;
    int                 icon_size;
    char                internal;	/* Don't destroy when reloading */
-   char                dynamic;	/* May be emptied on close */
+   char                dynamic;	/* Empty when closed */
+   char                transient;	/* Destroy when closed */
    char                shown;
-   char                stuck;
    char                redraw;
    char                filled;	/* Has been filled */
    Menu               *parent;
@@ -154,7 +154,7 @@ MenuFindItemByChild(Menu * m, Menu * mc)
    return NULL;
 }
 
-void
+static void
 MenuHide(Menu * m)
 {
    EWin               *ewin;
@@ -175,7 +175,6 @@ MenuHide(Menu * m)
      }
    m->ewin = NULL;
 
-   m->stuck = 0;
    m->shown = 0;
    m->parent = NULL;
    m->last_access = time(0);
@@ -203,6 +202,7 @@ _MenuEwinInit(EWin * ewin)
    ewin->props.skip_focuslist = 1;
    ewin->props.skip_winlist = 1;
    ewin->props.ignorearrange = 1;
+   ewin->state.active = 1;
    EwinInhSetWM(ewin, focus, 1);
 
    ICCCM_SetSizeConstraints(ewin, m->w, m->h, m->w, m->h, 0, 0, 1, 1,
@@ -277,7 +277,7 @@ MenuShow(Menu * m, char noshow)
    if (m->shown)
       return;
 
-   if (MenuLoad(m))
+   if (MenuLoad(m) || m->redraw)
       MenuRealize(m);
 
    if (m->num <= 0)
@@ -343,7 +343,6 @@ MenuShow(Menu * m, char noshow)
 
 	ewin->head = head_num;
 
-	EwinMoveToDesktop(ewin, EoGetDesk(ewin));
 	EwinResize(ewin, ewin->client.w, ewin->client.h);
 
 	if (Conf.menus.animate)
@@ -359,7 +358,6 @@ MenuShow(Menu * m, char noshow)
 	  }
      }
 
-   m->stuck = 0;
    m->shown = 1;
    m->last_access = time(0);
    Mode_menus.just_shown = 1;
@@ -468,6 +466,12 @@ MenuSetDynamic(Menu * m)
    m->dynamic = 1;
 }
 
+void
+MenuSetTransient(Menu * m)
+{
+   m->transient = 1;
+}
+
 void
 MenuSetName(Menu * m, const char *name)
 {
@@ -568,7 +572,7 @@ MenuCreate(const char *name, const char *title, Menu * parent, MenuStyle * ms)
    return m;
 }
 
-void
+static void
 MenuDestroy(Menu * m)
 {
    if (!m)
@@ -610,8 +614,6 @@ MenuEmpty(Menu * m, int destroying)
    for (i = 0; i < m->num; i++)
      {
 	mi = m->items[i];
-	if (!mi)
-	   continue;
 
 	if (mi->child)
 	  {
@@ -634,6 +636,7 @@ MenuEmpty(Menu * m, int destroying)
 
    FreePmapMask(&m->pmm);
 
+   m->last_change = 0;
    m->filled = 0;
 }
 
@@ -646,8 +649,6 @@ MenuFreePixmaps(Menu * m)
    for (i = 0; i < m->num; i++)
      {
 	mi = m->items[i];
-	if (!mi)
-	   continue;
 
 	for (j = 0; j < 3; j++)
 	   FreePmapMask(mi->pmm + j);
@@ -655,6 +656,7 @@ MenuFreePixmaps(Menu * m)
 
    FreePmapMask(&m->pmm);
 
+   m->last_change = 0;
    m->filled = 0;
 }
 
@@ -718,7 +720,7 @@ MenuRealize(Menu * m)
    if (!m->win)
      {
 	m->win = ECreateClientWindow(VROOT, 0, 0, 1, 1);
-	EventCallbackRegister(m->win, 0, MenuHandleEvents, m);
+	EventCallbackRegister(m->win, MenuHandleEvents, m);
      }
 
    maxh = maxw = 0;
@@ -737,7 +739,7 @@ MenuRealize(Menu * m)
 	   has_i = 1;
 
 	m->items[i]->win = ECreateWindow(m->win, 0, 0, 1, 1, 0);
-	EventCallbackRegister(m->items[i]->win, 0, MenuItemHandleEvents,
+	EventCallbackRegister(m->items[i]->win, MenuItemHandleEvents,
 			      m->items[i]);
 	ESelectInput(m->items[i]->win, MENU_ITEM_EVENT_MASK);
 	EMapWindow(m->items[i]->win);
@@ -1035,7 +1037,7 @@ MenuShowMasker(Menu * m __UNUSED__)
 	EobjSetLayer(eo, 11);
 	ESelectInput(EobjGetWin(eo), ButtonPressMask | ButtonReleaseMask |
 		     EnterWindowMask | LeaveWindowMask);
-	EventCallbackRegister(EobjGetWin(eo), 0, MenuMaskerHandleEvents, NULL);
+	EventCallbackRegister(EobjGetWin(eo), MenuMaskerHandleEvents, NULL);
 
 	Mode_menus.cover_win = eo;
      }
@@ -1051,7 +1053,7 @@ MenuHideMasker(void)
    if (!eo)
       return;
 
-   EventCallbackUnregister(EobjGetWin(eo), 0, MenuMaskerHandleEvents, NULL);
+   EventCallbackUnregister(EobjGetWin(eo), MenuMaskerHandleEvents, NULL);
    EobjWindowDestroy(eo);
    Mode_menus.cover_win = NULL;
 }
@@ -1170,9 +1172,15 @@ MenusActive(void)
 static void
 MenusHide(void)
 {
+   Menu               *m;
+
    TIMER_DEL(menu_timer_submenu);
 
-   MenuHide(Mode_menus.first);
+   m = Mode_menus.first;
+   if (m && m->transient)
+      MenuDestroy(m);
+   else
+      MenuHide(m);
    Mode_menus.first = NULL;
    MenuHideMasker();
    TooltipsEnable(1);
@@ -1937,7 +1945,9 @@ MenuConfigLoad(FILE * fs)
 	     ic = NULL;
 	     if (strcmp("NULL", s2))
 		ic = ImageclassFind(s2, 0);
-	     _EFDUP(txt, p3);
+	     _EFREE(txt);
+	     if (p3 && *p3)
+		txt = Estrdup(p3);
 	     continue;
 	  }
 
@@ -2014,7 +2024,6 @@ MenusTimeout(void *data __UNUSED__)
 	  ts - m->last_access < MENU_UNLOAD_CHECK_INTERVAL)
 	 continue;
 
-      m->last_change = 0;
       if (!m->loader)
 	 MenuFreePixmaps(m);
       else if (m->ref_count)
@@ -2037,10 +2046,6 @@ MenusSighan(int sig, void *prm __UNUSED__)
 
    switch (sig)
      {
-     case ESIGNAL_CONFIGURE:
-	ConfigFileLoad("menus.cfg", NULL, MenuConfigLoad, 1);
-	break;
-
      case ESIGNAL_START:
 	TIMER_ADD(menu_unload_timer, 1. * MENU_UNLOAD_CHECK_INTERVAL,
 		  MenusTimeout, NULL);
@@ -2067,47 +2072,84 @@ MenusSighan(int sig, void *prm __UNUSED__)
  * Configuration dialog
  */
 
-static char         tmp_warpmenus;
-static char         tmp_animated_menus;
-static char         tmp_menusonscreen;
+typedef struct {
+   char                warp;
+   char                animate;
+   char                onscreen;
+   int                 icon_size;
+} MenudDlgData;
 
 static void
-CB_ConfigureMenus(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
+CB_ConfigureMenus(Dialog * d, int val, void *data __UNUSED__)
 {
-   if (val < 2)
-     {
-	Conf.menus.warp = tmp_warpmenus;
-	Conf.menus.animate = tmp_animated_menus;
-	Conf.menus.onscreen = tmp_menusonscreen;
-     }
+   MenudDlgData       *dd = DLG_DATA_GET(d, MenudDlgData);
+
+   if (val >= 2)
+      return;
+
+   Conf.menus.warp = dd->warp;
+   Conf.menus.animate = dd->animate;
+   Conf.menus.onscreen = dd->onscreen;
+   Conf.menus.icon_size = dd->icon_size;
+
+   MenusTouch();
+
    autosave();
 }
 
 static void
-_DlgFillMenus(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
+_DlgCbIconSize(Dialog * d, int val __UNUSED__, void *data)
 {
-   DItem              *di;
+   MenudDlgData       *dd = DLG_DATA_GET(d, MenudDlgData);
+   DItem              *di = (DItem *) data;
+   char                s[256];
 
-   tmp_warpmenus = Conf.menus.warp;
-   tmp_animated_menus = Conf.menus.animate;
-   tmp_menusonscreen = Conf.menus.onscreen;
+   Esnprintf(s, sizeof(s), _("Icon size: %2d"), dd->icon_size);
+   DialogItemSetText(di, s);
+}
 
-   DialogItemTableSetOptions(table, 3, 0, 0, 0);
+static void
+_DlgFillMenus(Dialog * d, DItem * table, void *data __UNUSED__)
+{
+   DItem              *di, *label;
+   MenudDlgData       *dd;
+
+   dd = DLG_DATA_SET(d, MenudDlgData);
+   if (!dd)
+      return;
+
+   dd->warp = Conf.menus.warp;
+   dd->animate = Conf.menus.animate;
+   dd->onscreen = Conf.menus.onscreen;
+   dd->icon_size = Conf.menus.icon_size;
+
+   DialogItemTableSetOptions(table, 2, 0, 0, 0);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
-   DialogItemSetColSpan(di, 3);
+   DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Animated display of menus"));
-   DialogItemCheckButtonSetPtr(di, &tmp_animated_menus);
+   DialogItemCheckButtonSetPtr(di, &dd->animate);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
-   DialogItemSetColSpan(di, 3);
+   DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Always pop up menus on screen"));
-   DialogItemCheckButtonSetPtr(di, &tmp_menusonscreen);
+   DialogItemCheckButtonSetPtr(di, &dd->onscreen);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
-   DialogItemSetColSpan(di, 3);
+   DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Warp pointer after moving menus"));
-   DialogItemCheckButtonSetPtr(di, &tmp_warpmenus);
+   DialogItemCheckButtonSetPtr(di, &dd->warp);
+
+   di = label = DialogAddItem(table, DITEM_TEXT);
+   DialogItemSetAlign(di, 0, 512);
+
+   di = DialogAddItem(table, DITEM_SLIDER);
+   DialogItemSliderSetBounds(di, 12, 48);
+   DialogItemSliderSetUnits(di, 2);
+   DialogItemSliderSetJump(di, 4);
+   DialogItemSliderSetValPtr(di, &dd->icon_size);
+   DialogItemSetCallback(di, _DlgCbIconSize, 0, label);
+   DialogItemCallCallback(d, di);
 }
 
 const DialogDef     DlgMenus = {
@@ -2154,7 +2196,6 @@ MenusIpc(const char *params)
    else if (!strncmp(cmd, "reload", 2))
      {
 	MenusDestroyLoaded();
-	ConfigFileLoad("menus.cfg", NULL, MenuConfigLoad, 1);
      }
    else if (!strncmp(cmd, "show", 2))
      {
@@ -2176,7 +2217,7 @@ static const IpcItem MenusIpcArray[] = {
     "menus", "mnu",
     "Menu functions",
     "  menus list               Show existing menus\n"
-    "  menus reload             Reload menus.cfg without restarting\n"
+    "  menus reload             Force menu refresh\n"
     "  menus show <name>        Show named menu\n"}
    ,
 };
