@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -22,6 +22,7 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+#include "borders.h"
 #include "cursors.h"
 #include "desktops.h"
 #include "emodule.h"
@@ -36,6 +37,7 @@
 #include <X11/keysym.h>
 
 static struct {
+   Win                 events;
    EWin               *ewin;
    char                mode;
    char                using_kbd;
@@ -48,6 +50,8 @@ static struct {
    int                 resize_detail;
 } Mode_mr;
 
+static void         _MoveResizeInit(void);
+
 static int
 _NeedServerGrab(int mode)
 {
@@ -79,14 +83,16 @@ EwinShapeSet(EWin * ewin)
      }
 }
 
-int
-ActionMoveStart(EWin * ewin, int kbd, int constrained, int nogroup)
+void
+MoveResizeMoveStart(EWin * ewin, int kbd, int constrained, int nogroup)
 {
    EWin              **gwins;
    int                 i, num, cx, cy;
 
    if (!ewin || ewin->state.inhibit_move)
-      return 0;
+      return;
+
+   _MoveResizeInit();
 
    Mode_mr.ewin = ewin;
    Mode_mr.using_kbd = kbd;
@@ -96,11 +102,6 @@ ActionMoveStart(EWin * ewin, int kbd, int constrained, int nogroup)
 
    SoundPlay(SOUND_MOVE_START);
 
-   if (kbd)
-      GrabKeyboardSet(EoGetWin(ewin));
-   else
-      GrabPointerSet(EoGetWin(ewin), ECSR_ACT_MOVE, 1);
-
    Mode.mode = MODE_MOVE_PENDING;
    Mode.constrained = constrained;
 
@@ -133,21 +134,24 @@ ActionMoveStart(EWin * ewin, int kbd, int constrained, int nogroup)
      }
    Efree(gwins);
 
+   if (kbd)
+      GrabKeyboardSet(Mode_mr.events);
+   else
+      GrabPointerSet(Mode_mr.events, ECSR_ACT_MOVE, 1);
+
    Mode_mr.swapcoord_x = EoGetX(ewin);
    Mode_mr.swapcoord_y = EoGetY(ewin);
-
-   return 0;
 }
 
-static int
-ActionMoveEnd(EWin * ewin)
+static void
+_MoveResizeMoveEnd(EWin * ewin)
 {
    EWin              **gwins;
    int                 num, i;
    Desk               *d1, *d2;
 
    if (ewin && ewin != Mode_mr.ewin)
-      return 0;
+      return;
 
    GrabKeyboardRelease();
    GrabPointerRelease();
@@ -206,22 +210,20 @@ ActionMoveEnd(EWin * ewin)
 	EUngrabServer();
 	ModulesSignal(ESIGNAL_ANIMATION_RESUME, NULL);
      }
-
-   return 0;
 }
 
-static int
-ActionMoveSuspend(void)
+static void
+_MoveResizeMoveSuspend(void)
 {
    EWin               *ewin, **lst;
    int                 i, num;
 
    ewin = Mode_mr.ewin;
    if (!ewin)
-      return 0;
+      return;
 
    if (Mode.mode == MODE_MOVE_PENDING)
-      return 0;
+      return;
 
    /* If non opaque undraw our boxes */
    if (Mode_mr.grab_server)
@@ -238,12 +240,10 @@ ActionMoveSuspend(void)
 
 	EUngrabServer();
      }
-
-   return 0;
 }
 
-static int
-ActionMoveResume(void)
+static void
+_MoveResizeMoveResume(void)
 {
    EWin               *ewin, **lst;
    int                 i, num;
@@ -251,9 +251,9 @@ ActionMoveResume(void)
 
    ewin = Mode_mr.ewin;
    if (!ewin)
-      return 0;
+      return;
 
-   GrabPointerSet(EoGetWin(ewin), ECSR_ACT_MOVE, 1);
+   GrabPointerSet(Mode_mr.events, ECSR_ACT_MOVE, 1);
 
    fl = (Mode_mr.mode == 5) ? 4 : 0;
    if (Mode.mode == MODE_MOVE_PENDING)
@@ -286,22 +286,22 @@ ActionMoveResume(void)
 		      ewin->client.w, ewin->client.h, fl, i);
      }
    Efree(lst);
-
-   return 0;
 }
 
 #define RD(h, v) (((h) << 8) + (v))
 #define RD_H(hv) (((hv) >> 8) & 0xff)
 #define RD_V(hv) (((hv)     ) & 0xff)
 
-int
-ActionResizeStart(EWin * ewin, int kbd, int hv)
+void
+MoveResizeResizeStart(EWin * ewin, int kbd, int hv)
 {
    int                 x, y, w, h, ww, hh, cx, cy;
    unsigned int        csr;
 
    if (!ewin || ewin->state.inhibit_resize)
-      return 0;
+      return;
+
+   _MoveResizeInit();
 
    Mode_mr.ewin = ewin;
 
@@ -427,23 +427,21 @@ ActionResizeStart(EWin * ewin, int kbd, int hv)
    Mode_mr.win_h = ewin->client.h;
 
    if (kbd)
-      GrabKeyboardSet(EoGetWin(ewin));
+      GrabKeyboardSet(Mode_mr.events);
    else
-      GrabPointerSet(EoGetWin(ewin), csr, 1);
+      GrabPointerSet(Mode_mr.events, csr, 1);
 
    EwinShapeSet(ewin);
    ewin->state.show_coords = 1;
    DrawEwinShape(ewin, Conf.movres.mode_resize, EoGetX(ewin), EoGetY(ewin),
 		 ewin->client.w, ewin->client.h, 0, 0);
-
-   return 0;
 }
 
-static int
-ActionResizeEnd(EWin * ewin)
+static void
+_MoveResizeResizeEnd(EWin * ewin)
 {
    if (ewin && ewin != Mode_mr.ewin)
-      return 0;
+      return;
 
    Mode.mode = MODE_NONE;
 
@@ -482,12 +480,10 @@ ActionResizeEnd(EWin * ewin)
 	EUngrabServer();
 	ModulesSignal(ESIGNAL_ANIMATION_RESUME, NULL);
      }
-
-   return 0;
 }
 
 static void
-ActionMoveHandleMotion(void)
+_MoveResizeMoveHandleMotion(void)
 {
    int                 dx, dy, dd;
    EWin               *ewin, **gwins, *ewin1;
@@ -719,7 +715,7 @@ ActionMoveHandleMotion(void)
 }
 
 static void
-ActionResizeHandleMotion(void)
+_MoveResizeResizeHandleMotion(void)
 {
    int                 x, y, w, h;
    EWin               *ewin;
@@ -766,8 +762,8 @@ ActionResizeHandleMotion(void)
    DrawEwinShape(ewin, Conf.movres.mode_resize, x, y, w, h, 1, 0);
 }
 
-void
-ActionsHandleKey(unsigned int key)
+static void
+_MoveResizeHandleKey(unsigned int key)
 {
    EWin               *ewin;
    int                 resize, delta, end = 0;
@@ -787,6 +783,7 @@ ActionsHandleKey(unsigned int key)
      {
      default:
 	return;
+
      case XK_Escape:
 	Mode_mr.cur_x = Mode_mr.start_x;
 	Mode_mr.cur_y = Mode_mr.start_y;
@@ -832,17 +829,38 @@ ActionsHandleKey(unsigned int key)
      {
      case MODE_MOVE_PENDING:
      case MODE_MOVE:
-	ActionMoveHandleMotion();
+	_MoveResizeMoveHandleMotion();
 	if (end)
-	   ActionMoveEnd(NULL);
+	   _MoveResizeMoveEnd(NULL);
 	break;
 
      case MODE_RESIZE:
      case MODE_RESIZE_H:
      case MODE_RESIZE_V:
-	ActionResizeHandleMotion();
+	_MoveResizeResizeHandleMotion();
 	if (end)
-	   ActionResizeEnd(NULL);
+	   _MoveResizeResizeEnd(NULL);
+	break;
+
+     default:
+	break;
+     }
+}
+
+static void
+_MoveResizeHandleMotion(void)
+{
+   switch (Mode.mode)
+     {
+     case MODE_MOVE_PENDING:
+     case MODE_MOVE:
+	_MoveResizeMoveHandleMotion();
+	break;
+
+     case MODE_RESIZE:
+     case MODE_RESIZE_H:
+     case MODE_RESIZE_V:
+	_MoveResizeResizeHandleMotion();
 	break;
 
      default:
@@ -851,85 +869,83 @@ ActionsHandleKey(unsigned int key)
 }
 
 void
-ActionsHandleMotion(void)
+MoveResizeSuspend(void)
 {
    switch (Mode.mode)
      {
      case MODE_MOVE_PENDING:
      case MODE_MOVE:
-	ActionMoveHandleMotion();
+	_MoveResizeMoveSuspend();
 	break;
+     }
+}
 
+void
+MoveResizeResume(void)
+{
+   switch (Mode.mode)
+     {
+     case MODE_MOVE_PENDING:
+     case MODE_MOVE:
+	_MoveResizeMoveResume();
+	break;
+     }
+}
+
+void
+MoveResizeEnd(EWin * ewin)
+{
+   switch (Mode.mode)
+     {
      case MODE_RESIZE:
      case MODE_RESIZE_H:
      case MODE_RESIZE_V:
-	ActionResizeHandleMotion();
+	_MoveResizeResizeEnd(ewin);
 	break;
 
+     case MODE_MOVE_PENDING:
+     case MODE_MOVE:
+	_MoveResizeMoveEnd(ewin);
+	break;
+     }
+}
+
+static void
+_MoveResizeEventHandler(Win win __UNUSED__, XEvent * ev, void *prm __UNUSED__)
+{
+#if 0
+   Eprintf("%s: type=%2d win=%#lx\n", __func__, ev->type, ev->xany.window);
+#endif
+   switch (ev->type)
+     {
      default:
 	break;
+     case KeyPress:
+	_MoveResizeHandleKey(XLookupKeysym(&ev->xkey, 0));
+	break;
+#if 0
+     case ButtonPress:
+	break;
+#endif
+     case ButtonRelease:
+	MoveResizeEnd(NULL);
+	if (Mode_mr.ewin)
+	   BorderCheckState(Mode_mr.ewin, ev);
+	break;
+     case MotionNotify:
+	_MoveResizeHandleMotion();
+	break;
      }
 }
 
-int
-ActionsSuspend(void)
+static void
+_MoveResizeInit(void)
 {
-   switch (Mode.mode)
-     {
-     case MODE_MOVE_PENDING:
-     case MODE_MOVE:
-	ActionMoveSuspend();
-	break;
-     case MODE_RESIZE:
-     case MODE_RESIZE_H:
-     case MODE_RESIZE_V:
-	ActionResizeEnd(NULL);
-	break;
-     }
-
-   return 0;
-}
-
-int
-ActionsResume(void)
-{
-   switch (Mode.mode)
-     {
-     case MODE_MOVE_PENDING:
-     case MODE_MOVE:
-	ActionMoveResume();
-	break;
-     }
-
-   return 0;
-}
-
-int
-ActionsEnd(EWin * ewin)
-{
-   int                 did_end = 1;
-
-   switch (Mode.mode)
-     {
-     case MODE_RESIZE:
-     case MODE_RESIZE_H:
-     case MODE_RESIZE_V:
-	ActionResizeEnd(ewin);
-	Mode.action_inhibit = 1;
-	break;
-
-     case MODE_MOVE_PENDING:
-     case MODE_MOVE:
-	ActionMoveEnd(ewin);
-	Mode.action_inhibit = 1;
-	break;
-
-     default:
-	did_end = 0;
-	break;
-     }
-
-   return did_end;
+   if (Mode_mr.events)
+      return;
+   Mode_mr.events = ECreateEventWindow(VROOT, 0, 0, 1, 1);
+   EMapWindow(Mode_mr.events);
+   EventCallbackRegister(Mode_mr.events, _MoveResizeEventHandler, NULL);
 }
 
 void
