@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -145,7 +145,7 @@ PagerCreate(void)
 
    p->name = NULL;
    p->win = ECreateClientWindow(VROOT, 0, 0, 1, 1);
-   EventCallbackRegister(p->win, 0, PagerEvent, p);
+   EventCallbackRegister(p->win, PagerEvent, p);
    p->sel_win = ECreateWindow(p->win, 0, 0, 1, 1, 0);
 
    return p;
@@ -473,8 +473,7 @@ doPagerUpdate(Pager * p)
    PagerUpdateEwinsFromPager(p);
 
  done:
-   if (gc != None)
-      EXFreeGC(gc);
+   EXFreeGC(gc);
 }
 
 static void
@@ -692,7 +691,6 @@ static void
 PagerShow(Pager * p)
 {
    EWin               *ewin;
-   int                 w, h;
 
    if (!Conf_pagers.enable)
       return;
@@ -719,25 +717,23 @@ PagerShow(Pager * p)
       EnterWindowMask | LeaveWindowMask | PointerMotionMask;
    ESelectInput(p->win, ewin->client.event_mask);
 
-   w = ewin->client.w;
-   h = ewin->client.h;
-
-   EwinMoveToDesktop(ewin, EoGetDesk(ewin));
    if (ewin->state.placed)
      {
-	EwinMoveResize(ewin, EoGetX(ewin), EoGetY(ewin), w, h);
+	EwinMoveResize(ewin, EoGetX(ewin), EoGetY(ewin),
+		       ewin->client.w, ewin->client.h);
      }
    else
      {
 	/* no snapshots ? first time ? make a row on the bottom left up */
-	int                 ax, ay;
+	int                 ax, ay, x, y, w, h;
 
 	DesksGetAreaSize(&ax, &ay);
 	w = ((48 * WinGetW(VROOT)) / WinGetH(VROOT)) * ax;
 	h = 48 * ay;
 	EwinResize(ewin, w, h);	/* Does layout */
-	EwinMove(ewin, 0, WinGetH(VROOT) -
-		 (DesksGetNumber() - p->dsk->num) * EoGetH(ewin));
+	x = 0;
+	y = WinGetH(VROOT) - (DesksGetNumber() - p->dsk->num) * EoGetH(ewin);
+	EwinMove(ewin, x, y);
      }
 
    EwinShow(ewin);
@@ -988,7 +984,7 @@ EwinInPagerAt(Pager * p, int px, int py)
 static void
 PagerMenuShow(Pager * p, int x, int y)
 {
-   static Menu        *p_menu = NULL, *pw_menu = NULL;
+   Menu               *m;
    MenuItem           *mi;
    EWin               *ewin;
    char                s[1024];
@@ -999,62 +995,64 @@ PagerMenuShow(Pager * p, int x, int y)
    ewin = EwinInPagerAt(p, x, y);
    if (ewin)
      {
-	if (pw_menu)
-	   MenuDestroy(pw_menu);
+	m = MenuCreate("__pg_win", _("Window Options"), NULL, NULL);
+	if (!m)
+	   return;
 
-	pw_menu =
-	   MenuCreate("__DESK_WIN_MENU", _("Window Options"), NULL, NULL);
+	MenuSetTransient(m);	/* Destroy when hidden */
 
 	Esnprintf(s, sizeof(s), "wop %#lx ic", EwinGetClientXwin(ewin));
 	mi = MenuItemCreate(_("Iconify"), NULL, s, NULL);
-	MenuAddItem(pw_menu, mi);
+	MenuAddItem(m, mi);
 
 	Esnprintf(s, sizeof(s), "wop %#lx close", EwinGetClientXwin(ewin));
 	mi = MenuItemCreate(_("Close"), NULL, s, NULL);
-	MenuAddItem(pw_menu, mi);
+	MenuAddItem(m, mi);
 
 	Esnprintf(s, sizeof(s), "wop %#lx kill", EwinGetClientXwin(ewin));
 	mi = MenuItemCreate(_("Annihilate"), NULL, s, NULL);
-	MenuAddItem(pw_menu, mi);
+	MenuAddItem(m, mi);
 
 	Esnprintf(s, sizeof(s), "wop %#lx st", EwinGetClientXwin(ewin));
 	mi = MenuItemCreate(_("Stick / Unstick"), NULL, s, NULL);
-	MenuAddItem(pw_menu, mi);
+	MenuAddItem(m, mi);
 
-	EFunc(NULL, "menus show __DESK_WIN_MENU");
+	EFunc(NULL, "menus show __pg_win");
 	return;
      }
 
-   if (p_menu)
-      MenuDestroy(p_menu);
-   p_menu = MenuCreate("__DESK_MENU", _("Desktop Options"), NULL, NULL);
+   m = MenuCreate("__pg", _("Desktop Options"), NULL, NULL);
+   if (!m)
+      return;
+
+   MenuSetTransient(m);		/* Destroy when hidden */
 
    mi = MenuItemCreate(_("Pager Settings..."), NULL, "cfg pagers", NULL);
-   MenuAddItem(p_menu, mi);
+   MenuAddItem(m, mi);
 
    if (PagersGetMode() != PAGER_MODE_SIMPLE)
      {
 	mi = MenuItemCreate(_("Snapshotting Off"), NULL, "pg mode simp", NULL);
-	MenuAddItem(p_menu, mi);
+	MenuAddItem(m, mi);
 
 	if (Conf_pagers.hiq)
 	   mi = MenuItemCreate(_("High Quality Off"), NULL, "pg hiq off", NULL);
 	else
 	   mi = MenuItemCreate(_("High Quality On"), NULL, "pg hiq on", NULL);
-	MenuAddItem(p_menu, mi);
+	MenuAddItem(m, mi);
      }
    else
      {
 	mi = MenuItemCreate(_("Snapshotting On"), NULL, "pg mode live", NULL);
-	MenuAddItem(p_menu, mi);
+	MenuAddItem(m, mi);
      }
    if (Conf_pagers.zoom)
       mi = MenuItemCreate(_("Zoom Off"), NULL, "pg zoom off", NULL);
    else
       mi = MenuItemCreate(_("Zoom On"), NULL, "pg zoom on", NULL);
-   MenuAddItem(p_menu, mi);
+   MenuAddItem(m, mi);
 
-   EFunc(NULL, "menus show __DESK_MENU");
+   EFunc(NULL, "menus show __pg");
 }
 
 static void
@@ -1416,6 +1414,9 @@ PagerHandleMouseDown(Pager * p, int px, int py, int button)
    int                 in_pager;
    EWin               *ewin;
 
+   Dprintf("%s m=%d d=%d x,y=%d,%d\n", __func__, Mode.mode, p->dsk->num,
+	   px, py);
+
    in_pager = (px >= 0 && py >= 0 && px < p->w && py < p->h);
    if (!in_pager)
       return;
@@ -1432,121 +1433,103 @@ PagerHandleMouseDown(Pager * p, int px, int py, int button)
 	  {
 	     PagerHiwinShow(p, ewin, 1, !Mode.wm.window);
 	     Mode.mode = MODE_PAGER_DRAG_PENDING;
+	     EwinRaise(ewin);
 	     PagerEwinGroupSet();
 	  }
      }
 }
 
-static void
-PagerHiwinHandleMouseDown(Pager * p, int px, int py, int button)
-{
-   PagerHandleMouseDown(p, px, py, button);
-}
-
 static void
 PagerHandleMouseUp(Pager * p, int px, int py, int button)
 {
-   int                 in_pager;
-   EWin               *ewin;
-
-   in_pager = (px >= 0 && py >= 0 && px < p->w && py < p->h);
-   if (!in_pager)
-      return;
-
-   if (button == Conf_pagers.sel_button)
-     {
-	DeskGoto(p->dsk);
-	if (p->dsk != DesksGetCurrent())
-	   SoundPlay(SOUND_DESKTOP_SHUT);
-	DeskCurrentGotoArea(px / p->dw, py / p->dh);
-     }
-   else if (button == Conf_pagers.win_button)
-     {
-	DeskGoto(p->dsk);
-	DeskCurrentGotoArea(px / p->dw, py / p->dh);
-	ewin = EwinInPagerAt(p, px, py);
-	if (ewin)
-	  {
-	     EwinRaise(ewin);
-	     FocusToEWin(ewin, FOCUS_SET);
-	  }
-     }
-}
-
-static void
-PagerHiwinHandleMouseUp(Pager * p, int px, int py, int button)
-{
-   int                 i, num, in_vroot;
+   int                 i, num, in_pager, in_vroot;
    EWin               *ewin, *ewin2, **gwins;
    int                 x, y;
 
-   Dprintf("PagerHiwinHandleMouseUp m=%d d=%d x,y=%d,%d\n", Mode.mode,
-	   p->dsk->num, px, py);
+   Dprintf("%s m=%d d=%d x,y=%d,%d\n", __func__, Mode.mode, p->dsk->num,
+	   px, py);
+
+   in_pager = (px >= 0 && py >= 0 && px < p->w && py < p->h);
+
+   if (button == Conf_pagers.sel_button)
+     {
+	if (!in_pager)
+	   return;
+	DeskGoto(p->dsk);
+	DeskCurrentGotoArea(px / p->dw, py / p->dh);
+	return;
+     }
+
+   if (button != Conf_pagers.win_button)
+      return;
+
+   ewin = HiwinGetEwin(hiwin, 1);
+   PagerHiwinHide();
 
    if (Mode.mode != MODE_PAGER_DRAG)
      {
 	if (Mode.mode == MODE_PAGER_DRAG_PENDING)
 	   Mode.mode = MODE_NONE;
-	PagerHiwinHide();
-	PagerHandleMouseUp(p, px, py, button);
-	return;
+	if (!in_pager)
+	   goto done;
+	DeskGoto(p->dsk);
+	DeskCurrentGotoArea(px / p->dw, py / p->dh);
+	if (ewin)
+	   FocusToEWin(ewin, FOCUS_SET);
+	goto done;
      }
 
    Mode.mode = MODE_NONE;
 
-   ewin = HiwinGetEwin(hiwin, 1);
    if (!ewin)
       goto done;
 
+   /* Finish pager drag */
+
    in_vroot = (Mode.events.cx >= 0 && Mode.events.cx < WinGetW(VROOT) &&
 	       Mode.events.cy >= 0 && Mode.events.cy < WinGetH(VROOT));
 
-   if (button == Conf_pagers.win_button)
+   /* Find which pager or iconbox we are in (if any) */
+   ewin2 = GetEwinPointerInClient();
+   if ((ewin2) && (ewin2->type == EWIN_TYPE_PAGER))
      {
-	/* Find which pager or iconbox we are in (if any) */
-	ewin2 = GetEwinPointerInClient();
-	if ((ewin2) && (ewin2->type == EWIN_TYPE_PAGER))
-	  {
-	     PagerEwinMove(p, (Pager *) ewin2->data, ewin);
-	  }
-	else if ((ewin2) && (ewin2->type == EWIN_TYPE_ICONBOX))
-	  {
-	     /* Pointer is in iconbox */
-
-	     /* Iconify after moving back to pre-drag position */
-	     gwins = ListWinGroupMembersForEwin(ewin, GROUP_ACTION_MOVE,
-						Mode.nogroup, &num);
-	     for (i = 0; i < num; i++)
-	       {
-		  if (gwins[i]->type != EWIN_TYPE_PAGER)
-		    {
-		       EwinMove(gwins[i], gwin_px[i], gwin_py[i]);
-		       EwinIconify(gwins[i]);
-		    }
-	       }
-	     Efree(gwins);
-	  }
-	else if (ewin2 && ewin2->props.vroot)
-	  {
-	     /* Dropping onto virtual root */
-	     EwinReparent(ewin, EwinGetClientWin(ewin2));
-	  }
-	else if (!in_vroot)
-	  {
-	     /* Move back to real root */
-	     EwinReparent(ewin, RROOT);
-	  }
-	else
-	  {
-	     /* Pointer is not in pager or iconbox */
-	     /* Move window(s) to pointer location */
-	     x = Mode.events.cx - EoGetW(ewin) / 2;
-	     y = Mode.events.cy - EoGetH(ewin) / 2;
-	     EwinGroupMove(ewin, DesksGetCurrent(), x, y);
-	  }
+	PagerEwinMove(p, (Pager *) ewin2->data, ewin);
      }
+   else if ((ewin2) && (ewin2->type == EWIN_TYPE_ICONBOX))
+     {
+	/* Pointer is in iconbox */
 
-   PagerHiwinHide();
+	/* Iconify after moving back to pre-drag position */
+	gwins = ListWinGroupMembersForEwin(ewin, GROUP_ACTION_MOVE,
+					   Mode.nogroup, &num);
+	for (i = 0; i < num; i++)
+	  {
+	     if (gwins[i]->type != EWIN_TYPE_PAGER)
+	       {
+		  EwinMove(gwins[i], gwin_px[i], gwin_py[i]);
+		  EwinIconify(gwins[i]);
+	       }
+	  }
+	Efree(gwins);
+     }
+   else if (ewin2 && ewin2->props.vroot)
+     {
+	/* Dropping onto virtual root */
+	EwinReparent(ewin, EwinGetClientWin(ewin2));
+     }
+   else if (!in_vroot)
+     {
+	/* Move back to real root */
+	EwinReparent(ewin, RROOT);
+     }
+   else
+     {
+	/* Pointer is not in pager or iconbox */
+	/* Move window(s) to pointer location */
+	x = Mode.events.cx - EoGetW(ewin) / 2;
+	y = Mode.events.cy - EoGetH(ewin) / 2;
+	EwinGroupMove(ewin, DesksGetCurrent(), x, y);
+     }
 
  done:
    /* unallocate the space that was holding the old positions of the */
@@ -1596,7 +1579,7 @@ PagerEvent(Win win __UNUSED__, XEvent * ev, void *prm)
 }
 
 static void
-PagerHiwinEvent(Win win, XEvent * ev, void *prm)
+PagerHiwinEvent(Win win __UNUSED__, XEvent * ev, void *prm)
 {
    Pager              *p = (Pager *) prm;
    int                 px, py;
@@ -1624,10 +1607,10 @@ PagerHiwinEvent(Win win, XEvent * ev, void *prm)
 	     break;
 	  default:
 	     /* Translate x,y to pager window coordinates */
-	     ETranslateCoordinates(win, p->win,
-				   ev->xbutton.x, ev->xbutton.y, &px, &py,
-				   NULL);
-	     PagerHiwinHandleMouseDown(p, px, py, (int)ev->xbutton.button);
+	     ETranslateCoordinates(RROOT, p->win,
+				   ev->xbutton.x_root, ev->xbutton.y_root,
+				   &px, &py, NULL);
+	     PagerHandleMouseDown(p, px, py, (int)ev->xbutton.button);
 	     break;
 	  }
 	break;
@@ -1640,10 +1623,10 @@ PagerHiwinEvent(Win win, XEvent * ev, void *prm)
 	     break;
 	  default:
 	     /* Translate x,y to pager window coordinates */
-	     ETranslateCoordinates(win, p->win,
-				   ev->xbutton.x, ev->xbutton.y, &px, &py,
-				   NULL);
-	     PagerHiwinHandleMouseUp(p, px, py, (int)ev->xbutton.button);
+	     ETranslateCoordinates(RROOT, p->win,
+				   ev->xbutton.x_root, ev->xbutton.y_root,
+				   &px, &py, NULL);
+	     PagerHandleMouseUp(p, px, py, (int)ev->xbutton.button);
 	     break;
 	  }
 	break;
@@ -1766,82 +1749,92 @@ PagersReconfigure(void)
 /*
  * Configuration dialog
  */
-static char         tmp_show_pagers;
-static char         tmp_pager_hiq;
-static int          tmp_pager_mode;
-static char         tmp_pager_zoom;
-static char         tmp_pager_title;
-static char         tmp_pager_do_scan;
-static int          tmp_pager_scan_speed;
-static int          tmp_pager_sel_button;
-static int          tmp_pager_win_button;
-static int          tmp_pager_menu_button;
-static DItem       *pager_scan_speed_label = NULL;
+typedef struct {
+   char                show_pagers;
+   char                pager_hiq;
+   int                 pager_mode;
+   char                pager_zoom;
+   char                pager_title;
+   char                pager_do_scan;
+   int                 pager_scan_speed;
+   int                 pager_sel_button;
+   int                 pager_win_button;
+   int                 pager_menu_button;
+} PagerDlgData;
 
 static void
-CB_ConfigurePager(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
+CB_ConfigurePager(Dialog * d, int val, void *data __UNUSED__)
 {
-   if (val < 2)
+   PagerDlgData       *dd = DLG_DATA_GET(d, PagerDlgData);
+
+   if (val >= 2)
+      return;
+
+   PagersShow(dd->show_pagers);
+   if (Conf_pagers.hiq != dd->pager_hiq)
+      PagerSetHiQ(dd->pager_hiq);
+   Conf_pagers.zoom = dd->pager_zoom;
+   Conf_pagers.title = dd->pager_title;
+   Conf_pagers.sel_button = dd->pager_sel_button;
+   Conf_pagers.win_button = dd->pager_win_button;
+   Conf_pagers.menu_button = dd->pager_menu_button;
+   if ((Conf_pagers.scanspeed != dd->pager_scan_speed)
+       || ((!dd->pager_do_scan) && (Conf_pagers.scanspeed > 0))
+       || ((dd->pager_do_scan) && (Conf_pagers.scanspeed == 0)))
      {
-	PagersShow(tmp_show_pagers);
-	if (Conf_pagers.hiq != tmp_pager_hiq)
-	   PagerSetHiQ(tmp_pager_hiq);
-	Conf_pagers.zoom = tmp_pager_zoom;
-	Conf_pagers.title = tmp_pager_title;
-	Conf_pagers.sel_button = tmp_pager_sel_button;
-	Conf_pagers.win_button = tmp_pager_win_button;
-	Conf_pagers.menu_button = tmp_pager_menu_button;
-	if ((Conf_pagers.scanspeed != tmp_pager_scan_speed)
-	    || ((!tmp_pager_do_scan) && (Conf_pagers.scanspeed > 0))
-	    || ((tmp_pager_do_scan) && (Conf_pagers.scanspeed == 0)))
-	  {
-	     if (tmp_pager_do_scan)
-		Conf_pagers.scanspeed = tmp_pager_scan_speed;
-	     else
-		Conf_pagers.scanspeed = 0;
-	  }
-	PagersSetMode(tmp_pager_mode);
+	if (dd->pager_do_scan)
+	   Conf_pagers.scanspeed = dd->pager_scan_speed;
+	else
+	   Conf_pagers.scanspeed = 0;
      }
+   PagersSetMode(dd->pager_mode);
+
    autosave();
 }
 
 static void
-CB_PagerScanSlide(Dialog * d, int val __UNUSED__, void *data __UNUSED__)
+CB_PagerScanSlide(Dialog * d __UNUSED__, int val __UNUSED__, void *data)
 {
+   PagerDlgData       *dd = DLG_DATA_GET(d, PagerDlgData);
+   DItem              *di = (DItem *) data;
    char                s[256];
 
    Esnprintf(s, sizeof(s), "%s %03i %s", _("Pager scanning speed:"),
-	     tmp_pager_scan_speed, _("lines per second"));
-   DialogItemSetText(pager_scan_speed_label, s);
-   DialogDrawItems(d, pager_scan_speed_label, 0, 0, 99999, 99999);
+	     dd->pager_scan_speed, _("lines per second"));
+   DialogItemSetText(di, s);
 }
 
 static void
-_DlgFillPagers(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
+_DlgFillPagers(Dialog * d, DItem * table, void *data __UNUSED__)
 {
-   DItem              *di, *radio;
+   DItem              *di, *radio, *label;
    char                s[256];
+   PagerDlgData       *dd;
 
-   tmp_show_pagers = Conf_pagers.enable;
-   tmp_pager_hiq = Conf_pagers.hiq;
-   tmp_pager_mode = Conf_pagers.mode;
-   tmp_pager_zoom = Conf_pagers.zoom;
-   tmp_pager_title = Conf_pagers.title;
-   tmp_pager_sel_button = Conf_pagers.sel_button;
-   tmp_pager_win_button = Conf_pagers.win_button;
-   tmp_pager_menu_button = Conf_pagers.menu_button;
+   dd = DLG_DATA_SET(d, PagerDlgData);
+   if (!dd)
+      return;
+
+   dd->show_pagers = Conf_pagers.enable;
+   dd->pager_hiq = Conf_pagers.hiq;
+   dd->pager_mode = Conf_pagers.mode;
+   dd->pager_zoom = Conf_pagers.zoom;
+   dd->pager_title = Conf_pagers.title;
+   dd->pager_sel_button = Conf_pagers.sel_button;
+   dd->pager_win_button = Conf_pagers.win_button;
+   dd->pager_menu_button = Conf_pagers.menu_button;
    if (Conf_pagers.scanspeed == 0)
-      tmp_pager_do_scan = 0;
+      dd->pager_do_scan = 0;
    else
-      tmp_pager_do_scan = 1;
-   tmp_pager_scan_speed = Conf_pagers.scanspeed;
+      dd->pager_do_scan = 1;
+   dd->pager_scan_speed = Conf_pagers.scanspeed;
 
    DialogItemTableSetOptions(table, 2, 0, 0, 0);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Enable pager display"));
-   DialogItemCheckButtonSetPtr(di, &tmp_show_pagers);
+   DialogItemCheckButtonSetPtr(di, &dd->show_pagers);
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
    DialogItemSetColSpan(di, 2);
@@ -1871,7 +1864,7 @@ _DlgFillPagers(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 2);
 #endif
-   DialogItemRadioButtonGroupSetValPtr(radio, &tmp_pager_mode);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->pager_mode);
 
    di = DialogAddItem(table, DITEM_SEPARATOR);
    DialogItemSetColSpan(di, 2);
@@ -1879,30 +1872,30 @@ _DlgFillPagers(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Smooth high quality snapshots in snapshot mode"));
-   DialogItemCheckButtonSetPtr(di, &tmp_pager_hiq);
+   DialogItemCheckButtonSetPtr(di, &dd->pager_hiq);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Zoom in on pager windows when mouse is over them"));
-   DialogItemCheckButtonSetPtr(di, &tmp_pager_zoom);
+   DialogItemCheckButtonSetPtr(di, &dd->pager_zoom);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di,
 		     _("Pop up window title when mouse is over the window"));
-   DialogItemCheckButtonSetPtr(di, &tmp_pager_title);
+   DialogItemCheckButtonSetPtr(di, &dd->pager_title);
 
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Continuously scan screen to update pager"));
-   DialogItemCheckButtonSetPtr(di, &tmp_pager_do_scan);
+   DialogItemCheckButtonSetPtr(di, &dd->pager_do_scan);
 
-   di = pager_scan_speed_label = DialogAddItem(table, DITEM_TEXT);
+   di = label = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetColSpan(di, 2);
    DialogItemSetFill(di, 0, 0);
    DialogItemSetAlign(di, 0, 512);
    Esnprintf(s, sizeof(s), "%s %03i %s", _("Pager scanning speed:"),
-	     tmp_pager_scan_speed, _("lines per second"));
+	     dd->pager_scan_speed, _("lines per second"));
    DialogItemSetText(di, s);
 
    di = DialogAddItem(table, DITEM_SLIDER);
@@ -1910,8 +1903,8 @@ _DlgFillPagers(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSliderSetUnits(di, 1);
    DialogItemSliderSetJump(di, 1);
    DialogItemSetColSpan(di, 2);
-   DialogItemSliderSetValPtr(di, &tmp_pager_scan_speed);
-   DialogItemSetCallback(di, CB_PagerScanSlide, 0, NULL);
+   DialogItemSliderSetValPtr(di, &dd->pager_scan_speed);
+   DialogItemSetCallback(di, CB_PagerScanSlide, 0, label);
 
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetColSpan(di, 2);
@@ -1936,7 +1929,7 @@ _DlgFillPagers(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSetText(di, _("Right"));
    DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 3);
-   DialogItemRadioButtonGroupSetValPtr(radio, &tmp_pager_win_button);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->pager_win_button);
 
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetColSpan(di, 2);
@@ -1961,7 +1954,7 @@ _DlgFillPagers(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSetText(di, _("Right"));
    DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 3);
-   DialogItemRadioButtonGroupSetValPtr(radio, &tmp_pager_sel_button);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->pager_sel_button);
 
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetColSpan(di, 2);
@@ -1986,7 +1979,7 @@ _DlgFillPagers(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSetText(di, _("Right"));
    DialogItemRadioButtonSetFirst(di, radio);
    DialogItemRadioButtonGroupSetVal(di, 3);
-   DialogItemRadioButtonGroupSetValPtr(radio, &tmp_pager_menu_button);
+   DialogItemRadioButtonGroupSetValPtr(radio, &dd->pager_menu_button);
 }
 
 const DialogDef     DlgPagers = {
