@@ -21,6 +21,10 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
+#include <fcntl.h>
+#include <signal.h>
+#include <sys/time.h>
+
 #include "E.h"
 #include "dialog.h"
 #include "e16-ecore_hints.h"
@@ -33,9 +37,6 @@
 #include "snaps.h"
 #include "user.h"
 #include "xwin.h"
-#include <fcntl.h>
-#include <signal.h>
-#include <sys/time.h>
 
 #ifdef USE_EXT_INIT_WIN
 static Window       new_init_win_ext = None;
@@ -739,7 +740,7 @@ SessionRunProg(const char *prog, const char *params)
      }
    if (EDebug(EDBUG_TYPE_SESSION))
       Eprintf("SessionRunProg: %s\n", s);
-   system(s);
+   Esystem(s);
 }
 
 void
