@@ -108,7 +108,7 @@ static const char  *const sound_names[N_SOUNDS] = {
    "SOUND_SETTINGS_ALL",
    "SOUND_SETTINGS_AREA",	/* 28 0x10000000 */
    "SOUND_SETTINGS_AUDIO",
-   "SOUND_SETTINGS_AUTORAISE",
+   NULL,
    "SOUND_SETTINGS_BG",
    "SOUND_SETTINGS_COMPOSITE",	/*  0 0x00000001 */
    "SOUND_SETTINGS_DESKTOPS",
