@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2008 Kim Woelders
+ * Copyright (C) 2004-2009 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -74,12 +74,17 @@ _sound_esd_Load(const char *file)
 
 	s->id = esd_sample_cache(sound_fd, format, s->ssd.rate, s->ssd.size,
 				 file);
-	write(sound_fd, s->ssd.data, s->ssd.size);
+	if (write(sound_fd, s->ssd.data, s->ssd.size) != (ssize_t) s->ssd.size)
+	  {
+	     s->id = 0;
+	     goto done;
+	  }
 	confirm = esd_confirm_sample_cache(sound_fd);
 	if (confirm != s->id)
 	   s->id = 0;
      }
 
+ done:
    _EFREE(s->ssd.data);
    if (s->id <= 0)
       _EFREE(s);
