@@ -198,7 +198,8 @@ ThemeExtract(const char *path)
    char                s[FILEPATH_LEN_MAX];
    char                th[FILEPATH_LEN_MAX];
    FILE               *f;
-   unsigned char       buf[320];
+   unsigned char       buf[262];
+   size_t              ret;
    char               *name;
 
    /* its a directory - just use it "as is" */
@@ -215,7 +216,8 @@ ThemeExtract(const char *path)
    f = fopen(path, "r");
    if (!f)
       return NULL;
-   fread(buf, 1, 320, f);
+   ret = fread(buf, 1, sizeof(buf), f);
+   memset(buf + ret, 0, sizeof(buf) - ret);
    fclose(f);
 
    name = fileof(path);
@@ -242,7 +244,7 @@ ThemeExtract(const char *path)
    path = th;
 
    /* exec the untar if tarred */
-   system(s);
+   Esystem(s);
 
  done:
    if (ThemeCheckPath(path))
@@ -325,13 +327,20 @@ ThemeFind(const char *theme)
 void
 ThemePathFind(void)
 {
-   char               *name, *path;
+   char               *name, *path, *s;
 
    /*
     * Conf.theme.name is read from the configuration.
     * Mode.theme.path may be assigned on the command line.
     */
    name = (Mode.theme.path) ? Mode.theme.path : Conf.theme.name;
+   s = (name) ? strchr(name, '.') : NULL;
+   if (s)
+     {
+	*s = 0;
+	Efree(Mode.theme.variant);
+	Mode.theme.variant = Estrdup(s + 1);
+     }
    path = ThemeFind(name);
 
    if (!path && (!name || strcmp(name, "-")))
@@ -351,6 +360,7 @@ ThemePathFind(void)
 
 #if ENABLE_DIALOGS
 #include "dialog.h"
+#include "settings.h"
 /*
  * Configuration dialog
  */
