@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2009 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -85,7 +85,7 @@ WarpFocusWinCreate(void)
    EoSetFade(fw, 1);
    EoSetShadow(fw, 1);
 
-   EventCallbackRegister(EoGetWin(fw), 0, WarpFocusHandleEvent, NULL);
+   EventCallbackRegister(EoGetWin(fw), WarpFocusHandleEvent, NULL);
    ESelectInput(EoGetWin(fw), ButtonReleaseMask);
 
    fw->tc = TextclassFind("WARPFOCUS", 0);
@@ -103,7 +103,7 @@ WarpFocusWinCreate(void)
 static void
 WarpFocusWinDestroy(WarpFocusWin * fw)
 {
-   EventCallbackUnregister(EoGetWin(fw), 0, WarpFocusHandleEvent, NULL);
+   EventCallbackUnregister(EoGetWin(fw), WarpFocusHandleEvent, NULL);
    EoFini(fw);
    Efree(fw);
 }
@@ -477,24 +477,6 @@ WarpFocusHandleEvent(Win win __UNUSED__, XEvent * ev, void *prm __UNUSED__)
  * Warplist module
  */
 
-static void
-WarplistCfgValidate(void)
-{
-   if (Conf.warplist.icon_mode < 0 || Conf.warplist.icon_mode > 3)
-      Conf.warplist.icon_mode = 3;
-}
-
-static void
-WarplistSighan(int sig, void *prm __UNUSED__)
-{
-   switch (sig)
-     {
-     case ESIGNAL_INIT:
-	WarplistCfgValidate();
-	break;
-     }
-}
-
 static const CfgItem WarplistCfgItems[] = {
    CFG_ITEM_BOOL(Conf.warplist, enable, 1),
    CFG_ITEM_BOOL(Conf.warplist, showsticky, 1),
@@ -511,7 +493,7 @@ static const CfgItem WarplistCfgItems[] = {
 extern const EModule ModWarplist;
 const EModule       ModWarplist = {
    "warplist", "warp",
-   WarplistSighan,
+   NULL,
    {0, NULL},
    {N_CFG_ITEMS, WarplistCfgItems}
 };
