@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2008 Kim Woelders
+ * Copyright (C) 2004-2010 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,7 +21,6 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-#define DECLARE_WIN 1
 #include "E.h"
 #include "edebug.h"
 #include "util.h"
@@ -35,12 +34,15 @@
 #include <X11/Xutil.h>
 #include <X11/Xresource.h>
 #include <X11/extensions/shape.h>
-#if USE_COMPOSITE
+#if USE_XRENDER
 #include <X11/extensions/Xrender.h>
 #endif
 #if USE_GLX
 #include "eglx.h"
 #endif
+#if USE_XI2
+#include "X11/extensions/XInput2.h"
+#endif
 
 #define DEBUG_XWIN   0
 #define DEBUG_PIXMAP 0
@@ -59,62 +61,6 @@ static Win          win_last = NULL;
 
 #define WinBgInvalidate(win) if (win->bg_owned > 0) win->bg_owned = -1
 
-#if !EXPOSE_WIN
-Window
-WinGetXwin(const Win win)
-{
-   return win->xwin;
-}
-
-int
-WinGetX(const Win win)
-{
-   return win->x;
-}
-
-int
-WinGetY(const Win win)
-{
-   return win->y;
-}
-
-int
-WinGetW(const Win win)
-{
-   return win->w;
-}
-
-int
-WinGetH(const Win win)
-{
-   return win->h;
-}
-
-int
-WinGetBorderWidth(const Win win)
-{
-   return win->bw;
-}
-
-int
-WinGetDepth(const Win win)
-{
-   return win->depth;
-}
-
-Visual             *
-WinGetVisual(const Win win)
-{
-   return win->visual;
-}
-
-Colormap
-WinGetCmap(const Win win)
-{
-   return win->cmap;
-}
-#endif
-
 void
 EXInit(void)
 {
@@ -248,8 +194,7 @@ EXidSet(Window xwin, Win parent, int x, int y, int w, int h, int depth,
 }
 
 void
-EventCallbackRegister(Win win, int type __UNUSED__, EventCallbackFunc * func,
-		      void *prm)
+EventCallbackRegister(Win win, EventCallbackFunc * func, void *prm)
 {
    EventCallbackItem  *eci;
 
@@ -268,8 +213,7 @@ EventCallbackRegister(Win win, int type __UNUSED__, EventCallbackFunc * func,
 }
 
 void
-EventCallbackUnregister(Win win, int type __UNUSED__,
-			EventCallbackFunc * func, void *prm)
+EventCallbackUnregister(Win win, EventCallbackFunc * func, void *prm)
 {
    EventCallbackList  *ecl;
    EventCallbackItem  *eci;
@@ -716,12 +660,6 @@ EXWindowGetParent(Window xwin)
    else if (pch)
       XFree(pch);
 
-#if 0				/* FIXME - Remove? */
-   win = EXidLookup(xwin);
-   if (win)
-      win->parent = parent;
-#endif
-
    return parent;
 }
 
@@ -1083,9 +1021,60 @@ ESetWindowBackground(Win win, unsigned int col)
 void
 ESelectInput(Win win, unsigned int event_mask)
 {
+#if USE_XI2
+
+#define EVENTS_TO_XI \
+  (/* KeyPressMask | KeyReleaseMask | */ \
+   ButtonPressMask | ButtonReleaseMask | PointerMotionMask)
+
+   win->event_mask = event_mask;
+
+   if (Mode.server.extensions & XEXT_XI)
+     {
+	XIEventMask         em;
+	unsigned char       mask[(XI_LASTEVENT + 8) / 8];
+
+	em.deviceid = XIAllMasterDevices;	/* XIAllDevices; */
+	em.mask_len = sizeof(mask);
+	em.mask = mask;
+	memset(mask, 0, sizeof(mask));
+#if 0
+	if (event_mask & KeyPressMask)
+	   XISetMask(mask, XI_KeyPress);
+	if (event_mask & KeyReleaseMask)
+	   XISetMask(mask, XI_KeyRelease);
+#endif
+	if (event_mask & ButtonPressMask)
+	   XISetMask(mask, XI_ButtonPress);
+	if (event_mask & ButtonReleaseMask)
+	   XISetMask(mask, XI_ButtonRelease);
+	if (event_mask & PointerMotionMask)
+	   XISetMask(mask, XI_Motion);
+	XISelectEvents(disp, win->xwin, &em, 1);
+	event_mask &= ~EVENTS_TO_XI;
+     }
+#endif
+
    XSelectInput(disp, win->xwin, event_mask);
 }
 
+void
+ESelectInputChange(Win win, unsigned int set, unsigned int clear)
+{
+#if USE_XI2
+   win->event_mask |= set;
+   win->event_mask &= ~clear;
+   ESelectInput(win, win->event_mask);
+#else
+   XWindowAttributes   xwa;
+
+   XGetWindowAttributes(disp, win->xwin, &xwa);
+   xwa.your_event_mask |= set;
+   xwa.your_event_mask &= ~clear;
+   XSelectInput(disp, win->xwin, xwa.your_event_mask);
+#endif
+}
+
 void
 EChangeWindowAttributes(Win win, unsigned int mask, XSetWindowAttributes * attr)
 {
@@ -1182,17 +1171,6 @@ EQueryPointer(Win win, int *px, int *py, Window * pchild, unsigned int *pmask)
    return EXQueryPointer(xwin, px, py, pchild, pmask);
 }
 
-void
-ESelectInputChange(Win win, unsigned long set, unsigned long clear)
-{
-   XWindowAttributes   xwa;
-
-   XGetWindowAttributes(disp, win->xwin, &xwa);
-   xwa.your_event_mask |= set;
-   xwa.your_event_mask &= ~clear;
-   XSelectInput(disp, win->xwin, xwa.your_event_mask);
-}
-
 int
 EDrawableCheck(Drawable draw, int grab)
 {
@@ -1556,9 +1534,8 @@ EShapeSetShape(Win win, int x, int y, Win src_win)
    return win->num_rect != 0;
 }
 
-/* Build mask from window shape rects */
-Pixmap
-EWindowGetShapePixmap(Win win)
+static              Pixmap
+_EWindowGetShapePixmap(Win win, unsigned int fg, unsigned int bg)
 {
    Pixmap              mask;
    GC                  gc;
@@ -1571,10 +1548,10 @@ EWindowGetShapePixmap(Win win)
    mask = ECreatePixmap(win, win->w, win->h, 1);
    gc = EXCreateGC(mask, 0, NULL);
 
-   XSetForeground(disp, gc, 0);
+   XSetForeground(disp, gc, bg);
    XFillRectangle(disp, mask, gc, 0, 0, win->w, win->h);
 
-   XSetForeground(disp, gc, 1);
+   XSetForeground(disp, gc, fg);
    rect = win->rects;
    for (i = 0; i < win->num_rect; i++)
       XFillRectangle(disp, mask, gc, rect[i].x, rect[i].y,
@@ -1585,6 +1562,20 @@ EWindowGetShapePixmap(Win win)
    return mask;
 }
 
+/* Build mask from window shape rects */
+Pixmap
+EWindowGetShapePixmap(Win win)
+{
+   return _EWindowGetShapePixmap(win, 1, 0);
+}
+
+/* Build inverted mask from window shape rects */
+Pixmap
+EWindowGetShapePixmapInverted(Win win)
+{
+   return _EWindowGetShapePixmap(win, 0, 1);
+}
+
 Pixmap
 ECreatePixmap(Win win, unsigned int width, unsigned int height,
 	      unsigned int depth)
@@ -1676,10 +1667,11 @@ EXCreateGC(Drawable draw, unsigned long mask, XGCValues * val)
    return XCreateGC(disp, draw, mask, val);
 }
 
-int
+void
 EXFreeGC(GC gc)
 {
-   return XFreeGC(disp, gc);
+   if (gc)
+      XFreeGC(disp, gc);
 }
 
 void
@@ -1688,22 +1680,15 @@ EXSendEvent(Window xwin, long event_mask, XEvent * ev)
    XSendEvent(disp, xwin, False, event_mask, ev);
 }
 
-void
-EAllocColor(Colormap cmap, EColor * pec)
+unsigned int
+EAllocColor(Colormap cmap, unsigned int argb)
 {
    XColor              xc;
 
-   EAllocXColor(cmap, &xc, pec);
-   pec->pixel = xc.pixel;
-}
+   COLOR32_TO_RGB16(argb, xc.red, xc.green, xc.blue);
+   XAllocColor(disp, cmap, &xc);
 
-void
-EAllocXColor(Colormap cmap, XColor * pxc, EColor * pec)
-{
-   pxc->red = pec->red << 8;
-   pxc->green = pec->green << 8;
-   pxc->blue = pec->blue << 8;
-   XAllocColor(disp, cmap, pxc);
+   return xc.pixel;
 }
 
 /*
@@ -1868,7 +1853,7 @@ ESync(unsigned int mask)
  * Visuals
  */
 
-#if USE_COMPOSITE
+#if USE_XRENDER
 
 Visual             *
 EVisualFindARGB(void)
@@ -1948,3 +1933,325 @@ EGetTimestamp(void)
 
    return ev.xproperty.time;
 }
+
+#if USE_COMPOSITE
+
+#include <X11/extensions/Xcomposite.h>
+
+Pixmap
+EWindowGetPixmap(const Win win)
+{
+   XWindowAttributes   xwa;
+
+   if (XGetWindowAttributes(disp, win->xwin, &xwa) == 0 ||
+       xwa.map_state == IsUnmapped)
+      return None;
+
+   return XCompositeNameWindowPixmap(disp, WinGetXwin(win));
+}
+
+/*
+ * Pictures
+ */
+#define _R(x) (((x) >> 16) & 0xff)
+#define _G(x) (((x) >>  8) & 0xff)
+#define _B(x) (((x)      ) & 0xff)
+
+#if 0
+Picture
+EPictureCreate(Window win, int depth, Visual * vis)
+{
+   Picture             pict;
+   XRenderPictFormat  *pictfmt;
+
+   pictfmt = XRenderFindVisualFormat(disp, vis);
+   pict = XRenderCreatePicture(disp, win, pictfmt, 0, 0);
+
+   return pict;
+}
+#endif
+
+Picture
+EPictureCreateSolid(Window xwin, int argb, unsigned int a, unsigned int rgb)
+{
+   Display            *dpy = disp;
+   Pixmap              pmap;
+   Picture             pict;
+   XRenderPictFormat  *pictfmt;
+   XRenderPictureAttributes pa;
+   XRenderColor        c;
+
+   pmap = XCreatePixmap(dpy, xwin, 1, 1, argb ? 32 : 8);
+   pictfmt = XRenderFindStandardFormat(dpy,
+				       argb ? PictStandardARGB32 :
+				       PictStandardA8);
+   pa.repeat = True;
+   pict = XRenderCreatePicture(dpy, pmap, pictfmt, CPRepeat, &pa);
+
+   c.alpha = (unsigned short)(a * 0x101);
+   c.red = (unsigned short)(_R(rgb) * 0x101);
+   c.green = (unsigned short)(_G(rgb) * 0x101);
+   c.blue = (unsigned short)(_B(rgb) * 0x101);
+   XRenderFillRectangle(dpy, PictOpSrc, pict, &c, 0, 0, 1, 1);
+
+   XFreePixmap(dpy, pmap);
+
+   return pict;
+}
+
+Picture
+EPictureCreateBuffer(Win win, int w, int h, Pixmap * ppmap)
+{
+   Picture             pict;
+   Pixmap              pmap;
+   XRenderPictFormat  *pictfmt;
+
+   pmap = XCreatePixmap(disp, WinGetXwin(win), w, h, WinGetDepth(win));
+   pictfmt = XRenderFindVisualFormat(disp, WinGetVisual(win));
+   pict = XRenderCreatePicture(disp, pmap, pictfmt, 0, 0);
+   if (ppmap)
+      *ppmap = pmap;
+   else
+      XFreePixmap(disp, pmap);
+
+   return pict;
+}
+
+void
+EPictureSetClip(Picture pict, XserverRegion clip)
+{
+   XFixesSetPictureClipRegion(disp, pict, 0, 0, clip);
+}
+
+/*
+ * Regions
+ */
+#define DEBUG_REGIONS 0
+
+#if DEBUG_REGIONS
+static int          n_rgn_c = 0;
+static int          n_rgn_d = 0;
+#endif
+
+XserverRegion
+ERegionCreate(void)
+{
+   XserverRegion       rgn;
+
+   rgn = XFixesCreateRegion(disp, NULL, 0);
+
+#if DEBUG_REGIONS
+   n_rgn_c++;
+   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
+	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
+#endif
+   return rgn;
+}
+
+XserverRegion
+ERegionCreateRect(int x, int y, int w, int h)
+{
+   XserverRegion       rgn;
+   XRectangle          rct;
+
+   rct.x = x;
+   rct.y = y;
+   rct.width = w;
+   rct.height = h;
+   rgn = XFixesCreateRegion(disp, &rct, 1);
+
+#if DEBUG_REGIONS
+   n_rgn_c++;
+   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
+	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
+#endif
+   return rgn;
+}
+
+#if USE_DESK_EXPOSE
+XserverRegion
+ERegionCreateFromRects(XRectangle * rectangles, int nrectangles)
+{
+   XserverRegion       rgn;
+
+   rgn = XFixesCreateRegion(disp, rectangles, nrectangles);
+
+#if DEBUG_REGIONS
+   n_rgn_c++;
+   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
+	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
+#endif
+   return rgn;
+}
+#endif
+
+XserverRegion
+ERegionCreateFromWindow(Win win)
+{
+   XserverRegion       rgn;
+
+   rgn =
+      XFixesCreateRegionFromWindow(disp, WinGetXwin(win), WindowRegionBounding);
+
+#if DEBUG_REGIONS
+   n_rgn_c++;
+   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
+	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
+#endif
+   return rgn;
+}
+
+XserverRegion
+ERegionCopy(XserverRegion rgn, XserverRegion src)
+{
+   XFixesCopyRegion(disp, rgn, src);
+   return rgn;
+}
+
+XserverRegion
+ERegionClone(XserverRegion src)
+{
+   XserverRegion       rgn;
+
+   rgn = ERegionCreate();
+   ERegionCopy(rgn, src);
+
+   return rgn;
+}
+
+void
+ERegionDestroy(XserverRegion rgn)
+{
+#if DEBUG_REGIONS
+   n_rgn_d++;
+   Eprintf("%s: %#lx %d %d %d\n", __func__, rgn,
+	   n_rgn_c - n_rgn_d, n_rgn_c, n_rgn_d);
+#endif
+   XFixesDestroyRegion(disp, rgn);
+}
+
+void
+ERegionEmpty(XserverRegion rgn)
+{
+   XFixesSetRegion(disp, rgn, NULL, 0);
+}
+
+void
+ERegionSetRect(XserverRegion rgn, int x, int y, int w, int h)
+{
+   XRectangle          rct;
+
+   rct.x = x;
+   rct.y = y;
+   rct.width = w;
+   rct.height = h;
+   XFixesSetRegion(disp, rgn, &rct, 1);
+}
+
+void
+ERegionTranslate(XserverRegion rgn, int dx, int dy)
+{
+   if (dx == 0 && dy == 0)
+      return;
+   XFixesTranslateRegion(disp, rgn, dx, dy);
+}
+
+void
+ERegionIntersect(XserverRegion dst, XserverRegion src)
+{
+   XFixesIntersectRegion(disp, dst, dst, src);
+}
+
+void
+ERegionUnion(XserverRegion dst, XserverRegion src)
+{
+   XFixesUnionRegion(disp, dst, dst, src);
+}
+
+void
+ERegionSubtract(XserverRegion dst, XserverRegion src)
+{
+   XFixesSubtractRegion(disp, dst, dst, src);
+}
+
+void
+ERegionSubtractOffset(XserverRegion dst, int dx, int dy, XserverRegion src,
+		      XserverRegion tmp)
+{
+   Display            *dpy = disp;
+   XserverRegion       rgn;
+
+   rgn = src;
+   if (dx != 0 || dy != 0)
+     {
+	rgn = ERegionCopy(tmp, src);
+	XFixesTranslateRegion(dpy, rgn, dx, dy);
+     }
+   XFixesSubtractRegion(dpy, dst, dst, rgn);
+}
+
+#if 0				/* Unused */
+void
+ERegionUnionOffset(XserverRegion dst, int dx, int dy, XserverRegion src,
+		   XserverRegion tmp)
+{
+   Display            *dpy = disp;
+   XserverRegion       rgn;
+
+   rgn = src;
+   if (dx != 0 || dy != 0)
+     {
+	rgn = ERegionCopy(tmp, src);
+	XFixesTranslateRegion(dpy, rgn, dx, dy);
+     }
+   XFixesUnionRegion(dpy, dst, dst, rgn);
+}
+#endif
+
+#if 0				/* Unused (for debug) */
+int
+ERegionIsEmpty(XserverRegion rgn)
+{
+   int                 nr;
+   XRectangle         *pr;
+
+   pr = XFixesFetchRegion(disp, rgn, &nr);
+   if (pr)
+      XFree(pr);
+   return nr == 0;
+}
+#endif
+
+void
+ERegionShow(const char *txt, XserverRegion rgn,
+	    void (*prf) (const char *fmt, ...))
+{
+   int                 i, nr;
+   XRectangle         *pr;
+
+   prf = (prf) ? prf : Eprintf;
+
+   if (rgn == None)
+     {
+	prf(" - region: %s %#lx is None\n", txt, rgn);
+	return;
+     }
+
+   pr = XFixesFetchRegion(disp, rgn, &nr);
+   if (!pr || nr <= 0)
+     {
+	prf(" - region: %s %#lx is empty\n", txt, rgn);
+	goto done;
+     }
+
+   prf(" - region: %s %#lx:\n", txt, rgn);
+   for (i = 0; i < nr; i++)
+      prf("%4d: %4d+%4d %4dx%4d\n", i, pr[i].x, pr[i].y, pr[i].width,
+	  pr[i].height);
+
+ done:
+   if (pr)
+      XFree(pr);
+}
+
+#endif /* USE_COMPOSITE */
