@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2008 Kim Woelders
+ * Copyright (C) 2004-2009 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -164,12 +164,6 @@ InZoom(void)
    return 0;
 }
 
-char
-CanZoom(void)
-{
-   return zoom_can;
-}
-
 void
 ZoomInit(void)
 {
@@ -202,7 +196,7 @@ Zoom(EWin * ewin)
 {
    const XF86VidModeModeInfo *mode;
 
-   if (!CanZoom())
+   if (!zoom_can)
       return;
 
    if (!ewin)
@@ -289,12 +283,6 @@ InZoom(void)
    return 0;
 }
 
-char
-CanZoom(void)
-{
-   return 0;
-}
-
 void
 ZoomInit(void)
 {
